function latexfile = mfile2latex(inputfile,options,publishOptions)
   % MFILE2LATEX  Publish LaTeX w/o preamble.

   % The IRIS Toolbox.
   % Copyright 2007-2010 Jaromir Benes.

%**************************************************************************
% Function body.

thisDir = cd();

publishOptions.evalCode = publishOptions.evalCode && isscript(inputfile);

% Save the files that are not m-files with an extra .m extension. This way,
% the xml file contains the file name including the original extension (but
% not the .m extension) and the file name appears correctly in the final
% latex file.
[inputpath,inputtitle,inputext] = fileparts(inputfile);
ismfile = isequal(inputext,'.m');
if ~ismfile
   inputtitle = [inputtitle,inputext];
   inputext = '.m';
   copyfile(inputfile,fullfile(inputpath,[inputtitle,inputext]));
   publishOptions.evalCode = false;
end

if ~isempty(inputpath)
   cd(inputpath);
   inputpath = cd();
else
   inputpath = thisDir;
end

inputfile = [inputtitle,inputext];
% Temporary m-file name to store the original file.
% The original m-file will be modified before publishing as follows:
% * line numbers will be added to the ends of code lines (if
% options.linenumbers is true),
% * block comment marks %{ and %} within the cell headers will be converted
% disabled by replacing them with % { and % }.
% After publishing, the file will be replaced with the original one again.
tmpfile = tempname(inputpath);
copyfile(inputfile,tmpfile);

lines = file2char(inputfile,'cellstrl');
lines = deblank(lines);
nlines = numel(lines);
linecells = {};
linecells{1} = {};
numbercells = {};
numbercells{1} = [];
for i = 1 : nlines
   if ~isempty(regexp(lines{i},'^[ \t]*%%','once'))
      linecells{end+1} = {};
      numbercells{end+1} = [];
   end
   linecells{end}{end+1} = lines{i};
   numbercells{end}(end+1) = i;
end

% Cycle over each cell, insert line numbers and disable block comments in
% the cell headers.
ncells = numel(linecells);
for i = 1 : ncells
   linecells{i} = chkcell_(linecells{i},numbercells{i},options);
end

% Put together the lines of the modified file, and save it under the
% original name.
nl = sprintf('\n');
c = '';
for i = 1 : ncells
   for j = 1 : length(linecells{i})
      c = [c,linecells{i}{j},nl];
   end
end

char2file(c,inputfile);

% Call the built-in |publish| function to compile the latex file.
latexfile = publish(inputfile,publishOptions);

% Replace the modified m-file with the original one.
delete(inputfile);
copyfile(tmpfile,inputfile);
delete(tmpfile);
% Read the latex file to make some manual changes.
c = file2char(latexfile);

% Create line numbers from the %##123 comments in the final LaTeX file if
% requested by the user.
if options.linenumbers
   replacefunc = @createlinenumbers_;
   c = regexprep(c, ...
      '^(.*?)\n%##(\d+)/(\d+)','${replacefunc($1,$2,$3)}','lineanchors');
end

% Remove ugly blank lines in verbatim.
c = regexprep(c, ...
   '\s*\\end\{shadedverbatim\}', ...
   '\n\\end{shadedverbatim}');

% Remove the files' contents (itemised at the beginning of each file) if
% requested by the user.
if ~options.filecontents
   c = regexprep(c, ...
      '[ ]*% Start of Contents.*?[ ]*% End of Contents\n','', ...
      'once');
end

% Save the modified latex code.
char2file(c,latexfile);

% Find EPS figures and convert them to PDF.
latex.findeps2pdf(latexfile);

if ~ismfile
   delete(fullfile(inputpath,[inputtitle,inputext]));
end

cd(thisDir);

end

%**************************************************************************
% Subfunction chkcell_().
function lines = chkcell_(lines,numbers,options)
   nlines = numel(lines);
   emptyline = false([1,nlines]);
   commentline = false([1,nlines]);
   % Uninterrupted series of comment lines at the beginning.
   introline = false([1,nlines]);
   intro = true;
   for i = 1 : nlines
      emptyline(i) = isempty(regexp(lines{i},'[^ \t]','once'));
      commentline(i) = ~isempty(regexp(lines{i},'^[ \t]*%','once'));      
      introline(i) = intro && commentline(i);
      intro = introline(i);
      if introline(i) 
         % Remove spaces before % to enable publishing indented cell
         % headers.
         lines{i} = regexprep(lines{i},'^\s*','');
         % Replace %{ or %} with % { or % } to enable publishing
         % block-commented cell headers.
         lines{i} = regexprep(lines{i},'\s*%[\{\}]\s*','%','once');
      end
   end
   if ~options.linenumbers
      return
   end
   commandline = ~emptyline & ~introline;
   if ~any(commandline)
      return
   end
   % Create comments for line numbers below each line, e.g. \n%##123.
   % With a new line inserted, this works fine also for block comment
   % starts and ends.
   nl = sprintf('\n');
   first = find(commandline,1,'first');
   last = find(commandline,1,'last');
   for i = first : last
      lines{i} = [lines{i},nl, ...
         '%##',sprintf('%g/%g',numbers(i),length(lines{i}))];
   end
end
% End of subfunction chkcell_().

%**************************************************************************
% Subfunction createlinenumbers_().
function line = createlinenumbers_(line,number,len)
   len = sscanf(len,'%g');
   codeline = line(end-len+1:end);
   line(end-len+1:end) = '';
   line = [line,sprintf('%3s: ',number),codeline];
end
% End of subfunction cratelinenumbers_().
