function [outputfile,latexdir] = publish(inputfiles,outputfile,varargin)
   % PUBLISH  Publish m-files and models as PDF booklet.
   %
   % Syntax:
   % [outputfile,latexdir] = publish(inputfiles,outputfile,...)

   % The IRIS Toolbox.
   % Copyright 2007-2009 Jaromir Benes.

%TODO:
% * use pdftex to create hyperlinks
% * fix page breaks between chapters
% * reduce font size for subsections, create "introduction" subsection at
% the beginning

options = passvalopt('latex.publish',varargin{:});

if ischar(inputfiles)
   inputfiles = charlist2cellstr(inputfiles);
end

if strcmp(options.rerun,'auto')
   % Re-run LaTeX if table of contents is requested.
   options.rerun = double(options.toc);
end

if options.toc && ~options.numbered
   options.toc = false;
   warning('iris:latex', ...
      'Cannot create table of contents without numbering the sections.');
end

if strcmp(options.preamble,'config')
   options.preamble = irisget('publishpreamble');
end

%**************************************************************************
% Function body.

config = irisget();
thisDir = cd();
workingDir = tempname(thisDir);

latexPkgDir = fullfile(irisget('irisroot'),'+latex');
stylesheetFile = fullfile(latexPkgDir,'mxdom2latexnopreamble.xsl');
templateFile = fullfile(latexPkgDir,'publishTemplate.tex');
masterFile = [tempname(workingDir),'.tex'];

% Options struct for built-in publish.
publishOptions = struct();

% Fixed options.
publishOptions.format = 'latex';
publishOptions.outputDir = workingDir;
publishOptions.stylesheet = stylesheetFile;

% Customisable options.
publishOptions.useNewFigure = options.usenewfigure;
% evalCode == true is only used with script m-files.
publishOptions.evalCode = options.evalcode;
publishOptions.showCode = options.showcode;

% Publish latex files w/o preamble.
latexfiles = cell(size(inputfiles));
ninputfiles= numel(inputfiles);
for i = 1 : ninputfiles
   [ans,ans,extension] = fileparts(inputfiles{i});
   if isempty(extension)
      inputfiles{i} = [inputfiles{i},'.m'];
   end
   latexfiles{i} = latex.mfile2latex(inputfiles{i}, ...
      options,publishOptions);
end

% Create master document.
nl = sprintf('\n');
doc = '';
template = file2char(templateFile);
template = latex.docsubs(template,options);
% Add \input for each latex file.
for i = 1 : ninputfiles
   [ans,tmptitle] = fileparts(latexfiles{i});
   inputfile = latex.stringsubs(inputfiles{i});
   doc = [doc,nl,'\include{',tmptitle,'}'];
end
doc = strrep(template,'!Document',doc);

% Save master document in the temporary working directory.
char2file(doc,masterFile);

% Copy defaultHeadings.sty to the temporary working directory.
preambleFile = fullfile(config.irisroot,'+latex','defaultHeadings.sty');
copyfile(preambleFile,workingDir);

% Compile PDF from latex.
pdffile = latex.compilepdf(masterFile, ...
   'rerun',options.rerun,'display',options.display);

% Move PDF to the original m-file location. Catch the possible error (when
% the latex/PDF has not been correctly created and does not exist) so that
% we can subsequently delete the temporary files.
try
  movefile(pdffile,outputfile);
catch Error
   message = regexprep(Error.message,'\r\n|\r|\n',' ');
   message = strtrim(message);
   if ~isempty(message) && message(end) ~= '.'
      message(end+1) = '.';
   end
   warning('iris:latex', ...
      ['\n*** Error creating the LaTeX file or PDF. No output produced.', ...
      '\n*** Matlab says ''%s'''],message);
end

% Delete directory with latex and EPS files unless the user requests to
% keep the files.
if options.deletelatex
   rmdir(workingDir,'s');
   latexdir = '';
else
   latexdir = workingDir;
end

end
% End of primary function.