classdef figureobj < report.genericobj

   properties
      handle = [];
   end
   
   properties (Dependent = true)
      subplot
      visible
   end
   
   methods

      % Constructor. Define accepted children and default options.
      function this = figureobj(varargin)
         this = this@report.genericobj(varargin{:});
         this.accept = {'graph'};
         this.default = {...
            'separator','\medskip\par',@ischar,true, ...
            'scale',0.85,@isnumericscalar,true, ...
            'subplot','auto',@(x) (isnumeric(x) && numel(x) == 2) || strcmpi(x,'auto'),true, ...
            'visible',false,@islogical,true, ...
            'saveas','',@ischar,false, ...
            'style',struct(),@isstruct,true, ...
         };
      end
      
      % Process class-specific input arguments.
      function [this,varargin] = specargin(this,varargin)
         % Capture the figure handle in case this figure object will have
         % no children.
         if isempty(varargin)
            this.handle = get(0,'CurrentFigure');
         elseif isnumericscalar(varargin{1}) ...
               && ishandle(varargin{1}) ...
               && strcmp(get(varargin{1},'type'),'figure')
            this.handle = varargin{1};
            varargin(1) = [];
         end
      end

      % Produce LaTeX code for this figure.
      function [c,helperfiles] = latexcode(this,parent)
         if isempty(this.children)
            % Grab the figure whose handle was captured at the time the
            % figureobj constructor was called.
            if isempty(this.handle)
               warning('iris:report', ...
                  '\n*** No output produced for figure object ''%s'' because it has no children and no Matlab figure exists to be grabbed.', ...
                  this.caption);
               c = '';
               helperfiles = {};
               return
            end
            figure(this.handle);
         else
            % Plot all series.
            this.handle = figure('visible',this.visible);
            % Apply styles to this figure.
            qstyle(this.options.style,this.handle,'cascade',false,'warning',false);
            % Use try-catch to ensure the invisible figure is closed if the
            % execution is disrupted by an error.
            %try
               [ax,ln] = plot(this);
            %catch Error
            %   warning('iris:report', ...
            %      '\n*** Error creating figure.\nMatlab says: %s', ...
            %      Error.message);
            %end
         end
         % Use try-catch to ensure the invisible figure is closed if the
         % execution is disrupted by an error.
         try
            % Create PDF for figure handle.       
            [pdfile,helperfiles] = compilepdf(this);
         catch Error
            close(this.handle);
            warning('iris:report', ...
               '\n*** Error creating figure.\nMatlab says: %s', ...
               Error.message);
            c = '';
            helperfiles = {};
            return
         end
         close(this.handle);
         c = '\includegraphics[scale=!Scale,angle=-90]{!PDFileTitle.pdf}';
         c = strrep(c,'!Scale',sprintf('%g',this.options.scale));
         [pdfpath,pdftitle] = fileparts(pdfile);
         c = strrep(c,'!PDFileTitle',pdftitle);
         % If figure has caption, wrap it in a tabular.
         if ~isempty(this.caption)
            nl = sprintf('\n');
            c = ['\begin{tabular}{c}',nl, ...
               '\large\bfseries ',latex.stringsubs(this.caption),' \\',nl, ...
               c,nl,'\end{tabular}'];
         end
      end
      
      % Open figure and plot all children.
      function [ax,ln] = plot(this)
         % Open new figure.
         ax = nan([1,this.nChildren]);
         ln = cell([1,this.nChildren]);
         % Generate child graphs.
         for i = 1 : this.nChildren
            ax(i) = subplot(this.subplot{1:2},i);
            ln{i} = plot(this.children{i},ax(i));
         end
      end
      
      function [pdfile,helperfiles] = compilepdf(this)
      % COMPILEPDF  Generate PDF for a figure.
         % Generate unique name for EPS of use user-supplied name.
         if isempty(this.options.saveas)
            epsfile = [tempname(cd()),'.eps'];
            addtohelperfiles = true;
         else
            [tmppath,tmptitle] = fileparts(this.options.saveas);
            epsfile = fullfile(tmppath,[tmptitle,'.eps']);
            addtohelperfiles = false;
         end   
         [epspath,epstitle] = fileparts(epsfile);
         % Print EPS.
         orient(this.handle,'landscape');
         print(this.handle,'-depsc',epsfile);
         pdfile = fullfile(epspath,[epstitle,'.pdf']);
         if addtohelperfiles
            helperfiles = {epsfile,pdfile};
         else
            helperfiles = {};
         end
         % Convert EPS to PDF.
         % Use try-catch to make sure the helper files are deleted.
         latex.epstopdf(epsfile);
      end
      
      % Compute subdivision of the figure for graphs.
      function sub = get.subplot(this)
         if strcmp(this.options.subplot,'auto')
            candidate = ceil(sqrt(this.nChildren));
            if candidate*(candidate-1) >= this.nChildren
               sub = {candidate,candidate-1};
            else
               sub = {candidate,candidate};
            end
         else
            sub = {this.options.subplot(1),this.options.subplot(2)};
         end
      end
      
      function visible = get.visible(this)
         if this.options.visible
            visible = 'on';
         else
            visible = 'off';
         end
      end

   end

end