function C = lyapunov(T,Sigma,beta)
% LYAPUNOV  Solve Lyapunov equation.
%    C = beta*T*C*T' + Sigma,
% where T is quasi-triangular.

% The IRIS Toolbox 2009/08/01.
% Copyright 2007-2009 Jaromir Benes.

%********************************************************************
% Function body..

% Discounted equation.
if nargin > 2 && beta < 1
   T = sqrt(beta)*T;
end

C = zeros(size(T));
i = size(T,1);
Tt = T';
while i >= 1
   if i == 1 || T(i,i-1) == 0
      % 1x1 block with a real eigenvalue.
      C(i,i+1:end) = C(i+1:end,i)';
      c = (Sigma(i,1:i) + T(i,i)*C(i,i+1:end)*Tt(i+1:end,1:i) + ...
         T(i,i+1:end)*C(i+1:end,:)*Tt(:,1:i)) / (eye(i) - T(i,i)*Tt(1:i,1:i));
      C(i,1:i) = c;
      i = i - 1;
   else
      % 2x2 block corresponding to a pair of complex eigenvalues.
      C(i-1:i,i+1:end) = C(i+1:end,i-1:i)';
      X = T(i-1:i,i-1:i)*C(i-1:i,i+1:end)*Tt(i+1:end,1:i) + ...
         T(i-1:i,i+1:end)*C(i+1:end,:)*Tt(:,1:i) + Sigma(i-1:i,1:i);
      % Solve
      %    c = T(i-1:i,i-1:i)*c*Tt(1:i,1:i) + X
      % Transpose the equation first
      %    c' = Tt'*c'*T' + X'
      % so that the below kronecker product becomes faster to evaluate,
      % then vectorise
      %    vec(c') = kron(T,Tt')*vec(c') + vec(X').
      Xt = X';
      U = Tt(1:i,1:i)';
      k = [T(i-1,i-1)*U,T(i-1,i)*U;T(i,i-1)*U,T(i,i)*U];
      ct = (eye(2*i) - k) \ Xt(:);
      C(i-1:i,1:i) = reshape(ct,[i,2])';
      i = i - 2;
   end
end

end
% End of primary function.