%% Compare second-moment properties in model vs data.

%% Clear workspace.

clear('all');
close('all');
home();

%% Load the model, data, and dates.

load('read_model.mat','m');
load('read_data.mat','Gap','starthist','endhist');
D = Gap;

%% Estimate VAR and BVAR.

p = 2;
stdvec = nanstd([D.r_,D.dp_,log(D.Y_),log(D.Q_)]);
X = BVAR.litterman(0,stdvec*sqrt(50),0,[4,p,0]);

[w,wdata] = rvar([D.r_,D.dp_,log(D.Y_),log(D.Q_)],starthist:endhist,'order',p);
[bw,bwdata] = rvar([D.r_,D.dp_,log(D.Y_),log(D.Q_)],starthist:endhist,'order',p,'bvar',X);

home();
A = get(w,'A*');
bA = get(bw,'A*');
disp('First-order transition matrix for unrestricted VAR');
disp(A(:,:,1));
disp('First-order transition matrix for BVAR');
disp(bA(:,:,1));

%% Resample from estimated VAR.
%
% Use wild bootstrap, which is robust to heteroscedasticity of residuals.
%

nsim = 500;
fprintf('Resampling %g times from estimated VAR.\n',nsim);
Y = resample(w,wdata,nsim,'wild',true);
W = rvar(Y,Inf,'order',p);

%% Compute ACF and XSF for VAR, resampled VARs, and model.
%

freq = 0 : 0.05 : pi;
disp('Computing ACF and XSF for VAR, bootstrapped VARs, and model.');

[Cw,Rw] = acf(w,'order',1);
[Pw,Sw] = xsf(w,freq);

[CW,RW] = acf(W,'order',1);

[Cm,Rm,list] = acf(m,'order',1);
Cm0 = select(Cm,list,{'r_','dp_','log(Y_)','log(Q_)'});
Rm0 = select(Rm,list,{'r_','dp_','log(Y_)','log(Q_)'});
[Pm,Sm,list] = xsf(m,freq);
Sm0 = select(Sm,list,{'r_','dp_','log(Y_)','log(Q_)'});

%% Compare selected second moments.

figure();
[n,x] = hist(vec(RW(2,3,1,:)),15);
bar(x,n,'barWidth',1,'faceColor',0.9*[1,1,1]);
hold on;
maxn = max(n)*1.1;
stem(Rw(2,3,1),maxn,'color','blue','lineWidth',2);
stem(Rm(2,3,1),maxn,'color','red','lineWidth',2);
grid on;
legend('VAR: bootstrap','VAR: point estimate','Model: Asymptotic');
title('Cross-correlation: Inflation vs Output');

%% Compare spectral densitiies.

figure();

plot(freq,vec(real(Sw(2,2,:))),'color','blue');
hold on;
plot(freq,vec(real(Sm(2,2,:))),'color','red');
set(gca(),'xlim',[0,pi],'xLimMode','manual');
grid on;
legend('VAR: point estimate','Model: Asymptotic');
title('Spectral density: Inflation');
