%% Estimate parameters

%% Clear workspace and load model, data, and dates

clear('all');
close('all');
load('read_model.mat','m');
load('read_data.mat','Gap','starthist','endhist');
D = Gap;
histrange = starthist : endhist;

% Switch off productivity shocks throughout estimation.
m.std_ea = 0;

%% Estimate some of the structural parameters and std devs
% Available methods:
%
% * Maximum likelihood (with parameter constraints)
% * Minimum prediction errors
% * Maximum regularised likelihood
% * Posterior mode
%

%% Set up database with parameters to be estimated
% The input database is set up as follows:
%
%  E.parameter_name = [starting_value, lower_bound, upper_bound].
%
% If the starting value is NaN, then the currently assigned parameter value
% (from the model object) is used.

E = struct();

E.chi = [0.8,0.5,0.95];
E.xiw = [NaN,0.001,0.1];
E.xip = [NaN,0.001,0.1];
E.rhor = [NaN,0.10,0.95];
E.mup = [NaN,1.5,Inf];
E.mug = [NaN,0,Inf];

E.std_ec = [NaN,0.001,0.1];
E.std_ew = [NaN,0.001,0.1];
E.std_ep = [NaN,0.001,0.1];
E.std_er = [NaN,0.001,0.1];

disp(E);

%% Maximum likelihood with parameter bands
% The main output arguments are the following (these remain the same
% whatever the set-up of the estimation):
%
% * |Pstar| A database with point estimates.
% * |obj| A function handle that gives access to the objective function used inside the
% |estimate| command. You can call |obj1()| to get the value of the
% objective function at the optimum. Or you can e.g. use it to investigate the
% behaviour of the likelihood function in the neighbourhood of the optimum,
% using the |neighbourhood| function (see below).
% * |grad| Gradient of the objective function.
% * |hess| A cell array where hess{1} is the hessian of the objective
% function, and hess{2} is the contribution to the hessian of your penalty
% function and/or prior distributions.
% * |mstar| The model object solved with the new estimated parameters.
% * |var| Estimate of the common variance factor (if you run it with
% 'relative' set to true = default). All the std dev of all shocks are
% multiplied automatically by the square root of this number.

[Pstar1,obj1,grad1,hess1,mstar1,var1,F1,pe1,delta1] = ...
   estimate(m,D,starthist:endhist,E,'deviation',true,'tolx',1e-12);

disp('Value of the likelihood function at the optimum')
disp(obj1()); % You must use the empty round brackets to get the value.
disp('Maximum likelihood estimates');
disp(Pstar1);

%% Behaviour of the likelihood function in the neighbourhood of the optimum
% The |neighbourhood| function evaluates the objective function
% (accessible thanks to the |obj| handle) for a few values of each
% parameter around the optimum. In this case, each parameter estimate will
% investigate within the range of +/- 5 % of the estimated value (= 0.95 :
% 0.01 : 1.05 times the value of the estimate).

[d,h] = neighbourhood(m,obj1,0.95:0.01:1.05,'plot',true);
ax = findobj(gcf(),'type','axes');
linkaxes(ax,'y');

%% Examine the information matrix at the optimum
% Compare the expected Hessian returned by the Optimization Tbx with the
% estimate of the information matrix assuming asymptotically i.i.d.
% prediction errors. The latter information matrix estimate is computed
% using the diffloglik function again. Notice that the habit parameter is
% very tightly identified.

% List of estimated parameters.
plist = fieldnames(E);

[mloglik1,score1,info1] = ...
   diffloglik(mstar1,D,starthist:endhist,plist,'deviation',true);

disp('Information matrix based on the assumption of assymptotically i.i.d. prediction errors');
disp(info1);
disp('Hessian from the Optimization Tbx');
disp(hess1{1});
disp('Compare diagonal elements');
disp([char(plist),num2str([diag(info1),diag(hess1{1})],'%.1f')]);

%% Minimum predicition errors (equally weighted)

[Pstar2,obj2,grad2,hess2,mstar2,var2,F2,pe2] = ...
   estimate(m,D,starthist:endhist,E, ...
   'deviation',true,'objective','prederr');

disp('Minimum prediction error estimates');
disp(Pstar2);

%% Minimum weigthed prediction errors
% Use a diagonal weighting matrix based on the one-step-ahed prediction
% errors from an estimated VAR.

w = estimate(VAR(),D,get(m,'ylist'),starthist:endhist, ...
   'order',2,'constant',false);
Omg = get(w,'Omega');
W = diag(1./diag(Omg));
% Re-scale the weighting matrix a little bit (does not affect the
% estimation properties, only helps numerically).
W = W / 10000; 

[Pstar3,obj3,grad3,hess3,mstar3,var3,F3,pe3] = ...
   estimate(m,D,starthist:endhist,E,'deviation',true,'objective','prederr','weighting',W);

disp('Minimum weighted prediction error estimates');
disp(Pstar3);

%% Maximum penalised (regularised) likelihood
%
% Impose a penalty on deviations of the inividual parameter estimates from
% the "priors". The penalty function is specified using two numbers: the
% weight and the prior. The whole entry in the estimation input database
% then looks like this (note the curly braces):
%
%  E.parameter_name = { starting_value , lower , upper , [weight,prior] }
%
% On top of that, all the penalty functions are mutliplied by the overall
% 'penalty' option. The penalty term in the objective function is therefore
%
% $$ p \cdot \sum_i w_i (\theta_i - \Bar \theta_i)^2 $$
%
% where $p$ is the overall penalty weight specified through the 'penalty'
% option, $w_i$ is the parameter-specific penalty weight, and $\Bar
% \theta_i$ is the "prior".
%
% Note that we switch off priors on std deviations by omitting the penalty
% in the database.
%

E = struct();

E.chi = {0.5,0.5,0.95,[1,0.8]};
E.xiw = {NaN,0.001,0.1,[1,0.05]};
E.xip = {NaN,0.001,0.1,[1,0.05]};
E.rhor = {NaN,0.10,0.95,[1,0.50]};
E.mup = {NaN,1.5,Inf,[1,4]};
E.mug = {NaN,0,Inf,[1,0.5]};

E.std_ec = {m.std_ec,0.001,0.1};
E.std_ew = {m.std_ew,0.001,0.1};
E.std_ep = {m.std_ep,0.001,0.1};
E.std_er = {m.std_er,0.001,0.1};

[Pstar4,obj4,grad4,hess4,mstar4,var4,F4,pe4] = ...
   estimate(m,D,starthist:endhist,E,'deviation',true,'penalty',500);

disp('Maximum regularised likelihood estimates');
disp(Pstar4);

%% Contributions to information matrix
% 
% Examine the contributions to the information matrix (i.e. to
% identification): * the data likelihood, * the penalty function.
%

plist = fieldnames(E);

disp('Total information matrix');
disp(hess4{1});

disp('Contribution of the penalty function');
disp(hess4{2});

disp('Contribution of the data likelihood');
disp(hess4{1}-hess4{2});

disp('Diagonal elements of the Hessian');
disp('Total hessian, penalty function contribution, data likelihood contribution');
disp([char(plist),num2str([diag(hess4{1}),diag(hess4{2}),diag(hess4{1})-diag(hess4{2})],'%.1f')]);


%% Posterior mode
%
% We add a prior on the habit parameter, |chi|, a normal distribution with
% mean 0.5 and std dev 0.05. Of course, you you specifiy priors for other
% parameters, too. If you wish to specify a prior distribution function,
% you need to do the following three steps:
%
% # Build a function that takes one input argument, and returns one output
% arguments (so the distributions characteristics must be incorporated
% implicitly within that function). The best way is to write a so-called
% "anonymous function" for each distribution. If you are not familiar with
% anonymous functions, please read Matlab's help or manual. It's very easy!
%
% # Change the corresponding array in the dababase E to a cell  array (i.e.
% use curly braces instead of the square ones).
%
% # Include the function handle to your distribution function as the 4-th
% element in that cell array.
%
% Note also that the habit parameter was quite tightly estimated by maximum
% likelihood, so we specify a very narrow distribution (truncated normal in
% this case) to demonstrate the method and move the posterior mode
% considerably away from the ML estimate.
%

E = struct();

E.chi = {m.chi,0.30,0.95, @(x) normpdf(x,0.5,0.05) };
E.xiw = [m.xiw,0.001,0.1];
E.xip = [m.xip,0.001,0.1];
E.rhor = [m.rhor,0.10,0.95];
E.mup = [m.mup,1.1,5];
E.mug = [m.mug,0,3];

E.std_ec = [m.std_ec,0.001,0.1];
E.std_ew = [m.std_ew,0.001,0.1];
E.std_ep = [m.std_ep,0.001,0.1];
E.std_er = [m.std_er,0.001,0.1];

[Pstar5,obj5,grad5,hess5,mstar5,var5,F5,pe5] = ...
   estimate(m,D,starthist:endhist,E,'deviation',true);

disp('Posterior mode');
disp(Pstar5);
