%% Simulate and examine Fisher information matrix.

%% Clear workspace and load model

clear('all');
close('all');
home();
load('read_model.mat');

%% Calculate Fisher information matrix
%
% This is a useful piece of information telling you how much information
% you can expect to find in the data (under the assumption that the model
% with its current parameters is the true DGP).
%
% We compute the Fisher information matrix using two approaches:
% * a simulation approach in the time domain.
% * an analytical approach in the frequency domain.
%
% The results should be, of course, asymptotically equivalent. The
% differencies we observe in this exercise are due to
% * sampling errors,
% * the fact that the frequency domain approach assumes that the model
% variables follow a circular process (which is not true for samples of
% finite lenthgs).
%

% List of parameters for which we want to compute the Fisher matrix.
plist = {'chi','mup','mug','xip'}; 

% Set to a larger number in practice.
nsim = 50;

fprintf('Resampling %g times from calibrated model.\n',nsim);

% Simulate a total of nsim artificial data, length 40 periods.
Y = resample(m,[],1:40,nsim,'deviation',true);

disp('Computing Hessians for each draw, and the average of them.');
[mloglik,s,F1] = diffloglik(m,Y,1:40,plist,'deviation',true,'relative',false);
F1 = mean(F1,3);

disp('Computing information matrix in frequency domain.');
[F2,F2i,d] = fisher(m,40,plist,'deviation',true);

disp('Compare time-domain and frequency domain Fisher infomartion matrices.');
disp('Time domain');
disp(F1);
disp('Frequency domain');
disp(F2);

disp('Rank of Fisher information matrix');
disp('Time domain');
disp(rank(F1));
disp('Frequency domain');
disp(rank(F2));

%% Compute and report singular value decomposition

[u1,s1] = svd(F1);
[u2,s2] = svd(F2);

s1 = diag(s1);
s2 = diag(s2);
s1 = s1 / s1(1);
s2 = s2 / s2(1);

disp('Singular values (normalised and ordered)');
disp('Time domain');
disp(s1.');
disp('Frequency domain');
disp(s2.');

disp('Combinations of parameters ordered by degree of identification.');
disp('(Best identified first)');
disp('Time domain');

plist = regexprep(plist,'.*','$0: ');
disp([char(plist),num2str(u1,'   %.2g')]);
disp('Frequency domain');
disp([char(plist),num2str(u2,'   %.2g')]);
