%% How to get access to information inside model objects.

%% Clear workspace and load model.

clear('all');
close('all');
home();
load('read_model.mat','m');

%% Get names of variables and parameters

disp('List of transition variables');
get(m,'xList'),

disp('List of measurement variables');
get(m,'yList'),

disp('List of shocks');
get(m,'eList'),

disp('List of parameters');
get(m,'pList'),

%% Get a database with comments/labels attached to variables and parameters

disp('Database with comments attached to variables and paramters');
get(m,'comments'),

disp('List of comments attached to transition variables');
get(m,'xComments'),

%% Get a database with currently assigned parameter values.

disp('Database with currently assigned paramters');
get(m,'parameters'),

disp('Database with currently assigned std deviations');
get(m,'std'),

%% Use different ways to get and assign/change parameters.
% There are multiple equivalent ways how to view and assign parameters.
% In the examples below, we display the parameter 'gamma', and change
% the values for two std deviations, 'std_ep' and 'std_ew'.

P = get(m,'parameters');
P.gamma,

m.gamma,

s = struct();
s.std_ep = 0.02;
s.std_ew = 0.02;
m = assign(m,s);

m = assign(m,'std_ep',0.02,'std_ew',0.02);

m.std_ep = 0.02;
m.std_ew = 0.02;

%% Check stationarity.

%%%
% |true| (displayed as 1 on the screen) or |false| (displayed as 0)
disp('Is the model stationary?');
isstationary(m),

%%%
% Database with |true| or |false| indicating whether the respective variable is
% stationary.
disp('Is the variable stationary?');
get(m,'stationary'),

disp('List of stationary variables');
get(m,'stationaryList'),

disp('List of non-stationary variables');
get(m,'nonstationaryList'),

%% Get currently assigned steady state
% Steady state is described by complex numbers:
%
% * real part = steady-state levels
% * imaginary part = steady-state growth
%

sstate_database = get(m,'sstate')

% steady state levels
sslevel_database = get(m,'sstateLevel')

% steady-state growth rates
% interpretation differs for linearised and log-linearised variables:
% linearised vars: x(t) - x(t-1)
% log-linearised vars: x(t) / x(t-1)
ssgrowth_database = get(m,'sstateGrowth')

% database with true (displayed as 1) or false (0) for each variable
% depending on whether it is log-linear
is_variable_loglinear = get(m,'log')

%% Use different ways to get and assign/change steady state
% Getting and assigning/changing steady state is identical to getting and assigning parameters.
% In the example below, we display the steady state for Y.
% Then we change the steady state for Y and C, and show that the model's
% equations do not hold for the newly changed steady state.
% Finally, we set the two values back.

sstate_database = get(m,'sstatelevel');
sstate_database.Y

m.Y

s = struct();
s.Y = 2;
s.C = 2;
m = assign(m,s);

m = assign(m,'Y',2,'C',2);

m.Y = 2;
m.C = 2;

chksstate(m)

m.Y = sstate_database.Y;
m.C = sstate_database.C;

%% Eigenvalues

disp('Eigenvalues');

% eigenvalues
all_roots = get(m,'roots');
stable_roots = get(m,'stableRoots')
unit_roots = get(m,'unitRoots')
unstable_roots = get(m,'unstableRoots')

% plot all stable roots and the dominant unstable root

figure();
ploteig(stable_roots);

stable_roots = sort(stable_roots);
[ans,index] = sort(abs(unstable_roots));
unstable_roots = unstable_roots(index);

disp('Dominant (largest) stable root');
disp(stable_roots(end));
disp('Dominant (smallest) unstable root and its inverse');
disp([unstable_roots(1),1./unstable_roots(1)]);


%% Model solution (state-space form)
%
% [xu;a] = T*a(t-1) + K + R1*e + R2*e(t+1) + ...
% y = Z*a + D + H*e
% xp = U*a
% cov(e) = Omega
%
% |xu| non-predetermined transition variables
% |xp| predetermined transition variable
% |e| residuals
% |y| measurement variables
% |a| vector of transformed pre-determined variables making T
% quasi-triangular (computational efficiency)

[T,R,K,Z,H,D,U,Omega,list] = sspace(m);

disp('State-space matrices');
disp('Transition matrix T:');
disp(T);
disp('Size of T:');
size(T)
disp('Size of R:');
size(R)
disp('Size of K:');
size(K)
disp('Size of Z:');
size(Z)

disp('Vectors of variables in row and columns of state-space matrices')
disp('Vector y:');
disp(list{1});
disp('Vector [xu;xp]:');
disp(list{2});
disp('Vector e:');
disp(list{3});

m = expand(m,2);
[T,R,K,Z,H,D,U,Omega,list] = sspace(m);
disp('Size of R for expanded solution:')
size(R)

% List of initial conditions: not all xp(-1) are needed
disp('Initial conditions needed');
icondlist = get(m,'initcond');
display(icondlist);


