%% Read and transform data

%% Clear workspace

clear('all');
close('all');
home();

%% Load the raw database

Raw = dbload('US_data.csv');
disp(Raw);

%% Convert monthly series to quarterly periodicity
% Default method: averaging.

Raw.TB3MS = convert(Raw.TB3MS,'quarterly');
Raw.PCE = convert(Raw.PCE,'quarterly');
Raw.PCEC96 = convert(Raw.PCEC96,'quarterly');
Raw.AHETPI = convert(Raw.AHETPI,'quarterly');

% Define PCE deflator.
Raw.PCEP = Raw.PCE/Raw.PCEC96;

%% Create model-consistent database
% Create a new database with model-consistent measurement variable names.
% HP filter the data and create databases with trends and gaps.
% Impose flat trend conditions at the beginning and end of the sample for growth rates.
%
% The 'log' option set to true (used with Y_) means that the HP filter  is run on log(Y_)
% and the result is then de-logarithmised. This is because Y_ is a log-linear
% variable, so its gap is assumed to be
%
% $$ \hat x_t = x_t / \bar x $$
%

D = struct();
D.dp_ = Raw.PCEP/Raw.PCEP{-1} - 1;
D.r_ = Raw.TB3MS/400;
D.Y_ = Raw.PCEC96;
D.Q_ = Raw.AHETPI / Raw.PCEP;

starthist = get(D.dp_,'start');
endhist = get(D.dp_,'end');
nfcst = 12;

Tnd = struct();
Gap = struct();

[Tnd.Y_,Gap.Y_] = hpf(D.Y_,starthist:endhist+nfcst,'log',true,'lambda',1600);
[Tnd.Q_,Gap.Q_] = hpf(D.Q_,starthist:endhist+nfcst,'log',true,'lambda',1600);

tune = tseries();
tune(starthist) = 0;
tune(endhist) = 0;

[Tnd.dp_,Gap.dp_] = hpf(D.dp_,starthist:endhist+nfcst,'growth',tune);
[Tnd.r_,Gap.r_] = hpf(D.r_,starthist:endhist+nfcst,'growth',tune);

%% Plot data and HP trends

figure('position',get(0,'screenSize'));

subplot(2,2,1);
plot(100*(exp(4*[D.r_,Tnd.r_])-1));
grid('on');
title('Nominal rate');

subplot(2,2,2);
plot(100*(exp(4*[D.dp_,Tnd.dp_])-1));
grid('on');
title('Final goods prices, Q/Q');

subplot(2,2,3);
plot([D.Y_,Tnd.Y_]);
grid('on');
title('Output');

subplot(2,2,4);
plot([D.Q_,Tnd.Q_]);
grid('on');
title('Real wage');

ftitle({'','Data and HP trends',datestr(now())});

%% Plot HP filtered data
%
% Plot the last 6 years of HP gaps. Note you can use a date range as the
% first input argument when using PLOT for tseries objects. Y_ and Q_ have
% been HP filtered with the 'log' options true. This means these two series
% have been first logarithmised, then filtered, and the results
% de-logarithmised again. In other words, gap values like 1.01 or 0.95 mean
% +1 pct or -5 pct gaps.
%

plotrng = endhist-6*4+1 : endhist;
figure('position',get(0,'screenSize'));

subplot(2,2,1);
plot(plotrng,Gap.r_);
grid('on');
title('Nominal rate');

subplot(2,2,2);
plot(plotrng,Gap.dp_);
grid('on');
title('Final goods prices, Q/Q');

subplot(2,2,3);
plot(plotrng,Gap.Y_);
grid('on');
title('Output (not logarithmised)');

subplot(2,2,4);
plot(plotrng,Gap.Q_);
grid('on');
title('Real wage (not logarithmised)');

ftitle({'','HP filtered observables (last 6 years)',datestr(now())});

%% Save everything

save('read_data.mat');
