%% Read model into Matlab.

%% Clear workspace

clear('all');
close('all');
home();

%% Read the model
%
% Read the model code and creat a model object in Matlab's workspace.
%

m = model('SPBC.model','std',0.01);
display(m);

%% Create a parameter database
%
% Create a parameter database, and assign this database to the model
% object. Assign also some of the known steady-state values. These
% are then fixed and taken as granted when we call the sstate
% function.
%

P = struct();

%%
%
% Steady-state parameters.
%

P.beta = 0.99^(1/4);
P.eta = 1;
P.gamma = 0.50;
P.pie = 1.02^(1/4) - 1;

%%
%
% Transitory behavioural and policy parameters.
%

P.chi = 0;
P.xiw = Inf;
P.xip = Inf;
P.mup = 3;
P.mug = 0.5/4;

%%
%
% Autoregressions.
%

P.rhoa = 0;
P.rhor = 0;

%%
%
% Advise as many steady-state values that are obvious or easy to
% compute as possible.
%

% Some of the transition variables.
P.A = 1;
P.dp = P.pie;
P.dp4 = P.pie;
P.dw = P.pie;
P.g = 0;
P.r = (1 + P.pie)/P.beta - 1;

% Some of the measurement variables.
P.dp_ = P.dp;
P.r_ = P.r;

% Transitory parameters.

P.chi = 0.85;
P.xiw = 0.02;
P.xip = 0.04;
P.rhor = 0.8;
P.rhoa = 0.8;

P.std_ea = 0.001;

%%
%
% Assign parameters *_and_* steady-state values, and save the
% parameter database as Matlab's binary file.
%

m = assign(m,P);
get(m,'parameters')

%% Find steady state
%
% Use Optimisation Toolbox to solve the model for steady state. Fix
% those steady-state values that we have advised above. These will be
% taken as granted, and not solved for. Note that there are other,
% more sophisticated options in IRIS how to handle the problem of
% finding steady state in more complex models.
%
% There is a more sophisticated way how to find a model's steady
% state, which suitable for larger and more complex models. Check out
% the sample file |find_sstate.m| for an example.
%

m = sstate(m,'fix',{'A','dp','dw','g','r','dp4','dp_','r_'});
get(m,'sstatelevel')

%% Calibrate transitory parameters
%
% Transitory parameters were switched off above. This helps to find
% steady state faster. We assign proper values now (remember that
% these parameters don't change the model's steady state).
%
% There are several ways we can assign or change the model's
% parameters. All of them are perfectly equivalent.
%

%%
%
% Use the |assign| function's alternative syntax.
%

m = assign(m,'chi',0.85,'xiw',0.03,'xip',0.05,'rhor',0.8,'rhoa',0.8);

%%
%
% Reference directly the model object.
%

m.chi = 0.85;
m.xiw = 0.03;
m.xip = 0.05;
m.rhor = 0.8;
m.rhoa = 0.8;

%% Check that all equations hold in steady state

[flag,discrepancy,list] = chksstate(m);
if flag
   disp('Steady state OK.');
else
   discrepancy,
   error('Steady state error: "%s".\n',list{:});
end

%% Solve model
%
% Linearise the model around its steady state and solve the model by
% generlised Schur decomposition. Display the model object. The
% function |solve| returns a model object with a solution only when
% the Blanchard-Kahn saddle-path condition is satisfied. Otherwise it
% throws an warning. The second output argument, |npath|, can be one
% of the following:
%
% * 0: the model has no stable solution.
%
% * 1: the model has a unique stable solution, i.e. the Blanchard-Kahn
% saddle path condition is satisfied. This is the only time when the
% function |solve| returns a properly solved model object.
%
% * Inf: the model has mutliple stable solutions.
% 
% * NaN: some of the derivatives of the model's equations produced NaNs.
%
% * -1: some part of the state-space form is singular and cannot be fixed.
%
% * imaginary unit: some of the derivatives of the model's equations
% produced complex numbers.
%

[m,npath] = solve(m);

if npath == 1
   disp('Dynamic solution OK.');
end

display(m);

%% Save everything

save('read_model.mat');
