function [d,c,f,u,e] = forecast(this,data,range,j,varargin)

% Parse required input arguments.
p = inputParser();
p.addRequired('f',@isfavar);
p.addRequired('init',@(x) istseries(x) || isstruct(x));
p.addRequired('range',@isnumeric);
p.addRequired('j',@(x) isempty(x) || istseries(x) || isstruct(x));
p.parse(this,data,range,j);

% Parse options.
options = passvalopt('FAVAR.forecast',varargin{:});

%**************************************************************************
% Function body.

[ny,nx,p,q,nalt] = size(this);

range = range(1) : range(end);

if isstruct(data) ...
      && ~isfield(data,'init') ...
      && isfield(data,'mean') ...
      && istseries(data.mean)
   data = data.mean;
end
if istseries(data)
   % Only mean tseries supplied. No uncertainty in initial condition.
   [ans,ans,x0] = datarequest(this,data,range(1)-p:range(1)-1);
   x0 = x0(:,end:-1:1,:,:);
   x0 = x0(:);
   P0 = 0;
else
   % Complete description of initial conditions.
   index = abs(range(1)-1 - data.init{3}) <= 0.01;
   if isempty(index) || ~any(index)
      % Initial condition not available.
      FAVAR.error(4);
   end
   x0 = data.init{1}(:,index,:,:);
   P0 = data.init{2}(:,:,index,:,:);
end
nper = length(range);
ndata = size(x0,3);

if ~isempty(j)
   if isstruct(j) && isfield(j,'mean')
      j = j.mean;
   end
   [outputformat,range,y] = datarequest(this,j,range,options);
   [this,y] = standardise(this,y);
else
   y = nan([ny,nper,ndata]);
   outputformat = options.output;
   if strcmpi(outputformat,'auto')
      if isempty(this.ynames)
         outputformat = 'tseries';
      else
         outputformat = 'dbase';
      end
   end   
end

% Inversion method for the FMSE matrices.
if strcmpi(options.tolerance,'auto')
   invfunc = @pinv;
else
   invfunc = @(x) pinv(x,options.tolerance);
end

% Zero off-diagonal elements in idiosyncratic cov matrix if requested.
Sigma = this.Sigma;
if ~options.cross
   Sigma = diag(diag(Sigma));
end

% Run Kalman smoother to improve on factor estimates.
[x,Px,e,u,y,Py,yindex] = time_domain.smoothervar( ...
   this.A,this.B,[],this.C,1,Sigma,y,[],x0,P0,invfunc,true);

if options.meanonly
   Px = Px(:,:,[],[]);
   Py = [];
end

[y,Py] = FAVAR.destandardise(this.Mean,this.Std,y,Py);
d = outputdata(this,outputformat,range,y,Py,'ynames');

if nargout > 1
   % Common components.
   [c,Pc] = FAVAR.cc(this.C,x(1:nx,:,:),Px(1:nx,1:nx,:,:));
   [c,Pc] = FAVAR.destandardise(this.Mean,this.Std,c,Pc);
   c = outputdata(this,outputformat,range,c,Pc,'ynames');   
end

if nargout > 2
   f = outputdata(this,'tseries',range,x(1:nx,:,:),Px(1:nx,1:nx,:,:));
   if ~options.meanonly
      % Means and MSEs that can be used as initial condition.
      lastobs = find(any(yindex ~= 0,1),1,'last');
      if isempty(lastobs)
         index = 1 : nper;
      else
         index = lastobs : nper;
      end
      f.init = {x(:,index,:),Px(:,:,index,:),range(index)};
   end
end

if nargout > 3
   u = FAVAR.destandardise(0,this.Std,u);
   u = outputdata(this,outputformat,range,u,NaN,'ynames');
end

if nargout > 5
   template = tseries();
   e = replace(template,permute(e,[2,1,3]),range(1));
   if ~options.meanonly
   e = struct( ...
      'mean',e, ...
      'std',replace(template,zeros([0,size(e,1),size(e,3)]),range(1)) ...
      );
   end
end

end
% End of primary function.