function [this,data,B] = svar(this,data,varargin)
% SVAR  Convert reduced-form VAR to structural VAR.

% The IRIS Toolbox 2009/10/29.
% Copyright 2007-2009 Jaromir Benes.

if nargin == 1
   data = [];
end

% Parse required input arguments.
P = inputParser();
P.addRequired('w',@isvar);
P.addRequired('input',@(x) isnumeric(x) || istseries(x) || isstruct(x));
P.parse(this,data);

% Parse options.
options = passvalopt('VAR.SVAR',varargin{1:end});

%**************************************************************************
% Function body.

[ny,p,nalt] = size(this);
A = time_domain.var2poly(this.A);
Omega = this.Omega;

% Handle residuals.
doresiduals = nargin > 1 && nargout > 1 && ~isempty(data);

if doresiduals
   % Get data.
   [outputformat,range,y,e] = datarequest(this,data,Inf,options);
   if size(e,3) == 1 && nalt > 1
      y = y(:,:,ones([1,nalt]));
      e = e(:,:,ones([1,nalt]));
   end
else
   y = [];
   e = [];
end

% Expand option 'std'.
if nalt > 1 && length(options.std) == 1
   options.std = options.std(ones([1,nalt]));
else
   options.std = options.std(:)';
end
this.std = options.std;

this.B = zeros([ny,ny,nalt]);
switch lower(options.method)
   case 'chol'
      reorder_();
      for ialt = 1 : nalt
         this.B(:,:,ialt) = transpose(chol(Omega(:,:,ialt)));
         if options.std(ialt) ~= 1
            this.B(:,:,ialt) = this.B(:,:,ialt)/options.std(ialt);
         end
      end
      backorder_();
      convertresiduals_();
   case 'qr'
      reorder_();
      C = sum(A,3);
      for ialt = 1 : nalt
         B0 = transpose(chol(Omega(:,:,ialt)));
         if rank(C(:,:,1,ialt)) == ny
            [Q,R] = qr(transpose(C(:,:,1,ialt)\B0));
         else
            [Q,R] = qr(transpose(pinv(C(:,:,1,ialt))*B0));
         end
         this.B(:,:,ialt) = B0*Q;
         if options.std(ialt) ~= 1
            this.B(:,:,ialt) = this.B(:,:,ialt)/options.std(ialt);
         end
      end
      backorder_();
      convertresiduals_();      
   case 'svd'
      q = options.rank;
      [this.B,e] = time_domain.orthonormalise(Omega,q,options.std,e);
      % Recompute covariance matrix of reduced-form residuals if rank is
      % reduced.
      if q < ny
         varvec = options.std .^ 2;
         for ialt = 1 : nalt
            this.Omega(:,:,ialt) = ...
               this.B(:,1:q,ialt)*this.B(:,1:q,ialt)'*varvec(ialt);
         end
      end
end

if doresiduals
   % Output data.
   data = outputdata(this,outputformat,range,[y;e],[],'names');
end

% End of function body.

   %***********************************************************************
   % Nested function reorder_().
   function reorder_()
      if ~isempty(options.ordering)
         options.ordering = options.ordering(:)';
         if length(options.ordering) ~= ny ...
               || length(intersect(1:ny,options.ordering)) ~= ny
            VAR.error(14);
         end
         A = A(options.ordering,options.ordering,:,:);
         Omega = Omega(options.ordering,options.ordering,:);
      end
   end
   % End of nested function reorder_().

   %***********************************************************************
   % Nested function backorder_().
   function backorder_()
      % Put variables (and residuals, if request) back in order.
      if ~isempty(options.ordering)
         [aux,backorder] = sort(options.ordering);
         if options.reorderresiduals
            this.B = this.B(backorder,backorder,:);
         else
            this.B = this.B(backorder,:,:);
         end
      end
   end
   % End of nested function backorder_().
   
   %***********************************************************************
   % Nested function convertresiduals_().
   function convertresiduals_()
      if doresiduals
         for iloop = 1 : nalt
            if iloop <= nalt
               Bi = this.B(:,:,iloop);
            end
            e(:,:,iloop) = Bi \ e(:,:,iloop);
         end
      end
   end
   % End of nested function convertresiduals_().
   
end
% End of primary function.