function [w,data,rangefitted,Rr,count] = estimate(w,varargin)
% Type <a href="matlab: idoc VAR.estimate">idoc VAR.estimate</a> or click the link to get help on this function.

% The IRIS Toolbox.
% Copyright 2007-2010 Jaromir Benes.

% Get input data including pre-sample.
[y,w.range,ynames,inputformat,varargin] = inputdata(varargin{:});

% Pass and validate options.
options = passvalopt('VAR.estimate',varargin{:});

% Names of endogenous variables.
if isempty(ynames)
   w.ynames = options.ynames;
else
   w.ynames = ynames;
end
if iscellstr(w.ynames)
   w.ynames = w.ynames(:)';
end
% Names of residuals.
w.enames = options.enames;

% Both 'const' and 'constant' allowed for bkw cmp.
if ~isempty(options.const)
   options.constant = options.const;
end

options.mean = vec(options.mean);

if strcmpi(options.output,'auto')
   outputformat = inputformat;
else
   outputformat = options.output;
end

if ~isempty(options.userdata)
   w = userdata(w,options.userdata);
end

%**************************************************************************
% Function body.

p = options.order;
ny = size(y,1);
nper = size(y,2);
ndata = size(y,3);

% All data sets must have the same structure of missing observations.
if ndata > 1
   nanindex = isnan(y);
   allnan = all(nanindex,3);
   anynan = any(nanindex,3);
   if any(anynan & ~allnan)
      VAR.error(32);
   end
end

if ~isempty(options.mean)
   ymean = options.mean(:,ones([1,size(y,2)]));
   for idata = 1 : ndata
      y(:,:,idata) = y(:,:,idata) - ymean;
   end
   options.constant = false;
end

% Arrange data on LHS and RHS; size(y0,2) is nper-p.
[y0,k0,y1,g1] = VAR.stackdata(y,options);
ng = size(g1,1);
nk = size(k0,1);

% Read parameter restrictions, and set up their hyperparameter form.
% They are organised as follows:
% * Rr = [R,r],
% * beta = R*gamma + r.
w = restrict_(w,ny,nk,ng,options);

% Get the number of hyperparameters.
if isempty(w.Rr)
   % Unrestricted VAR.
   w.nhyper = ny*(nk+p*ny+ng);
else
   % Parameter restrictions in the hyperparameter form:
   % beta = R*gamma + r;
   % The number of hyperparams is given by the number of columns of R.
   % The Rr matrix is [R,r], so we need to subtract 1.
   w.nhyper = size(w.Rr,2) - 1;
end

% Number of priors.
nprior = size(options.bvar,3);

% Total number of alternatives.
nloop = max([ndata,nprior]);

% Estimate rVAR parameters.
p = options.order;
w.K = nan([ny,nloop]);
w.A = nan([ny,ny*p,nloop]);
w.Omega = nan([ny,ny,nloop]);
w.T = nan([ny*p,ny*p,nloop]);
w.U = nan([ny*p,ny*p,nloop]);
w.eigval = nan([1,ny*p,nloop]);
w.Sigma = [];
resid = nan([ny,nper,nloop]);
count = zeros([1,nloop]);
w.fitted = false([1,nper]);
w.aic = nan([1,nloop]);
w.sbc = nan([1,nloop]);

% Do not pass options.bvar into glsq().
bvar = options.bvar;
options = rmfield(options,'bvar');

% Create command-window progress bar.
if options.progress
   progress = progressBar('IRIS VAR.estimate progress');
end

% Main loop.
use = struct();
for iloop = 1 : nloop   
   if iloop <= ndata
      use.y0 = y0(:,:,iloop);
      use.y1 = y1(:,:,iloop);
      use.g1 = g1(:,:,iloop);
   end
   if isempty(bvar)
      use.bvar = [];
   elseif iloop <= nprior
      nd = size(bvar,2);
      py0 = bvar(1:ny,:,iloop);
      pk0 = bvar(ny+1,:,iloop);
      pk0 = pk0(1:nk,:);
      py1 = bvar(ny+1+(1:ny*p),:,iloop);
      pg1 = zeros([ng,nd]);
      if ng > 0 && size(bvar,1) > ny*(p+1)+1
         pg1(:,:) = bvar(end-ny+1:end,:,iloop);
         % Reduce number of lags
         % because VAR will be estimated as VECM in first differences.
         py1 = py1(1:end-ny,:);
      end
      use.bvar = [py0;pk0;py1;pg1];
   end
   % Run generalised least squares.
   [w.K(1:end,iloop),w.A(:,:,iloop),...
      w.Omega(:,:,iloop),w.Sigma(:,:,iloop),...
      resid(:,p+1:end,iloop),count(iloop),w.fitted(1,p+1:end)] = ...
      glsq(w,use.y0,k0,use.y1,use.g1,use.bvar,options);
   % Compute Schur triangular representation.
   if options.schur
      [w.T(:,:,iloop),w.U(:,:,iloop),w.eigval(1,:,iloop)] = ...
         VAR.schur(w.A(:,:,iloop));
   end
   % Compute information criteria (AIC, SBC).
   [w.aic(iloop),w.sbc(iloop)] = ...
      VAR.infocrit(w.Omega(:,:,iloop),w.nhyper,w.fitted);
   if options.progress
      progress = update(progress,iloop/nloop);
   end
end
% End of main loop.

if size(y,3) == 1 && nloop > 1
   y = y(:,:,ones([1,nloop]));
end

% Back out VAR constant for given mean restriction.
if ~isempty(options.mean)
   for iloop = 1 : nloop
      w.K(:,iloop) = sum(var2poly(w.A(:,:,iloop)),3)*options.mean;
   end
end

% Report observations that could not be fitted.
if options.warning && any(~w.fitted(p+1:end))
   missing = w.range(p+1:end);
   missing = missing(~w.fitted(p+1:end));
   VAR.warning(4,dat1charlist(missing));
end

% Range fitted.
rangefitted = w.range(w.fitted);

if nargout > 2
   Rr = w.Rr;
end

% Convert data to output time series.
data = outputdata(w,outputformat,w.range,[y;resid],[],'names');

end
% End of primary function.