function data = forecast(w,data,range,jdata,varargin)

% The IRIS Toolbox.
% Copyright 2007-2010 Jaromir Benes.

if nargin < 4
   jdata = [];
end

% Validate required input arguments.
p = inputParser();
p.addRequired('w',@isvar);
p.addRequired('input',@(x) isnumeric(x) || isstruct(x) || istseries(x));
p.addRequired('range',@isnumeric);
p.addRequired('cond',@(x) isempty(x) || isstruct(x) || istseries(x));
p.parse(w,data,range,jdata);

options = passvalopt('VAR.forecast',varargin{1:end});

% Bkw compatibility only.
if ~isempty(options.returnmse)
   warning('iris:obsolete', ...
      ['\n*** The ''returnMSE'' option is obsolete and will not be supported in future versions. ', ...
      'Use ''meanOnly'' instead.']);
   options.meanonly = ~options.returnmse;
end

[ny,p,nalt] = size(w);
if istseries(data) && ~any(size(data,2) == [2*ny,ny])
   VAR.error(16,'input time series');
end

if istseries(jdata) && ~any(size(jdata,2) == [2*ny,ny])
   VAR.error(16,'conditions');
end

if isequal(range,Inf)
   VAR.error(27,'<<matlab:idoc(''VAR.forecast'') forecast>>');
end
   
%**************************************************************************
% Function body.

if isempty(range)
   warning('iris:VAR', ...
      '\n*** Forecast range is empty.');
   if options.meanonly
      data = [];
   else
      data = struct();
      data.mean = [];
      data.std = [];
   end
end

if (range(1) > range(end))
   % Go backward in time.
   backcast = true;
   w = reverse(w);
   xrange = range(end) : range(1)+p;
   range = range(end) : range(1);
else
   backcast = false;
   xrange = range(1)-p : range(end);
end

% Include pre-sample.
[outputformat,xrange,x0,e] = datarequest(w,data,xrange,options);
% Do not include pre-sample.
[ans,ans,jx] = datarequest(w,jdata,range);

if backcast
   x0 = x0(:,end:-1:1,:,:);
   e = e(:,end:-1:1,:,:);
   jx = jx(:,end:-1:1,:,:);
end

nper = length(xrange);
x0 = x0(:,1:p,:);
e = e(:,p+1:end,:);

ndata = size(x0,3);
ncond = size(jx,3);
nloop = max([nalt,ndata,ncond]);

% Stack initial conditions.
x0 = reshape(vec(x0(:,p:-1:1,:)),[ny*p,nloop]);

X = nan([ny,nper,nloop]);
E = nan([ny,nper,nloop]);
P = zeros([ny,ny,nper,nloop]);

Z = eye(ny);
for iloop = 1 : nloop
   if iloop <= nalt
      Ai = w.A(:,:,iloop);
      if options.deviation
         Ki = [];
      else
         Ki = w.K(:,iloop);
      end
      if isempty(w.B)
         Omgi = w.Omega(:,:,iloop);
         Bi = [];
      else
         Omgi = diag(w.std.^2*ones([1,ny]));
         Bi = w.B(:,:,iloop);
      end
   end
   if iloop <= ndata
      x0i = x0(:,iloop);
      ei = e(:,:,iloop);
   end
   if iloop <= ncond
      jxi = jx(:,:,iloop);
   end
   % Run Kalman filter and smoother.
   X(:,p:-1:1,iloop) = reshape(x0i,[ny,p]);
   % Add pre-sample initial condition.
   [Xi,Pi,E(:,p+1:end,iloop)] = ...
      time_domain.smoothervar(Ai,Bi,Ki,Z,Omgi,0,jxi,ei,x0i,0,@inv,true);
   X(:,p+1:end,iloop) = Xi(1:ny,:);
   P(:,:,p+1:end,iloop) = Pi(1:ny,1:ny,:);
   % Set MSE matrix entries to zero for observed data points.
   index = ~isnan(jxi);
   for t = find(any(index,1))
      P(index(:,t),:,p+t,iloop) = 0;
      P(:,index(:,t),p+t,iloop) = 0;
   end
end

if backcast
   X = X(:,end:-1:1,:);
   E = E(:,end:-1:1,:);
   P = P(:,:,end:-1:1,:);
end

% Output data.
if options.meanonly
   data = outputdata(w,outputformat,xrange,[X;E],[],'names');
else
   data = outputdata(w,outputformat,xrange,[X;E],P,'names');
end

end
% End of primary function.
