function [K,A,Omega,Sigma,resid,count,sample] ...
   = glsq(w,y0,k0,y1,g1,bvar,options)
% GLSQ  Generalised least squares estimator for reduced-form VARs and BVARs.

% The IRIS Toolbox.
% Copyright 2007-2010 Jaromir Benes.

%**************************************************************************
% Function body.

p = options.order;
ny = size(y0,1);
nk = size(k0,1);
ng = size(g1,1);
if ng > 0
   % Order of VEC is reduced by 1.
   p = p - 1;
end

% Matrix of explanatory observations.
X = [k0;y1;g1];

% Exclude NaN observations.
sample = all(~isnan([y0;X]),1);
nobs = sum(sample);
y0 = y0(:,sample);
X = X(:,sample);

nb = size(bvar,2);
if nb > 0
   % Include dummy observations.
   % They must not contain NaNs.
   y0 = [bvar(1:ny,:),y0];
   X = [bvar(ny+1:end,:),X];
else
   nb = 0;
end

if ~isempty(w.Rr)
   R = w.Rr(:,1:end-1);
   r = w.Rr(:,end);
else
   R = [];
   r = [];
end

if ~isempty(R) && options.eqtnbyeqtn
   % Estimate equation by equation with parameter restrictions.
   pos = (1:ny)';
   pos = pos(:,ones([1,nk+ny*p+ng]));
   pos = pos(:);
   M = X*X';
   beta = nan([ny*(nk+ny*p+ng),1]);
   realsmall = getrealsmall();
   count = 1;
   for i = 1 : ny
      % Get restrictions for equation i.
      betaindex = pos == i;
      Ri = R(betaindex,:);
      gammaindex = any(abs(Ri) > realsmall,1);
      Ri = Ri(:,gammaindex);
      ri = r(betaindex);
      % Estimate free hyperparameters.
      ci = vec(y0(i,:)) - X'*ri;      
      gammai = (Ri'*M*Ri) \ (Ri'*X*ci);
      beta(betaindex) = Ri*gammai + ri;
   end
   beta = reshape(beta,[ny,ny*p+nk+ng]);
   resid = y0(:,nb+1:end) - beta*X(:,nb+1:end);
   Omega = resid*resid' / nobs;
else
   % Ordinary least squares for unrestricted or bayesian VAR.
   beta = y0 / X;
   resid = y0(:,nb+1:end) - beta*X(:,nb+1:end);
   Omega = resid*resid' / nobs;
   count = 1;
   % Run generalized least squares only if there are parameter restrictions.
   if ~isempty(R)
      M = X*X';
      maxdiff = Inf;
      while maxdiff > options.tolerance && count <= options.maxiter
         beta0 = beta;
         Omegainv = inv(Omega);
         c = vec(y0) - kron(X',eye(ny))*r;
         % Estimate free hyperparameters.
         gamma = (R'*kron(M,Omegainv)*R) \ (R'*kron(X,Omegainv)*c);
         % Compute parameters.
         beta = reshape(R*gamma + r,[ny,ny*p+nk+ng]);
         resid = y0(:,nb+1:end) - beta*X(:,nb+1:end);
         Omega = resid*resid' / nobs;
         maxdiff = max(vec(abs(beta - beta0)));
         count = count + 1;
      end
   end
end

% Asymptotic cov of parameters.
% Not available for cointegrated systems.
if options.covparameters && ng == 0
   if nb > 0
      % Compute Omega based on unrestricted VAR and diffuse priors.
      tmpbeta = y0(:,nb+1:end) / X(:,nb+1:end);
      tmpresid = y0(:,nb+1:end) - tmpbeta*X(:,nb+1:end);
      tmpomega = tmpresid*tmpresid' / nobs;
      % Note that cov of prior distribution is implicitly
      % kron(X(:,1:nb)*X(:,1:nb)',tmpomega)).
   else
      tmpomega = Omega;
   end
   if isempty(R)
      % Unrestricted parameters.
      M = X*X';
      Sigma = kron(inv(M),tmpomega);
   else
      % Parameter restrictions.
      if nb > 0
         tmpomegainv = inv(tmpomega);
      else
         tmpomegainv = inv(Omega);
      end
      % M already available.
      Sigma = R*((R'*kron(M,tmpomegainv)*R) \ R');
   end
else
   Sigma = [];
end

% Constant vector.
if nk > 0
   K = beta(:,1:nk);
   beta(:,1:nk) = [];
else
   K = zeros([ny,1]);
end

% Transition matrices.
A = beta(:,1:ny*p);
beta(:,1:ny*p) = [];

% Convert VEC to co-integrated VAR.
if ng > 0
   % Coefficients on co-integrating vectors.
   G = beta(:,1:ng);
   if size(options.cointeg,2) == ny+1
      K = K + G*options.cointeg(:,1);
      options.cointeg(:,1) = [];
   end
   A = reshape(A,[ny,ny,p]);
   A = time_domain.polyprod(A,cat(3,eye(ny),-eye(ny)));
   A = time_domain.polysum(A,eye(ny)+G*options.cointeg);
   p = p + 1;
   A = reshape(A,[ny,ny*p]);
end

% Residuals.
tmpresid = resid;
resid = nan([ny,length(sample)]);
resid(:,sample) = tmpresid;

end
% End of primary function.