function [stat,crit,range] = portest(w,data,h,level)
% PORTEST  Portmanteau test for autocorrelation in VAR residuals.
%
% Syntax:
%   [stat,crit,range] = portest(w,resid,h,level)
% Output arguments:
%   stat [ numeric ] Portmanteau test statistic.
%   crit [ numeric ] Critical value at desired level of significance.
%   range [ numeric ] Actually used range (IRIS serial date numbers).
% Required input arguments:
%   w [ VAR | swar ] VAR from which tested residuals were obtained.
%   data [ tseries ] VAR residuals (or VAR output data) to be tested for autocorrelation.
%   h [ numeric ] Test horizon (greater than VAR order).
%   level [ numeric ] Desired level of significance.

% The IRIS Toolbox 2009/07/03.
% Copyright 2007-2009 Jaromir Benes.

if nargin < 3
   level = 0.05;
end

%**************************************************************************
% Function body.

[ny,p,nalt] = size(w);
if h <= p
   VAR.error(8);
end

% Request residuals.
[ans,ans,ans,e] = datarequest(w,data,w.range);
ndata = size(e,3);
if ndata ~= nalt
   VAR.error(24);
end

% Orthonormalise residuals by Choleski factor of Omega.
for ialt = 1 : nalt
   P = chol(w.Omega(:,:,ialt));
   e(:,:,ialt) = P\e(:,:,ialt);
end

% Test statistic.
nobs = sum(w.fitted);
stat = zeros([1,nalt]);
for ialt = 1 : nalt
   ei = e(:,w.fitted,ialt);
   for i = 1 : h
      Ci = ei(:,1+i:end)*ei(:,1:end-i)' / nobs;
      stat(ialt) = stat(ialt) + trace(Ci'*Ci) / (nobs-i);
   end
   stat(ialt) = (nobs^2)*stat(ialt);
end

% Critical value
if nargout > 1
   crit = chi2inv(1-level,ny^2*(h-p));
end

end
% End of primary function.