function data = simulate(w,data,range,varargin)
% <a href="matlab: edit rvar/simulate">SIMULATE</a>  Simulate RVAR model.
%
% Syntax:
%   output = simulate(w,input,range,...)
% Output arguments:
%   output [ tseries ] Output data as multivariate time series.
% Required input arguments:
%   w [ rvar ] RVAR model to be simulated.
%   input [ tseries ] Input data as multivariate time series.
%   range [ numeric ] Simulation range.
% <a href="options.html">Optional input arguments:</a>
%   'contributions' [ true | <a href="default.html">false</a> ] Decompose simulation into contributions of individual shocks.
%   'deviation' [ true | <a href="default.html">false</a> ] Input and output data are deviations from steady state.

% The IRIS Toolbox 2009/05/21.
% Copyright 2007 Jaromir Benes.

% Validate required input arguments.
p = inputParser();
p.addRequired('w',@isvar);
p.addRequired('input',@(x) isnumeric(x) || istseries(x) || isstruct(x));
p.addRequired('range',@isnumeric);
p.parse(w,data,range);

[ny,p,nalt] = size(w);

options = passvalopt('VAR.simulate',varargin{1:end});

if istseries(data) && ~any(size(data,2) == [2*ny,ny])
   VAR.error(16,'input time series');
end

if isequal(range,Inf)
   VAR.error(27,'<<matlab:idoc(''VAR.simulate'') simulate>>');
end

%**************************************************************************
% Function body.

if isempty(range)
   return
end

if range(1) > range(end)
   backcast = true;
   w = reverse(w);
   range = range(end) : range(1)+p;
else
   backcast = false;
   range = range(1)-p : range(end);
end

[outputformat,range,x,e] = datarequest(w,data,range,options);

if backcast
   x = x(:,end:-1:1,:,:);
   e = e(:,end:-1:1,:,:);
end

e(:,1:p,:) = NaN;
nper = length(range);
ndata = size(x,3);
nloop = max([nalt,ndata]);

if options.contributions
   if nloop > 1
      % Cannot run contributions for multiple data sets or params.
      VAR.error(17,'SIMULATE');
   else
      % Simulation of contributions.
      nloop = ny + 1;
   end
end

if ndata < nloop
   expand = ones([1,nloop - ndata]);
   x = cat(3,x,x(:,:,end*expand));
   e = cat(3,e,e(:,:,end*expand));
end

for iloop = 1 : nloop
   if iloop <= nalt
      Ai = w.A(:,:,iloop);
      Ki = w.K(:,iloop);
      if isempty(w.B)
         Bi = [];
      else
         Bi = w.B(:,:,iloop);
      end
   end
   constant = ~options.deviation;
   if options.contributions
      if iloop <= ny
         index = true([1,ny]);
         index(iloop) = false;
         e(index,:,iloop) = 0;
         x(:,1:p,iloop) = 0;
         constant = false;
      else
         e(:,:,iloop) = 0;
      end
   end
   if isempty(Bi)
      Bei = e(:,:,iloop);
   else
      Bei = Bi*e(:,:,iloop);
   end
   xi = x(:,:,iloop);
   for t = p + 1 : nper
      xilags = xi(:,t-(1:p));
      xi(:,t) = Ai*xilags(:) + Bei(:,t);
      if constant
         xi(:,t) = xi(:,t) + Ki;
      end
   end
   x(:,:,iloop) = xi;
end

if backcast
   x = x(:,end:-1:1,:,:);
   e = e(:,end:-1:1,:,:);
end

% Output data.
data = outputdata(w,outputformat,range,[x;e],[],'names');

end
% End of primary function.