function [x,flag] = specget(this,query)
% <a href="matlab: edit VAR/get">GET</a>  Access/query VAR object attributes and properties.
%
% Syntax:
%   [value,value,...] = get(this,attrib,attrib,...)
% Output arguments:
%   value [ any ] Value of requested attribute.
% Required input arguments:
%   m [ VAR ] VAR object.
%   attrib [ char ] Requested attribute.
% Attributes:
%   'A' [ numeric ] Matrix polynomial A from A(L) x(t) = K + B e(t).
%   'A*' [ numeric ] Matrix polynominal A* from x(t) = A*(L) x(t-1) + K + B e(t).
%   'AIC' [ numeric ] Akaike information criterion.
%   'B' [ numeric ] Matrix B from "A(L) x(t) = K + B e(t)".
%   'Comment' [ char | cellstr ] User comments attached to VAR model.
%   'CumLongRun' [ numeric ] Asymptotic cumulative shock responses.
%   'EigVal' [ numeric ] All eigenvalues.
%   'C' [ numeric ] Constant term K from "A(L) x(t) = K + B e(t)".
%   'NHyper' [ numeric ] Number of estimated hyperparameters.
%   'NPer' [ numeric ] Length of effective estimation sample.
%   'Omega' [ numeric ] Residual (shock) covariance matrix.
%   'Order' [ numeric ] Order of VAR model.
%   'Roots' alias for 'eigval'.
%   'Sample' [ numeric ] Effective estimation sample, i.e. <a href="dates.html">IRIS serial date numbers</a>.
%   'StableRoots'  Eigenvalues smaller than 1 in magnitude.
%   'SBC' [ numeric ] Schwarz Bayesian criterion.
%   'ExplosiveRoots' [ numeric ] Eigenvalues greater than 1 in magnitude.
%   'UnitRoots' [ numeric ] Eigenvalues indistinguishable from 1 in magnitude.

% The IRIS Toolbox 2009/10/29.
% Copyright 2007-2009 Jaromir Benes.

%**************************************************************************
% Function body.

[ny,p,nalt] = size(this);
realsmall = getrealsmall();
flag = true;

switch query
   case 'parameters'
      x = struct();
      x.A = get(this,'A');
      x.B = get(this,'B');
      x.K = get(this,'K');
      x.Omega = get(this,'Omega');
   case 'a'
      if all(size(this.A) == 0)
         x = [];
      else
         x = time_domain.var2poly(this.A);
      end
   case 'a*'
      x = get(this,'a');
      x = -x(:,:,2:end,:);
   case 'b'
      x = this.B;
   case 't'
      x = this.T;
   case 'u'
      x = this.U;
   case {'const','c','k'}
      x = this.K;
   case {'omega','cove','covresiduals'}
      x = this.Omega;
   case 'std'
      x = this.std;
   case {'sigma','covp','covparameters'}
      x = this.Sigma;
   case 'aic'
      x = this.aic;
   case 'sbc'
      x = this.sbc;
   case {'eig','eigval','roots'}
      x = this.eigval;
   case {'stableroots','explosiveroots','unstableroots','unitroots'}
      switch query
         case 'stableroots'
            test = @(x) abs(x) < (1 - realsmall);
         case {'explosiveroots','unstableroots'}
            test = @(x) abs(x) > (1 + realsmall);
         case 'unitroots'
            test = @(x) abs(abs(x) - 1) <= realsmall;
      end
      x = nan(size(this.eigval));
      for ialt = 1 : nalt
         index = test(this.eigval(1,:,ialt));
         x(1,1:sum(index),ialt) = this.eigval(1,index,ialt);
      end
      index = all(isnan(x),3);
      x(:,index,:) = [];
   case 'nhyper'
      x = this.nhyper;
   case {'nper','nobs'}
      x = sum(this.fitted);
   case {'order','p'}
      x = p;
   case {'sample','fitted'}
      x = this.range(this.fitted);
   case 'comment'
      x = this.comment;
   case 'cumlongrun'
      C = sum(time_domain.var2poly(this.A),3);
      x = nan([ny,ny,nalt]);
      for ialt = 1 : nalt
         if rank(C(:,:,1,ialt)) == ny
            x(:,:,ialt) = C(:,:,1,ialt)\this.B(:,:,ialt);
         else
            x(:,:,ialt) = pinv(C(:,:,1,ialt))*this.B(:,:,ialt);
         end
      end
   case {'constraints','restrictions'}
      x = this.Rr;
   case {'ynames','ylist'}
      x = getynames_();
   case {'enames','elist'}
      x = getenames_();
   case {'names','list'}
      x = [getynames_(),getenames_()];
   otherwise
      x = [];
      flag = false;
end

% End of function body.

   %***********************************************************************
   % Nested function getynames_().
   function ynames = getynames_()
      if isfunc(this.ynames)
         ynames = cell([1,ny]);
         for i = 1 : ny
            ynames{i} = this.ynames(i);
         end
      else
         ynames = this.ynames;
      end
   end
   % End of nested function getynames_().
   
   %***********************************************************************
   % Nested function getynames_().      
   function enames = getenames_()
      if isfunc(this.enames)
         ynames = getynames_();
         enames = cell([1,ny]);
         for i = 1 : ny
            enames{i} = this.enames(ynames{i},i);
         end
      else
         enames = this.enames;
      end
   end
   % End of nested function getynames_().

end
% End of primary function.