function [code,d] = sprintf(this,varargin)
% <a href="matlab: edit VAR/sprintf">SPRINTF</a>  Write formatted VAR model code to string.
%
% Syntax:
%   s = sprintf(this,...)
% Required input arguments:
%   s [ char ] VAR model code.
%   this [ VAR ] VAR model.
% <a href="options.html">Optional input arguments:</a>
%   'decimal' [ numeric | <a href="default.html">empty</a> ] Precision (decimal places).
%   'declare' [ true | <a href="default.html">false</a> ]  Add declaration blocks for VAR variables, residuals, and equations.
%   'enames' [ cellstr | char | <a href="default.html">{'e1','e2',...}</a> ] Names of VAR residuals.
%   'format' [ char | <a href="default.html">'%+.16e'</a> ] Numeric format for parameter values.
%   'ynames' [ cellstr | char | <a href="default.html">{'y1','y2',...}</a> ] Names of VAR variables.
%   'tolerance' [ numeric | <a href="default.html">getrealsmall()</a> ] Ignore VAR parameters smaller than tolerance in absolute value.

% The IRIS Toolbox 2009/09/24.
% Copyright 2007-2009 Jaromir Benes.

if isempty(this.B)
   VAR.error(30,'SPRINTF or FPRINTF');
end

options = passvalopt('VAR.sprintf',varargin{:});

if isempty(strfind(options.format,'%+'))
   error('Format string must contain %+.');
end

if ~isempty(options.decimal)
   options.format = ['%+.',sprintf('%g',options.decimal),'e'];
end

%********************************************************************
% Function body.

[ny,p,nalt] = size(this);
if nalt > 1
   % Cannot apply to multiple parameterisations.
   VAR.error(19,'SPRINTF or FPRINTF');
end

if ~isempty(options.ynames)
   this.ynames = options.ynames;
end
if isempty(this.ynames) || numel(this.ynames) < ny
   VAR.error(29);
end

if ~isempty(options.enames)
   this.enames = options.enames;
end
enames = get(this,'enames');

for i = 1 : ny
   if isempty(strfind(this.ynames{i},'{t}'))
      this.ynames{i} = sprintf('%s{t}',this.ynames{i});
   end
end

% Time subscripts.
this.ynames = strrep(this.ynames,'{t}','{%+g}');
ynames = cell([1,ny]);
for i = 1 : ny
   ynames{i} = cell([1,p+1]);
   for j = 0 : p
      ynames{i}{1+j} = sprintf(this.ynames{i},-j);
   end
   ynames{i}{1} = strrep(ynames{i}{1},'{-0}','');
end

% Retrieve system matrices.
A = get(this,'A');
A = -A(:,:,2:end);
B = this.B;
if options.constant
   K = this.K;
else
   K = zeros([ny,1]);
end

% Number of digits for printing parameter indices.
if ~options.hardparameters
   pformat = ['%',sprintf('%g',floor(log10(max([ny,p])))+1),'g'];
end

% Write equations.
equation = cell([1,ny]);
d = struct();
for eq = 1 : ny
   equation{eq} = [ynames{eq}{1},' = '];
   rhs = false;
   if abs(K(eq)) > options.tolerance
      equation{eq} = [equation{eq},' ', ...
         printparameter_('K',{eq},K(eq))];
      rhs = true;
   end
   for t = 1 : p
      for y = 1 : ny
         if abs(A(eq,y,t)) > options.tolerance
            equation{eq} = [equation{eq},' ', ...
               printparameter_('A',{eq,y,t},A(eq,y,t)),'*',ynames{y}{1+t}];
            rhs = true;
         end
      end
   end
   for e = 1 : ny
      if abs(B(eq,e)) > options.tolerance
         equation{eq} = [equation{eq},' ', ...
            printparameter_('B',{eq,e},B(eq,e)),'*',enames{e}];
         rhs = true;
      end
   end
   if ~rhs
      % If nothing occurs on the RHS, add zero.
      equation{eq} = [equation{eq},' 0'];
   end
end
code = sprintf('%s;\n',equation{:});

if ~options.hardparameters
   for i = 1 : ny
      d.(['std_',enames{i}]) = this.std;
   end
end

% Declare variables if requested.
if options.declare
   ynames = regexprep(ynames,'\{.*\}','');
   decl = sprintf('@variables:transition%s\n@variables:residual%s\n@equations:transition\n',sprintf(' %s',ynames{:}),sprintf(' %s',options.enames{:}));
   code = [decl,code];
end

   function x = printparameter_(matrix,position,value)
      if options.hardparameters
         x = sprintf(options.format,value);
      else
         if p <= 1 && numel(position) == 3
            position = position(1:2);
         end
         x = [matrix,sprintf(pformat,position{:})];
         d.(x) = value;
         x = ['+',x];
      end
   end

end
% End of primary function.