function [m,assign] = model(p,m,options)
% MODEL  Parse model code file.

% The IRIS Toolbox.
% Copyright 2007-2010 Jaromir Benes.

%**************************************************************************
% Function body.

assign = p.params;
m.fname = p.fname;

% Linear or non-linear model.
% First, check for the !linear keyword in the model code.
[m.linear,p.code] = strfun.findremove(p.code,'!linear');
% However, if the user have specified the 'linear' option in the |model|
% function, use that.
if ~isempty(options.linear)
   m.linear = options.linear;
end

block = irisparser.model_readblocks(p.code,p.fname);

% Untyped shocks will not be supported in future versions.
if ~isempty(block{3})
   irisparser.warning(2,m.fname);
end

%**************************************************************************
% Variables and parameters.


% Read the names of the variables, shocks, and parameters. 
[m.name,m.nametype,m.shocktype,m.namelabel,value] = ...
   irisparser.model_readnames(block);

% Evaluate values assigned in the model code and/or in the 'assign'
% database. Evaluate parameters first so that they are available for
% steady-state expressions.
value = regexprep(value,'(\<[A-Za-z]\w*\>)(?!\()','assign.$1');
for i = 4 : -1 : 1
   index = m.nametype == i ...
      & ~cellfun(@isempty,value) ...
      & ~isfield(assign,m.name);
   % Assign a value from declaration if not in the input database.      
   for j = find(index)
      try
         temp = eval(value{j});
         if isnumeric(temp) && length(temp) == 1
            assign.(m.name{j}) = temp;
         end
      catch
         assign.(m.name{j}) = NaN;
      end
   end
end

% Remove user-declared stdevs from list of params.
ne = sum(m.nametype == 3);
stdnames = strrep('std_$','$',m.name(m.nametype == 3));
index = findnames(m.name(m.nametype == 4),stdnames);
index(isnan(index)) = [];
if ~isempty(index)
   offset = sum(m.nametype < 4);
   m.name(offset+index) = [];
   m.namelabel(offset+index) = [];
   m.nametype(offset+index) = [];
end
% Add stdevs at the end of the list of params.
m.name(end+(1:ne)) = stdnames;
m.nametype(end+(1:ne)) = 4;
% Create labels for stdev parameters.
stdlabels = m.namelabel(m.nametype == 3);
nonempty = ~cellfun(@isempty,stdlabels);
stdlabels(nonempty) = stdlabels(nonempty);
stdlabels(nonempty) = strrep('Std of $','$',stdlabels(nonempty));
m.namelabel(end+(1:ne)) = stdlabels;

% Read substitutions.
[subsname,subsbody] = irisparser.model_readsubs(block);
nsubs = length(subsname);

chklist = [m.name,subsname];

% Leading characters of names and substitutions must not be 0-9 or _.
patt = '^[0-9_]';
match = regexp(chklist,patt,'match');
try
   aux = ~cellfun(@isempty,match);
catch
   aux = ~cellfun('isempty',match);
end
if any(aux)
   % Invalid variable or parameter names.
   irisparser.error(2,m.fname,chklist(aux));
end

% Check for multiple declarations of names and substitutions.
chklist{end+1} = 'ttrend';
[check,index] = sort(chklist);
aux = all(diff(double(char(check)),1,1) == 0,2);
if any(aux)
   % Multiple declarations.
   irisparser.error(3,m.fname,chklist(sort(index(aux))));
end

% Read log declarations.
m.log = false(size(m.name));
% Determine default log status: log == false unless !allbut is found.
[logdefault,block{5}] = strfun.findremove(block{5},'!allbut');
m.log(m.nametype == 1 | m.nametype == 2) = logdefault;
xylist = m.name(m.nametype == 1 | m.nametype == 2);
% Find variables matching any regular expression \<...\> found in the
% section and adjust their status. Remove the regular expressions from the
% section.
[index,block{5}] = logregexp_(block{5},xylist);
m.log(index) = ~logdefault;
% Cycle over all names found in the section and try to match them with the
% names of the measurement or transition variables.
logname = irisparser.model_findnames(block{5});
logname = unique(logname);
invalid = cell([1,0]);
for i = 1 : length(logname)
   aux = strmatch(logname{i},xylist,'exact');
   if ~isempty(aux)
      m.log(aux) = ~logdefault;
   else
      invalid{end+1} = logname{i};
   end
end

if ~isempty(invalid)
   % Invalid names in log declaration.
   irisparser.error(4,m.fname,invalid);
end

% Add substitutions for parameters (hard number strings).
chk = [block{6:7}];
for i = find(m.nametype == 4)
   if isempty(strfind(chk,['$',m.name{i},'$']))
      continue
   end
   subsname{end+1} = m.name{i};
   try
      tmp = assign.(m.name{i})(1);
   catch
      tmp = NaN;
   end
   subsbody{end+1} = sprintf('%.16g',tmp);
end

maxCharCode = 1999;
namecode = char(maxCharCode+(1:length(m.name)));
subscode = char(maxCharCode+length(m.name)+(1:length(subsname)));

%**************************************************************************
% Reporting equations.

p1 = p;
p1.code = block{10};
m.outside = p1.reporting();

%**************************************************************************
% Equations:
% measurement and transition equations & deterministic trends & links.

% Remove extra ;s from equation-like blocks.
patt = '(\s*;){2,}';
block([6,7,9,13]) = regexprep(block([6,7,9,13]),patt,';');
      
% Read measurement equations.
[eqtn,eqtnF,eqtnS,eqtnlabel] = readeqtns_(block{6});
n = length(eqtn);
m.eqtn(end+(1:n)) = eqtn;
m.eqtnF(end+(1:n)) = eqtnF;
if ~m.linear
   m.eqtnS(end+(1:n)) = eqtnS;
end
m.eqtnlabel(end+(1:n)) = eqtnlabel;
m.eqtntype(end+(1:n)) = 1;

% Read transition equations.
[eqtn,eqtnF,eqtnS,eqtnlabel] = readeqtns_(block{7});
n = length(eqtn);
m.eqtn(end+(1:n)) = eqtn;
m.eqtnF(end+(1:n)) = eqtnF;
if ~m.linear
   m.eqtnS(end+(1:n)) = eqtnS;
end
m.eqtnlabel(end+(1:n)) = eqtnlabel;
m.eqtntype(end+(1:n)) = 2;

% Read individual deterministic regressors.
% Add them to m.eqtn and m.eqtnF.
m = readdtrends_(m,block{9});

% Read dynamic links.
% Add them to m.eqtn and m.eqtnF.
m = readlinks_(m,block{13});

% Remove tag marks #(xx) from equations.
m.eqtn = regexprep(m.eqtn,'^\s*#\(\d+\)\s*','');

% Check the steady-state references & and &&.
index = ~cellfun(@isempty,strfind(m.eqtn,'&&')) ...
   | ~cellfun(@isempty,strfind(m.eqtn,'&'));
% Not allowed in linear models.
if m.linear
   temp = index & m.eqtntype <= 2;
   if any(temp)
      irisparser.error(74,m.fname,m.eqtn(temp));
   end
end
% Not allowed in deterministic trends.
temp = index & m.eqtntype == 3;
if any(temp)
   irisparser.error(75,m.fname,m.eqtn(temp));
end
% Not allowed in dynamic links.
temp = index & m.eqtntype == 4;
if any(temp)
   irisparser.error(76,m.fname,m.eqtn(temp));
end

% Remove ! from math functions.
% This is for bkw compatibility only.
m.eqtnF = strrep(m.eqtnF,'!','');
if ~m.linear
   m.eqtnS = strrep(m.eqtnS,'!','');
end

% Replace [] with ().
m.eqtnF = strrep(m.eqtnF,'[','(');
m.eqtnF = strrep(m.eqtnF,']',')');
if ~m.linear
   m.eqtnS = strrep(m.eqtnS,'[','(');
   m.eqtnS = strrep(m.eqtnS,']',')');
end

% Remove blank spaces.
m.eqtn = regexprep(m.eqtn,{'\s+','".*?"'},{'',''});
m.eqtnF = regexprep(m.eqtnF,'\s+','');
if ~m.linear
   m.eqtnS = regexprep(m.eqtnS,'\s+','');
end
m.eqtnlabel = strrep(m.eqtnlabel,'"','');

% Replace names with code characters.
% Substitutions.
nname = length(m.name);
patt = cell([1,nsubs+nname]);
repl = cell([1,nsubs+nname]);
for i = 1 : nsubs
   patt{i} = ['\$',subsname{i},'(\{[^\}]*\})?\$'];
   repl{i} = ['\$',subscode(i),'$1','\$'];
end
% Variables and parameters.
len = cellfun(@length,m.name);
[aux,index] = sort(len,2,'descend');
for i = index
   patt{nsubs+i} = ['\<',m.name{i},'\>'];
   repl{nsubs+i} = namecode(i);
end
subsbody = regexprep(subsbody,patt,repl);
m.eqtnF = regexprep(m.eqtnF,patt,repl);
if ~m.linear
   m.eqtnS = regexprep(m.eqtnS,patt,repl);
end

% Try to catch undeclared names at this point. They haven't been
% substituted for by the name codes.
chkundeclarednames_();

% Expand substitutions.
% First, expand substitutions that have no time subscripts.
patt = cell([1,nsubs]);
for i = 1 : nsubs
   patt{i} = ['\$',subscode(i),'\$'];
end
subsbody = regexprep(subsbody,patt,subsbody);
m.eqtnF = regexprep(m.eqtnF,patt,subsbody);
if ~m.linear
   m.eqtnS = regexprep(m.eqtnS,patt,subsbody);
end

invalid = {};
maxt = 0;
mint = 0;
key = cell(size(m.eqtnF));

% Evaluate time subscripts {...} or {...}{...}, and replace them with
% (...).
   function c = evalsubs_(cc)
      c = strrep(cc,'}{',')+(');
      c = strrep(c,'{','(');
      c = strrep(c,'}',')');
      if strcmp(c,'(0)')
         c = '';
         return
      end
      tmp = sscanf(c(2:end-1),'%g');
      if length(tmp) == 1 && isnumeric(tmp) && ~isnan(tmp)
         tmp = round(tmp);
         if tmp == 0
            c = '';
         else
            c = sprintf('(%+g)',tmp);
            maxt = max([maxt,tmp]);
            mint = min([mint,tmp]);
         end
         return
      end
      tmp = eval(c);
      if length(tmp) == 1 && isnumeric(tmp) && ~isnan(tmp)
         tmp = round(tmp);
         if tmp == 0
            c = '';
         else
            c = sprintf('(%+g)',tmp);
            maxt = max([maxt,tmp]);
            mint = min([mint,tmp]);
         end
         return
      end
      invalid{end+1} = cc;
      c = '';
   end
replace = @evalsubs_;
m.eqtnF = regexprep(m.eqtnF,'(\{[^\}\{;]*\})+','${replace($0)}');
m.eqtnS = regexprep(m.eqtnS,'(\{[^\}\{;]*\})+','${replace($0)}');
if ~isempty(invalid)
   % Error evaluating time subscripts.
   irisparser.error(9,m.fname,invalid);
end

% Copy full equations where steady-state equations are not supplied.
if ~m.linear
   index = cellfun(@isempty,m.eqtnS);
   m.eqtnS(index) = m.eqtnF(index);
end

% Replace variable codes with x(...).
   function c = transformF_(c0,c1,c2,ieq)
      % Variable or parameter number.
      realid = double(c1) - maxCharCode;
      if realid < 0
         % Undeclared name. Will be captured later.
         c = [c1,c2];
         return
      end
      if isempty(c2)
         t = 0;
      else
         t = sscanf(c2,'(%g)');
      end
      if m.nametype(realid) <= 2
         % Measurement and transition variables.
         if t == 0
            time = 't';
         else
            time = sprintf('t%+g',t);
         end
         if isempty(c0)
            c = sprintf('x(:,%g,%s)',realid,time);
            key{ieq}(end+1) = realid + 1i*t;
         else
            if strcmp(c0,'&')
               % Stead-state level reference.
               c = sprintf('L(:,%g,%s)',realid,time);
            elseif strcmp(c0,'&&')
               % Steady-state growth reference.
               c = sprintf('G(:,%g,t)',realid);
            end
         end
      elseif m.nametype(realid) == 3
         % Shocks.
         if isempty(c0)
            c = sprintf('x(:,%g,t)',realid);
            key{ieq}(end+1) = realid + 1i*t;
         else
            c = '0';
         end
      else
         % Parameters.
         c = sprintf('x(:,%g,t)',realid);
         key{ieq}(end+1) = realid + 1i*t;
      end
   end
   function c = transformS_(c1,c2)
      % Variable or parameter number.
      realid = double(c1) - maxCharCode;
      if realid < 0
         % Undeclared name. Will be captured later.
         c = [c1,c2];
         return
      end
      if isempty(c2)
         t = 0;
      else
         t = sscanf(c2,'(%g)');
      end
      switch m.nametype(realid)
         case 1
            % Measurement variables.
            c = sprintf('x(%g)',realid);
         case 2
            % Transition variables.
            if t == 0
               c = sprintf('x(%g)',realid);
            else
               c = sprintf('(x(%g)+(%g)*dx(%g))',realid,t,realid);
            end
         case 3
            % Shocks.
            c = '0';
         case 4
            % Parameters.
            c = sprintf('x(%g)',realid);
      end
      if m.log(realid)
         c = ['exp(',c,')'];
      end      
   end
% We need to capture the equation number ieq, use therefore a for loop, and
% not a single |regexprep| command.
replaceF = @transformF_;
replaceS = @transformS_;
pattern = ['(&&?)?([',namecode(1),'-',namecode(end),'])','(\([\+\-]\d+\))?'];
for ieq = 1 : numel(m.eqtnF)
   m.eqtnF{ieq} = regexprep(m.eqtnF{ieq},pattern, ...
      '${replaceF($1,$2,$3,ieq)}');
   if ~m.linear && m.eqtntype(ieq) <= 2
      m.eqtnS{ieq} = regexprep(m.eqtnS{ieq},pattern, ...
         '${replaceS($2,$3)}');
   end
end

maxt = maxt + 1;
mint = mint - 1;
t = 1 - mint;
m.tzero = t;
nt = maxt - mint + 1;

occur = false([length(m.eqtnF),length(m.name),nt]);
for i = 1 : length(m.eqtnF)
   index = sub2ind(size(occur),i*ones(size(key{i})),real(key{i}),t+imag(key{i}));
   occur(index) = true;
end

% Fold full 3d [nrow,ncol,npage] into sparse 2d [nrow,ncol*npage].
m.occur = sparse(occur(:,:));

% Replace matrix operators with scalar operators.
m.eqtnF = strrep(m.eqtnF,'*','.*');
m.eqtnF = strrep(m.eqtnF,'/','./');
m.eqtnF = strrep(m.eqtnF,'^','.^');

% Replace #xx with labels.
m.namelabel = p.labelstore(m.namelabel);
m.eqtnlabel = p.labelstore(m.eqtnlabel);

chksyntax_(m,nt);
chkstructure_();

% Determine types of the shocks if untyped in the model code. The procedure
% assumes that each shocks does not occur at the same time in both the
% measurement equations and transition equations.
if any(isnan(m.shocktype))
   m = shocktype_(m);
end
% TODO: Set shocktype to NaN for shocks that do not effectively occur in
% any of the model equations.

% End of function body.





   %***********************************************************************
   % Nested function chkstructure_().
   function chkstructure_()
      
      ixname = cell([1,4]);
      for ii = 1 : 4
         ixname{ii} = find(m.nametype == ii);
      end
      ixeqtn = cell([1,3]);
      for ii = 1 : 3
         ixeqtn{ii} = find(m.eqtntype == ii);
      end
      ny = length(ixname{1});
      nx = length(ixname{2});
      ne = length(ixname{3});
      np = length(ixname{4});
      
      % one transition variable at least
      if ~any(m.nametype == 2)
         irisparser.error(6,m.fname,{},'No transition variable.');
      end
      
      % current dates of all transition variables
      aux = ~any(occur(ixeqtn{2},ixname{2},t),1);
      if any(aux)
         irisparser.error(6,m.fname,m.name(ixname{2}(aux)),'No current date of this transition variable: ''%s''.');
      end
      
      % current dates of all measurement variables
      aux = ~any(occur(ixeqtn{1},ixname{1},t),1);
      if any(aux)
         irisparser.error(6,m.fname,m.name(ixname{1}(aux)),'No current date of this measurement variable: ''%s''.');
      end
      
      % # measurement equations = # measurement variables
      if sum(m.nametype == 1) ~= sum(m.eqtntype == 1)
         irisparser.error(6,m.fname,{sum(m.eqtntype == 1),sum(m.nametype == 1)},'%g measurement equation(s) for %g measurement variable(s).');
      end
      
      % # transition equations = # transition variables
      if sum(m.nametype == 2) ~= sum(m.eqtntype == 2)
         irisparser.error(6,m.fname,{sum(m.eqtntype == 2),sum(m.nametype == 2)},'%g transition equation(s) for %g transition variable(s).');
      end
      
      % no lags/leads of measurement variables
      aux = any(any(occur(:,ixname{1},[1:t-1,t+1:end]),3),1);
      if any(aux)
         irisparser.error(6,m.fname,m.name(ixname{1}(aux)),'Lag(s) or lead(s) of this measurement variable: ''%s''.');
      end
      
      % no lags/leads of shocks
      aux = any(any(occur(:,ixname{3},[1:t-1,t+1:end]),3),1);
      if any(aux)
         irisparser.error(6,m.fname,m.name(ixname{3}(aux)),'This shock has lag(s) or lead(s): ''%s''.');
      end
      
      % no lags/leads of parameters
      aux = any(any(occur(:,ixname{4},[1:t-1,t+1:end]),3),1);
      if any(aux)
         irisparser.error(6,m.fname,m.name(ixname{4}(aux)),'This parameter has lag(s) or lead(s): ''%s''.');
      end
      
      % no measurement variables in transition equations
      aux = any(any(occur(ixeqtn{2},ixname{1},:),3),2);
      if any(aux)
         irisparser.error(6,m.fname,m.eqtn(ixeqtn{2}(aux)),'This transition equation has measurement variable(s): ''%s''.');
      end
      
      % no leads of transition variables in measurement equations
      aux = any(any(occur(ixeqtn{1},ixname{2},t+1:end),3),2);
      if any(aux)
         irisparser.error(6,m.fname,m.eqtn(ixeqtn{1}(aux)),'Lead(s) of transition variable(s) in this measurement equation: ''%s''.');
      end
      
      % current date of any transition variable in each transition equation
      aux = ~any(occur(ixeqtn{2},ixname{2},t),2);
      if any(aux)
         irisparser.error(6,m.fname,m.eqtn(ixeqtn{2}(aux)),'No current-dated transition variable in this transition equation: ''%s''.');
      end
      
      % Current date of any measurement variable in each measurement equation.
      aux = ~any(occur(ixeqtn{1},ixname{1},t),2);
      if any(aux)
         irisparser.error(6,m.fname,m.eqtn(ixeqtn{1}(aux)),'No current-dated measurement variables in this measurement equation: ''%s''.');
      end
      
      if ne > 0
         % Find index of shocks in measurement equations.
         aux1 = any(occur(ixeqtn{1},ixname{3},t),1);
         % Find index of shocks in transition equations.
         aux2 = any(occur(ixeqtn{2},ixname{3},t),1);
         % No measurement shock in transition equations.
         aux = aux2 & m.shocktype == 1;
         if any(aux)
            irisparser.error(6,m.fname,m.name(ixname{3}(aux)), ...
               'This measurement shock occurs in transition equation(s): ''%s''.');
         end
         % No transition shock in measurement equations.
         aux = aux1 & m.shocktype == 2;
         if any(aux)
            irisparser.error(6,m.fname,m.name(ixname{3}(aux)), ...
               'This transition shock occurs in measurement equation(s): ''%s''.');
         end
         % No non-linear shock in measurement equations.
         aux = aux1 & m.shocktype == 0;
         if any(aux)
            irisparser.error(6,m.fname,m.name(ixname{3}(aux)), ...
               'This non-linear shock occurs in measurement equation(s): ''%s''.');
         end
         % Non-linear shocks.
         if any(m.shocktype == 0)
            aux0 = occur(ixeqtn{2},ixname{3},t);
            aux0(:,m.shocktype ~= 0) = false;
            % At most one non-linear shock in each transition equation.
            index = sum(aux0,2) > 1;
            if any(index,1)
               irisparser.error(6,m.fname,m.eqtn(ixeqtn{2}(index)), ...
                  'More than one non-linear shock in this equation: ''%s''.');
            end
            % Each non-linear shock in no more than one transition equation.
            index = sum(aux0,1) > 1;
            if any(index,2)
               irisparser.error(6,m.fname,m.name(ixname{3}(index)), ...
                  'This non-linear shock occurs in more than one equation: ''%s''.');
            end
         end
         % No shock simultaneously in transition and measurement equations.
         % This is for bkw compatibility when untyped shocks are declared.
         aux = all([aux1;aux2],1);
         if any(aux)
            irisparser.error(6,m.fname,m.name(ixname{4}(aux)), ...
               'This shock occurs in both measurement and transition equation(s): ''%s''.');
         end
      end
      
      % Only parameters can occur in deterministic trend equations.
      aux = any(any(occur(:,m.nametype ~= 4,:),3),2);
      aux = aux(:)';
      aux = aux & m.eqtntype == 3;
      if any(aux)
         irisparser.error(6,m.fname,m.eqtn(aux), ...
            'This deterministic trend refers to name(s) other than parameters: ''%s''.');
      end
      
   end
   % End of nested function chkstructure_().





   %***********************************************************************
   % Nested function undeclarednames_().
   function chkundeclarednames_()
      % Undeclared names have not been substituted for by the name codes.
      % Distinguish variable names from function names (func names are
      % immediately followed by an opening bracket). Note that |regexp|
      % unfortunately interprets high char codes as \w, so we need to
      % explicitly type the ranges.
      undeclaredNames = regexp(m.eqtnF, ...
         '\<[a-zA-Z][a-zA-Z0-9_]*\>(?!\()','match');
      if ~isempty([undeclaredNames{:}])
         index = ~cellfun(@isempty,undeclaredNames);
         list = {};
         for i = find(index)
            for j = 1 : length(undeclaredNames{i})
               list{end+1} = undeclaredNames{i}{j};
               list{end+1} = m.eqtn{i};
            end
         end
         irisparser.error(38,m.fname,list);
      end
   end
   % End of nested function chkundeclarednames_().





end
% End of primary function.





%**************************************************************************
% Subfunction chksyntax_().
function chksyntax_(m__,nt__) 
   t = m__.tzero;
   x = rand([1,length(m__.name),nt__]);
   L = x;
   G = zeros([1,length(m__.name),nt__]);
   G(1,m__.log,:) = 1;
   dx = zeros([1,length(m__.name),nt__]);
   ttrend = 0;
   undeclared__ = {};
   syntax__ = {};
   for eq__ = 1 : length(m__.eqtn)
      try
         eqtn__ = m__.eqtnF{eq__};
         eval(eqtn__);
         if eq__ <= length(m__.eqtnS)
            eqtn__ = m__.eqtnS{eq__};
            eval(eqtn__);
         end
      catch E__
         % Undeclared names should haave been already captured. But a few
         % exceptions may persist.
         [match__,tokens__] = ...
            regexp(E__.message,'Undefined function or variable ''(\w*)''','match','tokens','once');
         if ~isempty(match__)
            undeclared__{end+1} = tokens__{1};
            undeclared__{end+1} = m__.eqtn{eq__};
         else
            syntax__{end+1} = m__.eqtn{eq__};
            if E__.message ~= '.'
               E__.message(end+1) = '.';
            end
            syntax__{end+1} = sprintf('Matlab says: %s',E__.message);
         end
      end
   end
   if ~isempty(undeclared__)
      irisparser.error(38,m__.fname,undeclared__);
   end
   if ~isempty(syntax__)
      irisparser.error(8,m__.fname,syntax__);
   end
end
% End of subfunction chksyntax_().





%**************************************************************************
% Subfunction readeqtns_().
% Read measurement or transition equations.
function [eqtn,eqtnF,eqtnS,eqtnlabel] = readeqtns_(thisblock)
   % Regular expressions to match model code elements.
   thisblock = regexprep(thisblock,'\s+','');
   pattern = '(#\(\d+\))?([^;]+);';
   [eqtn,tokens] = regexp(thisblock,pattern,'match','tokens');
   if isempty(eqtn)
      eqtnlabel = {};
      eqtnF = {};
      eqtnS = {};
      return
   end
   tokens = [tokens{:}];
   eqtnlabel = tokens(1:2:end);
   % Pull out full and steady-state versions.   
   eqtnF = regexp(tokens(2:2:end),'^[^!]+','match','once');
   eqtnS = regexp(tokens(2:2:end),'(?<=!!).+$','match','once');
   % Replace LHS=RHS with LHS-(RHS); and LHS; with LHS-();
   patt = '^([^=]+)(=)?(.+)?$';
   repl = '$1-($3);';
   eqtnF = regexprep(eqtnF,patt,repl);
   eqtnF = strrep(eqtnF,'-()','');
   index = ~cellfun(@isempty,eqtnS);
   if any(index)
      eqtnS(index) = regexprep(eqtnS(index),patt,repl);
      eqtnS(index) = strrep(eqtnS(index),'-()','');
   end
end
% End of subfunction readeqtns_().





%**************************************************************************
% Subfunction readdtrends_().
function m = readdtrends_(m,thisblock)
   n = sum(m.nametype == 1);
   eqtn = emptycellstr([1,n]);
   eqtnF = emptycellstr([1,n]);
   eqtnlabel = emptycellstr([1,n]);
   % Remove ! from dtrend equations.
   thisblock = strrep(thisblock,'!','');
   pattern = '\s*(#\(\d+\))?\s*([\(\)\w]*)\s*([\+\*])=\s*([^";]*?;)';
   [match,tokens] = regexp(thisblock,pattern,'match','tokens');
   match = strtrim(match);
   match = regexprep(match,'#\(\d+\)','');
   % Replace x *= ...; with log(x) += log(...);
   for i = 1 : length(tokens)
      tokens{i}{2} = strtrim(tokens{i}{2});
      tokens{i}{4} = strtrim(tokens{i}{4});
      if strcmp(tokens{i}{3},'*')
          tokens{i}{2} = sprintf('log(%s)',tokens{i}{2});
          tokens{i}{4} = sprintf('log(%s);',tokens{i}{4}(1:end-1));
      end
   end
   % Create list of measurement variable names.
   % Add log(...) for log variables.
   list = m.name(m.nametype == 1);
   aux = m.log(m.nametype == 1);
   list(aux) = regexprep(list(aux),'(.*)','log($1)','once');
   invalid = {};
   multiple = {};
   nmatch = length(match);
   for i = 1 : nmatch
      index = find(strcmp(list,tokens{i}{2}),1);
      if isempty(index)
         invalid{end+1} = match{i};
         continue;
      end
      if ~isempty(eqtn{index})
         multiple{end+1} = m.name{index};
         continue;
      end
      eqtn{index} = match{i};
      eqtnF{index} = tokens{i}{4};
      eqtnlabel{index} = tokens{i}{1};
   end
   if ~isempty(invalid)
      invalid = regexprep(invalid,'\s+','');
      irisparser.error(55,m.fname,invalid);
   end
   if ~isempty(multiple)
      multiple = unique(multiple);
      irisparser.error(6,m.fname,multiple,'Multiple deterministic trend equations for ''%s''.');
   end
   m.eqtn(end+(1:n)) = eqtn;
   m.eqtnF(end+(1:n)) = eqtnF;
   m.eqtnlabel(end+(1:n)) = eqtnlabel;
   m.eqtntype(end+(1:n)) = 3;
end
% End of subfunction readdtrends_().





%**************************************************************************
% Subfunction readlinks_().

function m = readlinks_(m,thisblock)

   n = sum(m.nametype <= 4);
   eqtn = emptycellstr([1,n]);
   eqtnF = emptycellstr([1,n]);
   eqtnlabel = emptycellstr([1,n]);
   pattern = '\s*(#\(\d+\))?([^;]*?):?(=[^;]*?)?;';
   [match,tokens] = regexp(thisblock,pattern,'match','tokens');
   match = strtrim(match);
   match = regexprep(match,'#\(\d+\)','');
   
   nmatch = length(match);

   invalid = {};
   refresh = [];
   for j = 1 : nmatch
      tokens{j} = strtrim(tokens{j});
      if isempty(tokens{j}{3})
         invalid{end+1} = match{j};
         continue
      end
      lhs = strtrim(tokens{j}{2});
      index = strcmp(m.name(m.nametype <= 4),lhs);
      if ~any(index)
         invalid{end+1} = match{j};
         continue
      end
      eqtn{index} = match{j};
      eqtnF{index} = sprintf('%s;',strtrim(tokens{j}{3}(2:end)));
      eqtnlabel{index} = tokens{j}{1};
      refresh(end+1) = find(index);
   end
   
   if ~isempty(invalid)
      invalid = regexprep(invalid,'\s+','');
      irisparser.error(67,m.fname,invalid);
   end
   
   % Replace all time subscripts with 0.
   eqtnF = regexprep(eqtnF,'\{.*?\}','{0}');
   
   m.eqtn(end+(1:n)) = eqtn;
   m.eqtnF(end+(1:n)) = eqtnF;
   m.eqtnlabel(end+(1:n)) = eqtnlabel;
   m.eqtntype(end+(1:n)) = 4;
   m.refresh = refresh;

end
% End of subfunction readlinks_().





%**************************************************************************
% Subfunction logregexp_().
function [index,block] = logregexp_(block,xylist)
   function s = replace(pattern)
      s = '';
      index = index | strfun.matchindex(xylist,pattern);
   end
   index = false(size(xylist));
   replacefunc = @replace;
   block = regexprep(block,'\\<(.*?)\\>','${replacefunc($1)}');
end