function [this,labels] = readcode(filelist,assign,labels,fname)
   % Read and pre-parse code files.

   % The IRIS Toolbox.
   % Copyright 2007-2010 Jaromir Benes.

%**************************************************************************
% Function body.

this = '';
if ischar(filelist)
   this = file2char(filelist);
elseif iscellstr(filelist)
   for i = 1 : numel(filelist)
      this = [this,sprintf('\n'),file2char(filelist{i})];
   end
end

% Convert end of lines.
this = strfun.converteols(this);

% Read tags/labels 'xxx' and "xxx" and replace them with #(00).
% The tags/labels may not stretch over more than one line.
[this,labels] = readlabels_(this,labels);

% Remove standard line and block comments.
this = removecomments(this);

% Remove !!!.
% This mark is meant to be used to highlight some bits of the code.
this = strrep(this,'!!!','');

% Characters beyond char(highcharcode) not allowed except comments.
% Default is 1999.
highCharCode = irisget('highcharcode');
if any(this > char(highCharCode))
   irisparser.error(68,fname,{},highCharCode);
end

% Replace @keywords with !keywords. This is for backward compatibility
% only.
this = prefix_(this);

% Discard everything after !stop.
index = strfind(this,'!stop');
if ~isempty(index)
  this = this(1:index(1)-1);
end

% Replace !ifversion with !if.
this = strrep(this,'!ifversion','!if');

% Evaluate
% !if .. !else .. !end
% !for .. !do .. !end
% !switch .. !case ... !otherwise .. !end
% Create m-file functions
% !function .. !end
invalid = cell([1,0]);
patterns = {...
  '!if([^!if!for!switch!function]*?)!end', ...
  '!for([^!if!for!switch!function]*?)!end', ...
  '!switch([^!if!for!switch!function]*?)!end', ...
  '!function([^!if!for!switch!function]*?)!end'};
patt = convert_(patterns,'>');
this = convert_(this,'>');
[type,body,start,finish] = findblock_(this,patt);
nl = sprintf('\n');
while type ~= 0
   switch type
      case 1
         [replace,flag] = ifblock_(body,assign,labels);
      case 2
         [replace,flag] = forblock_(body,assign,labels);
      case 3
         [replace,flag] = switchblock_(body,assign,labels);
      case 4
         [replace,flag] = function_(body,assign,labels,fname);
   end
   if ~flag 
      % Something is wrong inside the body of the command.
      type = regexp(patterns{type},'^!\w+','match','once');
      body = labelsback_(body,labels);
      command = regexprep([type,' ',body,' !end'],'\s+',' ');
      irisparser.error(10,fname,command,type);
   end
   replace = [nl,replace,nl];
   this = [this(1:start-1),replace,this(finish+1:end)];   
   [type,body,start,finish] = findblock_(this,patt);
end
[this,residual] = convert_(this,'<');
if ~isempty(residual)
   irisparser.error(69,fname,residual);
end

% Import/include/input external files.
[this,labels] = import_(this,labels,assign);

% Handle multiple or obsolete syntax options.
this = multiple_(this);

end
% End of primary function.

%**************************************************************************
% Subfunction convert_().
% Replace
%    !if, !else, !for, !do, !end, !switch, !case, !otherwise
% with char(offset+1...)
function [this,residual] = convert_(this,direction)
   list = {'end','if','else','for','do','switch','case','otherwise','function'};
   offset = irisget('highcharcode');
   if direction == '>'
      for i = 1 : numel(list) 
         this = strrep(this,sprintf('!%s',list{i}),char(offset+i));
      end
   else
      residual = {};
      for i = 1 : numel(list) 
         n = numel(this);
         this = strrep(this,char(offset+i),sprintf('!%s',list{i}));
         if numel(this) > n
            residual{end+1} = ['!',list{i}];
         end
      end
   end
end
% End of subfunction convert_().

%**************************************************************************
% Subfunction findblock_().
% Find first block with no nested block inside.
function [type,body,start,finish] = findblock_(this,patt)
   type = 0;
   for i = 1 : length(patt)
       [tokens,start,finish] = ... 
          regexp(this,patt{i},'tokens','start','end','once');
      if ~isempty(tokens)
         type = i;
         break
      end
   end
   if type ~= 0 && ~isempty(tokens) && iscell(tokens)
      body = strtrim(tokens{1});
   else
      body = '';
   end
end
% End of subfunction findblock_().

%**************************************************************************
% Subfunction ifblock_().
function [replace,flag] = ifblock_(body,assign,labels)
   flag = true;
   pattern = '^([^\n;]+)[\n;]+(.*?)(!else)?(?(3).*|)$';
   pattern = convert_(pattern,'>');
   tokens = regexp(body,pattern,'once','tokens');
   if isempty(tokens)
      replace = '';
      flag = false;
      return
   end
   value = evalexpression_(assign,tokens{1},labels);
   replace = tokens{4};
   try
      if value
         replace = tokens{2};
      end
   end
end
% End of subfunction ifblock_().

%**************************************************************************
% Subfunction forblock_().
function [replace,flag] = forblock_(body,assign,labels)
   flag = true;
   pattern = '^\s*(\?\S*\s*=)?\s*(.*?)\s*!do\s(.*?)\s*$';
   pattern = convert_(pattern,'>');
   tokens = regexp(body,pattern,'once','tokens');
   if isempty(tokens)
      replace = '';
      flag = false;
      return
   end
   % Name of control variables. If not user-supplied, use ? by default.
   control = tokens{1};
   if ~isempty(control)
      control = regexp(control,'^\?\S*','match','once');
   else
      control = '?';
   end
   list = tokens{2};
   template = tokens{3};
   list = import_(list,assign);
   % Expand [1 : 10] or [2 : 2 : 10] in the list.
   if ~isempty(list)
      replacecolonfunc = @replacecolon;
      list = regexprep(list,'\s*:\s*',':');
      list = regexprep(list, ...
         '\[\s*(.*?:.*?)(:.*?)?\]', ...
         '${replacecolonfunc($1,$2)}');
   end
   list = charlist2cellstr(list);
   llist = lower(list);
   ulist = upper(list);
   replace = '';
   nl = sprintf('\n');
   for i = 1 : length(list)
      templatei = template;
      % These are the preferred options.
      templatei = strrep(templatei,['!lower',control],llist{i});
      templatei = strrep(templatei,['!upper',control],ulist{i});      
      % The following ones are for bkw compatibility only.
      templatei = strrep(templatei,['<lower(',control,')>'],llist{i});
      templatei = strrep(templatei,['<upper(',control,')>'],ulist{i});
      templatei = strrep(templatei,['lower(',control,')'],llist{i});
      templatei = strrep(templatei,['upper(',control,')'],ulist{i});
      templatei = strrep(templatei,['<-',control,'>'],llist{i});
      templatei = strrep(templatei,['<+',control,'>'],ulist{i});
      templatei = strrep(templatei,['<',control,'>'],list{i});
      % Finally, this is the plain control variable substitution.
      templatei = strrep(templatei,control,list{i});
      replace = [replace,templatei];
      if i < numel(list)
         replace = [replace,nl];
      end
   end
   % Nested function replace().
   function x = replacecolon(s1,s2)
      try
         x = eval([s1,s2]);
      catch
         x = '';
      end
      if ~isempty(x)
         if isnumeric(x) || islogical(x)
            x = sprintf('%g, ',x);
            x = x(1:end-2);
         end
      end     
   end
   % End of nested function replace().
end
% End of subfunction forblock_().

%**************************************************************************
% Subfunction function_().
function [replace,flag] = function_(body,assign,labels,fname)
flag = true;
% Find the name of the function in the header.
firstLine = regexp(body,'^.*?(\n|$)','once','match');
firstLine = strtrim(firstLine);
% Remove the first line from the body of the function.
body = regexprep(body,'^.*?(\n|$)','','once');
funName = regexp(firstLine,'=\s*(\w+)','once','tokens');
if isempty(funName) || isempty(funName{1})
   flag = false;
   return
end
funName = funName{1};
body = labelsback_(body,labels);
nl = sprintf('\n');
body = ['function ',firstLine,nl, ...
   '% Function based on |',fname,'|.',nl, ...
   '% Created ',datestr(now()),'.',nl, ...
   body,nl,'end'];
char2file(body,[funName,'.m']);
replace = '';
end
% End of subfunction function_().

%**************************************************************************
% Subfunction switchblock_().
function [replace,flag] = switchblock_(body,assign,labels)
   flag = true;
   controlpattern = '^[^!case!otherwise]*';
   controlpattern = convert_(controlpattern,'>');
   casepattern = '!case([^\n;]*?)[\n;]+(.*?)(?=!case|!otherwise|$)';
   casepattern = convert_(casepattern,'>');
   otherwisepattern = '(?<=!otherwise).*$';
   otherwisepattern = convert_(otherwisepattern,'>');
   control = regexp(body,controlpattern,'once','match');
   cases = regexp(body,casepattern,'tokens');
   other = regexp(body,otherwisepattern,'match','once');
   control = evalexpression_(assign,control,labels);
   matched = false;
   for i = 1 : numel(cases)
      tmpvalue = evalexpression_(assign,cases{i}{1},labels);
      if isequal(control,tmpvalue)
         replace = cases{i}{2};
         matched = true;
         break
      end
   end
   if ~matched
      replace = other;
   end
end
% End of subfunction switchblock_().

%**************************************************************************
% Subfunction evalexpression_().
% Evaluate the !if and !switch expressions within assign database.
function value = evalexpression_(assign,expr,labels) 
   expr = strtrim(expr);
   expr = strrep(expr,'!','');
   % Add |assign.| to all of its fields.
   list = fieldnames(assign)';
   for i = 1 : length(list)
     expr = regexprep(expr,['(?<![\.!])\<',list{i},'\>'],['?.',list{i}]);
   end
   expr = strrep(expr,'?.','assign.');
   % Put labels back because some of them can be actually strings in !if or
   % !switch expressions.
   expr = labelsback_(expr,labels);
   % Evaluate the expression.
   try
      value = eval(expr);
   catch
      value = NaN;
   end
end
% End of subfunction evalexpression_().

%**************************************************************************
% Subfunction prefix_().
% Replace @keywords with !keywords.
function this = prefix_(this) 
   this = regexprep(this,'@([a-z]\w*)','!$1');
end
% End of subfunction prefix_().

%**************************************************************************
% Subfunction multiple_().
% Handle multiple or obsolete syntax options.
function this = multiple_(this)
   this = strrep(this,'!coefficients','!parameters');
   this = strrep(this,'!variables:residual','!shocks');
   this = strrep(this,'!variables:innovation','!shocks');
   this = strrep(this,'!residuals','!shocks');
   this = strrep(this,'!outside','!equations:reporting');
   this = strrep(this,'!equations:dtrends','!dtrends:measurement');
   % Replace single quotes with double quotes.
   this = strrep(this,'''','"');
end
% End of subfunction mutliple_().

%**************************************************************************
% Subfunction import_().
% Import external file.
function [this,labels] = import_(this,labels,assign)
   % import/include/input files with replacement
   pattern = '(?:!include|!input|!import)\((.*?)\)';
   [tokens,start,finish] = regexp(this,pattern,'tokens','start','end','once');
   while ~isempty(tokens)
      fname = strtrim(tokens{1});
      if ~isempty(fname)
         [replace,labels] = irisparser.readcode(fname,assign,labels);
         this = [this(1:start-1),replace,this(finish+1:end)];
      end
      [tokens,start,finish] = regexp(this,pattern,'tokens','start','end','once');
   end
end
% End of subfunction import_().

%**************************************************************************
% Subfunction readlabels_().
function [this,labels] = readlabels_(this,labels)
      function out = processLabel_(text)
         labels{end+1} = text;
         out = sprintf('#(%g)',length(labels));
      end
   replace = @processLabel_;
   % '\1' cannot be used within [^...], therefore we cannot exclude the
   % opening quote (either apostrophe or double quote) from the string.
   this = regexprep(this,'([''"])([^\n]*?)\1','${replace($2)}');
end
% End of subfunction readlabels_().

%**************************************************************************
% Subfunction labelsback_().
function x = labelsback_(x,labels)
   % Nested function replace().
   function s = replace(x)
      s = '''''';
      x = sscanf(x,'#(%g)');
      if ~isempty(x) && ~isnan(x)
         s = ['''',labels{x},''''];
      end
   end
   % End of nested function replace().
   replacefunc = @replace;
   x = regexprep(x,'(#\(\d+\))','${replacefunc($1)}');
end
% End of subfunction labelsback_().