function s = sstate(p,s,varargin)

options = passvalopt('sstate.sstate',varargin{:});
s.growthnames = options.growthnames;

% Steady-state code file keywords:
% * !parameters or !input
% * !equations or !assignments
% * !solvefor
% * !variables:log
% * !allbut
% * !symbolic

code = p.code;

% Mutliple or obsolete syntax.
code = strrep(code,'!parameters','!input');
code = strrep(code,'!assignments','!equations');
code = strrep(code,'!assign','!equations');

% Process input block.
[tokens,finish] = regexp(code, ...
   '^\s*!input\s*(.*?)\s*(?=!equations|$)','tokens','end','once');
code = code(finish+1:end);
inputblock = tokens{1};
[allbut,logs,inputblock] = findallbutlogs(inputblock);
input = uniquelist(inputblock);

% Read equation blocks.
tokens = regexp(code, ...
   '\s*(!equations)\s*(#\(\d+\))?\s*(.*?)\s*(?=!equations|$)','tokens');
nblock = numel(tokens);
block = cell([1,nblock]);
for i = 1 : nblock
   s.label{i} = strtrim(labelstore(p,tokens{i}{2}));
   block{i} = tokens{i}{3};
end

% Process equation blocks.
% Read equations, variables, log variables, and methods.
invalidallbut = {};
for i = 1 : nblock
   % Check for !allbut and !variables:log.
   [s.allbut(i),s.logs{i},block{i},found] = findallbutlogs(block{i});
   % Add global log variables if !allbut are consistent in both
   % declarations.
   if ~isempty(logs)
      if ~found || allbut == s.allbut(i)
         s.logs{i} = [logs,s.logs{i}];
      else
         invalidallbut{end+1} = i;
      end
   end
   % Check for !symbolic.
   [tmpsymbolic,block{i}] = strfun.findremove(block{i},'!symbolic');
   % Check for !solvefor variables.
   [tmpsolvefor,start,finish] = regexp(block{i}, ...
      '!solvefor\s*(.*?)\s*(?=!|$)','tokens','once','start','end');
   block{i}(start:finish) = '';
   if isempty(tmpsolvefor)
      % No !solvefor found, this is an assignment block.
      s.type{i} = 'assignment';
   else
      if tmpsymbolic
         s.type{i} = 'symbolic';
      else
         s.type{i} = 'numerical';
      end
      % Variables to solve for.
      s.solvefor{i} = regexp([tmpsolvefor{:}],'\w+','match');
   end
   % Read positive/log variables.
   [s.logs{i},start,finish] = regexp(block{i}, ...
      '!variables:(log|positive)\s*(.*?)\s*(?=!|$)','tokens','once','start','end');
   logsfound = ~isempty(start);
   block{i}(start:finish) = '';
   if ~isempty(s.logs{i})
      s.logs{i} = uniquelist(s.logs{i});
   end
   % Add global log variables.
   if ~isempty(logs)
      if ~logsfound || allbut == s.allbut(i)
         s.logs{i} = [logs,s.logs{i}];
         s.allbut(i) = allbut;
      else
         invalidallbut{end+1} = i;
      end
   end
   % Read individual equations.
   block{i} = regexprep(block{i},'\s+','');
   block{i} = regexprep(block{i},';{2,}',';');
   s.eqtn{i} = regexp(block{i},'[^;]*?(?=;)','match');
   if strcmp(s.type{i},'assignment')
      % Find the LHS variables in assignments.
      s.solvefor{i} = regexp(s.eqtn{i},'^\s*(\w+)\s*(?==)','match','once');
      index = cellfun(@isempty,s.solvefor{i});
      s.solvefor{i}(index) = [];
   end
   % Create list of names that are already known before this block runs.
   if i == 1
      s.input{i} = input;
   else
      s.input{i} = [s.input{i-1},s.solvefor{i-1}];
   end
end

if ~isempty(invalidallbut)
   irisparser.error(70,p.fname,invalidallbut);
end

% Check multiple declarations of variables.
% Check number or equations and variables.
% Remove empty blocks.
invalid = {};
multiple = {};
emptyblocks = [];
for i = 1 : nblock
   if isempty(s.eqtn{i}) && isempty(s.solvefor{i})
      % Indices of empty blocks.
      emptyblocks(end+1) = i;
      continue
   end
   if strcmp(s.type{i},'assignment')
      continue
   end
   tmpsolvefor = s.solvefor{i};
   [ans,index] = unique(tmpsolvefor);
   if length(tmpsolvefor) ~= length(index)
      tmpsolvefor(index) = [];
      for j = 1 : numel(tmpsolvefor)
         multiple{end+1} = i;
         multiple{end+1} = tmpsolvefor{j};
      end
   end
   if length(s.eqtn{i}) ~= length(s.solvefor{i})
      % # equations does not match # variables.
      invalid{end+1} = length(s.eqtn{i});
      invalid{end+1} = length(s.solvefor{i});
      invalid{end+1} = i;
   end
end

% Multiple declaration.
if ~isempty(multiple)
   irisparser.error(71,p.fname,multiple);
end

% Number of equations and number of variables do not match.
if ~isempty(invalid)
   irisparser.error(72,p.fname,invalid);
end

% Remove empty blocks.
if ~isempty(emptyblocks)
   s.type(emptyblocks) = [];
   s.input(emptyblocks) = [];
   s.eqtn(emptyblocks) = [];
   s.solvefor(emptyblocks) = [];
   s.logs(emptyblocks) = [];
   nblock = numel(s.type);
end

% Check for presence of Symbolic Math Tbx reserved words.
reserved = {};
for i = 1 : nblock
   if strcmp(s.type{i},'symbolic')
      list = sstate.chkreserved(s.eqtn{i},s.solvefor{i});
      if ~isempty(list)
         reserved = [reserved,list];
      end
   end
end
if ~isempty(reserved)
   reserved = unique(reserved);
   irisparser.error(73,p.fname,reserved);
end

% Handle lags and leads.

   function x = replacetime(name,shiftstring)
      try
         shift = str2num(shiftstring);
      catch
         shift = [];
      end
      if ~isnumeric(shift) || isempty(shift) ...
            || isnan(shift) || isinf(shift) ...
            || shift ~= round(shift)
         invalidtime{end+1} = i;
         invalidtime{end+1} = [name,'{',shiftstring,'}'];
         x = '';
         return
      end
      if shift == 0
         x = name;
         return
      end
      if ischar(s.growthnames)
         gname = sprintf(s.growthnames,name);
      else
         gname = s.growthnames(name);
      end
      if shift < 0
         gnameshift = sprintf('%sm%g',gname,-shift);
      else
         gnameshift = sprintf('%sp%g',gname,shift);
      end
      if sstate.islog(name,s.logs{i},s.allbut(i))
         growthassign = sprintf('%s = %s.^(%g);',gnameshift,gname,shift);
         x = sprintf('(%s*%s)',name,gnameshift);
      else
         growthassign = sprintf('%s = %s*(%g);',gnameshift,gname,shift);
         x = sprintf('(%s+%s)',name,gnameshift);
      end
      if ~any(strcmp(growthassign,s.growth{i}))
         s.growth{i}{end+1} = growthassign;
      end
   end

replacetimefunc = @replacetime;
invalidtime = {};
for i = 1 : nblock
   s.growth{i} = {};
   s.eqtn{i} = regexprep(s.eqtn{i}, ...
      '(\<[a-zA-Z]\w*\>)\{(.*?)\}','${replacetimefunc($1,$2)}');
end

if ~isempty(invalidtime)
   error('iris:sstate', ...
      '\n*** Invalid time index in block #%g: ''%s''.',invalidtime{:});
end

end
% End of primary function.

%**************************************************************************
% Subfunction findallbutlogs().
function [allbut,logs,block,found] = findallbutlogs(block)
   % Check for !allbut.
   [allbut,block] = strfun.findremove(block,'!allbut');
   % Read log variables.
   [logs,start,finish] = regexp(block, ...
      '!variables:(?:log|positive)\s*(.*?)\s*(?=!|$)','tokens','once','start','end');
   block(start:finish) = '';
   found = ~isempty(start);
   if ~isempty(logs)
      logs = uniquelist(logs{1});
   end
end
% End of subfunction findallbutlogs().

%**************************************************************************
% Subfunction uniquelist().
function list = uniquelist(s)
   list = regexp(s,'\w+','match');
end
% End of subfunction uniquelist().