function [CC,RR,list] = acf(this,varargin)
% acf:
% /class_library/@model/acf.m
% ? Autocovariance and autocorrelation functions for model variables.
% 
% Syntax:
%   [C,R,list] = acf(m)
%   [C,R,list] = acf(m,...)
% 
% Input arguments:
% * |m| [ model ] Solved model object.
% 
% Output arguments:
% * |C| [ numeric ] Covariance function matrices.
% * |R| [ numeric ] Correlation function matrices.
% * |list| [ cellstr ] List of variables in rows and columns of |C| and |R|.
% 
% Options:
% * 'applyto' [ cellstr | char | *Inf* ] List of variables to which the
%     'filter' will be applied; the default Inf means all variables.
% * 'filter' [ char  | *empty* ] Linear filter that is applied to
%     variables specified by 'applyto'.
% * 'nfreq' [ numeric | *256* ] Number of equally spaced frequencies over
%     which the 'filter' is numerically integrated.
% * 'order' [ numeric | *0* ] Order up to which ACF will be computed.
% 
% Description:
% 
% The |<a href="acf.html">acf</a>| function returns the cross- and autocovariances for the model's
% measurement and transition variables, including their auxiliary lags and leads
% created for the model's first-difference representation.
% 
% The output matrices |C| and |R| are both N by N by (K+1) by P, where
% N is the total number of measurement and transition variables, including
% their auxiliary lags and leads, K is the order up to which the ACF is
% computed, and P is the number of alternative parameterisations in the model
% object.
% 
% The |C(i,j,k+1,p)| element is the covariance between the i-th variable
% at time t and the j-th variable at time t&minus;k in the n-th
% parameterisation; |R(i,j,k+1,n)| is the corresponding correlation
% coefficient. You can use the |<a href="../other-functions/select.html">select</a>|
% function to retrieve submatrices for only a subset of variables. The
% ordering of variables (and their lags and leads) in the rows and columns
% of the |C| and |R| matrices is given by |list|.
% 
% The 'filter' option allows you to compute the ACFs for the model's
% variables as if they were transformed by a linear filter. The filter is
% defined by a text string, and can be specified either in the time domain
% using a lag operator (represented by an 'L'), or in the frequency
% domain using 'freq' or 'per' to refer to frequencies or
% periodicities, respectively. The filter can be applied to all or just a
% selection of the model's variables specified by the 'applyto' option
% otherwise it is applied to all of them. The filtered ACFs are calculated
% using the Fourier transform, and then converted back by the inverse
% Fourier transform. The latter involves evaluating an integral, which is
% done numerically on the interval (0,&pi;) divided into an evenly
% spaced grid. The number of the points in the grid can be modified through
% the option 'nfreq'.
% 
% Example:
% 
% See also:
% |<a href="../other-functions/select.html">select</a>|
% 
% For more help on this function, type
%   <a href="matlab:idoc model.acf">idoc model.acf</a>

% The IRIS Toolbox.
% Copyright (c) 2007-2010 Jaromir Benes.
% End of header.

options = passvalopt('model.acf',varargin{:});

if ischar(options.applyto) && ~isempty(options.applyto)
   options.applyto = charlist2cellstr(options.applyto);
end

if ischar(options.select)
   options.select = charlist2cellstr(options.select);
elseif isempty(options.select)
   options.select = {''};
end

%**************************************************************************
% Function body.

nx = size(this.solution{1},1);
nalt = size(this.assign,3);
ne = sum(this.nametype == 3);
ny = sum(this.nametype == 1);

if options.contributions
   ncont = ne;
else
   ncont = 1;
end
CC = nan([ny+nx,ny+nx,options.order+1,ncont,nalt]);

% Linear filter applied to some variables.
if isnumeric(options.applyto) && any(isinf(options.applyto))
   applyto = true([1,ny+nx]);
elseif ~isempty(options.applyto)
   applyto = regexprep(options.applyto,'log\((.*?)\)','$1');
   applyto = regexprep(applyto,'\{.*?\}','');
   applyto = findnames(this.name(this.nametype <= 2),applyto);
   index = isnan(applyto);
   if any(index)
      warning_(1,options.applyto(index));
   end
   applyto(index) = [];
   aux = applyto;
   id = real([this.solutionid{1:2}]);
   applyto = false([1,ny+nx]);
   for i = aux(:)'
      applyto = applyto | id == i;
   end
else
   applyto = false([1,ny+nx]);
end

if ~isempty(options.filter) && any(applyto)
   isfilter = true;
   [frq,filter] = freq_domain.fdfilter(options.nfreq,options.filter);
   nfrq = lenght(frq);
else
   isfilter = false;
end

% Solution not available for some parameterisations.
[flag,nansolution] = isnan(this,'solution');
if flag
   warning_(44,sprintf(' #%g',find(nansolution)));
end

% Autocovariance function.
for ialt = 1 : nalt
   if nansolution(ialt)
      continue
   end
   [T,R,K,Z,H,D,U,Omega] = sspace_(this,ialt,false);
   for icont = 1 : ncont
      if options.contributions
         index = false([1,ne]);
         index(icont) = true;
         if Omega(index,index) == 0
            CC(:,:,:,icont,ialt) = 0;
            continue
         end
      else
         index = true([1,ne]);
      end
      if isfilter
         % We should multiply by 2*width == 2*pi/nfreq but we skip
         % dividing S by 2*pi in XSF and hence skip multiplying it by
         % 2*pi here.
         CC(:,:,:,icont,ialt) = real(freq_domain.xsf( ...
            T,R(:,index),K,Z,H(:,index),D,U,Omega(index,index),frq,filter,applyto,options.order)) / nfrq;
      else
         CC(:,:,:,icont,ialt) = time_domain.acovf( ...
            T,R(:,index),K,Z,H(:,index),D,U,Omega(index,index),this.eigval(1,:,ialt),options.order);
      end
   end
end

% Squeeze the covariance matrices if ~contributions.
if ~options.contributions
   CC = reshape(CC,[ny+nx,ny+nx,options.order+1,nalt]);
end

% Fix negative variances.
tmpsize = size(CC);
if length(tmpsize) < 4
   tmpsize(end+1:4) = 1;
end
CC0 = reshape(CC(:,:,1,:),[tmpsize(1:2),tmpsize(4)]);
CC0 = time_domain.fixcov(CC0);
CC(:,:,1,:) = reshape(CC0,[tmpsize(1:2),1,tmpsize(4)]);

% Autocorrelation function.
if nargout > 1
   % Convert covariances to correlations.
   RR = time_domain.cov2corr(CC);
end

% List of variables.
if nargout > 2 || iscellstr(options.select)
   list = [vector_(this,'y'),vector_(this,'x')];
end

% Select variables.
if iscellstr(options.select)
  [CC,index,list,notfound] = select(CC,list,options.select);
  if nargout > 1
    RR = RR(index,index,:,:);
  end
  if ~isempty(notfound)
    warning_(1,notfound);
  end
end

end
% End of primary function.