function varargout = datarequest(request,m,data,range)
% DATAREQUEST Request data from database or datapack.

% The IRIS Toolbox.
% Copyright 2007-2010 Jaromir Benes.

%**************************************************************************
% Function body.

% Input data can be either a database, a datapack, or a struct with mean_
% and mse_ datapacks.

if iscell(data)
   datatype = 'dpack';
elseif isstruct(data)
   if isfield(data,'mean_') && isfield(data,'mse_')
      datatype = 'struct';
   else
      datatype = 'dbase';
   end
else
   error('iris:model','Unknown type of input data.');
end

nx = size(m.solution{1},1);
nb = size(m.solution{7},1);
nf = nx - nb;
nalt = size(m.assign,3);

switch request
   case 'init'
      if nargout < 4
         [xinitmean,naninit] = data2xinit_();
         xinitmse = [];
         ainitmean = data2init_();
         varargout{1} = ainitmean;
         varargout{2} = xinitmean;
         varargout{3} = naninit;
      else
         [xinitmean,naninit,xinitmse] = data2xinit_();
         [ainitmean,ainitmse] = data2init_();
         varargout{1} = ainitmean;
         varargout{2} = xinitmean;
         varargout{3} = naninit;
         varargout{4} = ainitmse;
         varargout{5} = xinitmse;
      end
   case 'xinit'
      [varargout{1:nargout}] = data2xinit_();
   case 'y'
      varargout{1} = data2y_();
   case {'e','resid'}
      varargout{1} = data2e_();
   case 'x'
      varargout{1} = data2x_();
   case 'alpha'
      varargout{1} = data2alpha_();
end

   %***********************************************************************
   % Nested function data2xinit_().
   function [xinitmean,naninit,xinitmse] = data2xinit_()
      xinitmean = nan([nb,1,nalt]);
      xinitmse = [];
      switch datatype
         case 'dbase'
            realid = real(m.solutionid{2}(nf+1:end));
            imagid = imag(m.solutionid{2}(nf+1:end));
            xinitmean = db2array(data,m.name(realid),range(1)-1, ...
               imagid,m.log(realid),false,true,true);
            xinitmean = permute(xinitmean,[2,1,3]);
         case 'dpack'
            xinitmean = resize_(data{2}(nf+1:end,:,:), ...
               data{4},range(1)-1,'mean');
         case 'struct'
            xinitmean = resize_(data.mean_{2}(nf+1:end,:,:), ...
               data.mean_{4},range(1)-1,'mean');
            if nargout > 2
               xinitmse = resize_(data.mse_{2}(nf+1:end,nf+1:end,:,:), ...
                  data.mse_{4},range(1)-1,'mse');
            end
      end
      % Search for NaNs in required initial conditions.
      % List of NaN init conds.
      realid = real(m.solutionid{2}(nf+1:end));
      ndata = size(xinitmean,3);
      naninit = false([nb,1]);
      for iloop = 1 : ndata
         if iloop <= nalt
            required = m.icondix(1,:,iloop);
            required = required(:);
         end
         naninit = naninit | (isnan(xinitmean(:,1,iloop)) & required);
      end
      naninit = m.name(unique(realid(naninit)));
   end
   % End of nested function data2xinit_().

   %***********************************************************************
   % Nested function data2init_().
   % Get initial conditions for xb and alpha.
   % Those that are not required are set to NaN in xinitmean, and
   % to 0 when computing ainitmean.
   function [ainitmean,ainitmse] = data2init_()
      % Transform mean x to alpha.
      ndata = size(xinitmean,3);
      if ndata < nalt
         xinitmean(:,1,end+1:nalt) = ...
            xinitmean(:,1,end*ones([1,nalt-ndata]));
         ndata = nalt;
      end
      ainitmean = xinitmean;
      for iloop = 1 : ndata
         if iloop <= nalt
            U = m.solution{7}(:,:,iloop);
            notrequired = ~m.icondix(1,:,iloop);
         end
         index = isnan(xinitmean(:,1,iloop)) & notrequired(:);
         ainitmean(index,1,iloop) = 0;
         ainitmean(:,1,iloop) = U\ainitmean(:,1,iloop);
      end
      % Transform MSE x to alpha.
      if nargout < 2 || isempty(xinitmse)
         ainitmse = xinitmse;
         return
      end
      ndata = size(xinitmse,4);
      if ndata < nalt
         xinitmse(:,:,1,end+1:nalt) = ...
            xinitmse(:,:,1,end*ones([1,nalt-ndata]));
         ndata = nalt;
      end
      ainitmse = xinitmse;
      for iloop = 1 : ndata
         if iloop <= nalt
            U = m.solution{7}(:,:,iloop);
            Ut = U.';
            %required = m.icondix(1,:,iloop);
         end
         %xinitmse(~required,:,1,iloop) = NaN;
         %xinitmse(:,~required,1,iloop) = NaN;
         %ainitmse(~required,:,1,iloop) = 0;
         %ainitmse(:,~required,1,iloop) = 0;
         ainitmse(:,:,1,iloop) = U\ainitmse(:,:,1,iloop);
         ainitmse(:,:,1,iloop) = ainitmse(:,:,1,iloop)/Ut;
      end
   end
   % End of nested function data2init_().
 
   %***********************************************************************
   % Nested function data2y_().
   % Get observables.
   function y = data2y_()
      switch datatype
         case 'dbase'
            realid = real(m.solutionid{1});
            imagid = imag(m.solutionid{1});
            tmplog = m.log(realid);
            y = db2array(data,m.name(realid),range, ...
               imagid,tmplog,false,true,true);
            y = permute(y,[2,1,3]);
         case 'dpack'
            y = resize_(data{1},data{4},range,'mean');
         case 'struct'
            y = resize_(data.mean_{1},data.mean_{4},range,'mean');
      end
   end
   % End of nested function data2y_().

   %***********************************************************************
   % Nested function data2e_().
   % Get residuals. Set NaN residuals to zero.
   function e = data2e_()
      switch datatype
         case 'dbase'
            realid = real(m.solutionid{3});
            imagid = imag(m.solutionid{3});
            e = db2array(data,m.name(realid),range, ...
               imagid,m.log(realid),false,true,true);
            e = permute(e,[2,1,3]);
         case 'dpack'
            e = resize_(data{3},data{4},range,'mean');
         case 'struct'
            e = resize_(data.mean_{3},data.mean_{4},range,'mean');
      end
      e(isnan(e)) = 0;
   end
   % End of nested function data2e_().

   %***********************************************************************
   % Nested function data2x_().
   % Get current dates of transition variables.
   % Set lags and leads to NaN.
   function x = data2x_()
      realid = real(m.solutionid{2});
      imagid = imag(m.solutionid{2});
      index = imagid == 0;
      switch datatype
         case 'dbase'
            realid = realid(index);
            imagid = imagid(index);
            tmplog = m.log(realid);
            tmpdata = db2array(data,m.name(realid),range, ...
               imagid,tmplog,false,true,true);
            x(index,:,:) = permute(tmpdata,[2,1,3]);
         case 'dpack'
            x = resize_(data{2},data{4},range,'mean');
         case 'struct'
            x = resize_(data.mean_{2},data.mean_{4},range,'mean');
      end
      % Set lags and leads to NaN.
      x(~index,:,:) = NaN;      
   end
   % End of nested function data2x_().

   %***********************************************************************
   % Nested function data2alpha_().
   function a = data2alpha_()
      switch datatype
         case 'dbase'
            realid = real(m.solutionid{2});
            imagid = imag(m.solutionid{2});
            realid = realid(nf+1:end);
            imagid = imagid(nf+1:end);
            a = db2array(data,m.name(realid),range, ...
               imagid,m.log(realid),false,true,true);
            a = permute(a,[2,1,3]);
         case 'dpack'
            a = resize(data{2}(nf+1:end,:,:),data{4},range,'mean');
         case 'struct'
            a = resize(data.mean_{2}(nf+1:end,:,:),data.mean_{4}, ...
               range,'mean');
      end
      ndata = size(a,3);
      if ndata < nalt
         a(:,:,end+1:nalt) = a(:,:,end*ones([1,nalt-ndata]));
         ndata = nalt;
      end
      for iloop = 1 : ndata
         if iloop <= nalt
            U = m.solution{7}(:,:,iloop);
         end
         a(:,:,iloop) = U\a(:,:,iloop);
      end
   end

end
% End of primary function.

%**************************************************************************
% Subfunction resize_().
function x = resize_(x,oldrange,newrange,type)

ismean = strcmp(type,'mean');

% Same input range and output range.
if round(oldrange(1) - newrange(1)) == 0 ...
      && round(oldrange(end) - newrange(end)) == 0
   return
end

% Resolve [-Inf,date] or [date,Inf].
if length(newrange) == 2
   if isinf(newrange(1))
      newrange(1) = oldrange(1);
   end
   if isinf(newrange(2))
      newrange(2) = oldrange(end);
   end
end
newrange = newrange(1) : newrange(end);

noldrange = length(oldrange);
index = round(newrange - oldrange(1)) + 1;
npre = sum(index < 1);
npost = sum(index > noldrange);
index(index < 1 | index > noldrange) = [];

if ismean
   tmpsize = [ ...
      size(x,1), ...
      size(x,2), ...
      size(x,3), ...
      ];
   x = [ ...
      nan([tmpsize(1),npre,tmpsize(3)]), ...
      x(:,index,:), ...
      nan([tmpsize(1),npost,tmpsize(3)]) ];
else
   tmpsize = [ ...
      size(x,1), ...
      size(x,2), ...
      size(x,3), ...
      size(x,4), ...
      ];
   x = cat(3, ...
      nan([tmpsize(1:2),npre,tmpsize(4)]), ...
      x(:,:,index,:), ...
      nan([tmpsize(1:2),npost,tmpsize(4)]) );
end

end
% End of subfunction resize_().