function [pstar_,obj_,Grad,Hess,m,se2,F,pe,delta,Pdelta,void,pred,smooth] = ...
   estimate(m,data,range,D,varargin)
   % ESTIMATE  Estimate model parameters by optimising selected objective function.
   % For help on this function, type <a href="matlab:idoc('model.estimate')">idoc model.estimate</a>.

   % The IRIS Toolbox.
   % Copyright 2007-2009 Jaromir Benes.

% Type <a href="matlab: model.estimate">idoc model.estimate</a> or click the link to get help on this function.

% The IRIS Toolbox 2009/09/22.
% Copyright 2007-2009 Jaromir Benes.

% Syntax:
%    [pstar,obj,grad,hess,m,se2,F,pe,A,Pa,setup,pred,smooth] = estimate(m,dpack,p,...) 
%    [pstar,obj,grad,hess,m,se2,F,pe,A,Pa,setup,pred,smooth] = estimate(m,dbase,range,p,...)
% Output arguments:
%    pstar [ struct ] Database with point estimates of requested parameters.
%    mloglik [ numeric ] Value of objective function.
%    grad [ numeric ] Gradient at optimum.
%    hess [ numeric ] Hessian at optimum.
% Required input arguments:
%    m [ model ] Model.
%    p [ struct ] Database of optimised parameters entered as 1x3 or 1x4 cell arrays: {init,lo,hi} or {init,lo,hi,prior_distrib}.
%    dpack [ cell ] Input datapack.
%    dbase [ cell ] Input database.
%    range [ numeric ] Time range, i.e. <a href="dates.html">IRIS serial date numbers</a>.
% <a href="options.html">Optional input arguments:</a>
%    'exclude' [ cellstr | <a href="">empty</a> ] List of measurement variables to be excluded from likelihood function (treated as deterministic).
%    'maxiter' [ numeric | <a href="default.html">2000</a> ] Maximum number of iterations allowed.
%    'objective' [ <a href="default.html">'mloglik'</a> | 'prederr' | function_handle ] Objective function to be minimised.
%    'objectivesample' [ <a href="default.html">Inf</a> | numeric ] Sub-sample on which the objective function is to be computed.
%    'output' [ <a href="default.html">'auto'</a> | 'dbase' | 'dpack' ] Format of output data.
%    'penalty' [ numeric | <a href="default.html">0</a> ] Regularise the likelihood function with a quadratic penalty function.
%    'refresh' [ <a href="default.html">true</a> | false ] Refresh dynamic links for each parameter set.
%    'relative' [ <a href="default.html">true</a> | false ] Scale MSE matrices by estimated variance factor.
%    'solve' [ <a href="default.html">true</a> | false ] Compute solution for each parameter set.
%    'sstate' [ true | <a href="default.html">false</a> | function_handle ] Function to compute steady state.
%    'tolfun' [ numeric | <a href="default.html">1e-8</a> ] Termination tolerance on log-likehood function.
%    'tolx' [ numeric | <a href="default.html">1e-8</a> ] Termination tolerance on optimised parameters.

% Validate required input arguments.
P = inputParser();
P.addRequired('model',@ismodel);
P.addRequired('data',@(x) isstruct(x) || iscell(x));
P.addRequired('range',@isnumeric);
P.addRequired('parameters',@isstruct);
P.parse(m,data,range,D);

% Process |estimate| options.
[options,varargin] = passvalopt('model.estimate',varargin{:});
% Process |loglik| options.
loglikoptions = loglikopt_(m,range,options.domain,varargin{:});

void = []; % Void arguments for bkw compatibility.

%**************************************************************************
% Function body.

% Get array of measurement variables.
% Get observables and determine the format of output data.
input = dataformat(data,true);
output = outputformat(input,options);
data = datarequest('y',m,data,range);

% Set up objective functions and handles to anonymous functions. This can
% be done only after data have been converted.
if strncmpi(options.domain,'t',1)
   options.domain = 1;
   % Time domain likelihood.
   loglik_ = @(x) kalman_(x,data,range,[],loglikoptions);
else
   options.domain = 2;
   % Frequency domain likelihood.
   [I,freq,delta] = fourierdata_(m,data,loglikoptions);
   loglik_ = @(x) loglikf_(x,I,freq,delta,loglikoptions);
end

np = sum(m.nametype == 4);
nalt = size(m.assign,3);

if isempty(m.refresh)
   options.refresh = false;
end

ndata = size(data,3);

% Mmultiple parameterisations not allowed.
if nalt > 1 || ndata > 1
   error_(47,'<<matlab:idoc(''model.maxloglik'') maxloglik>>');
end

% Retrieve names of optimised parameters,
% initial values, lower & upper bounds, and prior distributions.
% Set parameters in m to starting values.
[keep,plist,pindex,p0,pl,pu,...
   prior,priorindex,weight,penaltyindex,pbar,...
   notfound,invalidbounds,outofbounds] = ...
   optimparams_(m,D,options.penalty);
np = numel(pindex);
isprior = any(priorindex);
ispenalty = any(penaltyindex);

if ~isempty(notfound)
   warning_(31,notfound);
end

if ~isempty(invalidbounds)
   warning_(46,invalidbounds);
end

if ~isempty(outofbounds)
   warning_(49,outofbounds);
end

% Database P used only if steady state is requested.
if isa(options.sstate,'function_handle')
   P = struct();
   for i = find(m.nametype == 4)
      P.(m.name{i}) = m.assign(i);
   end
end

if isnumeric(options.typicalx)
   options.typicalx = options.typicalx(keep);
elseif strcmpi(options.typicalx,'start')
   options.typicalx = p0;
end

Grad = {zeros([1,np]),zeros([1,np])};
Hess = {zeros(np),zeros(np)};

if np > 0
   % Set up Optim Tbx problem.
   problem = struct();
   problem.objective = @objfunc;
   problem.x0 = p0;
   problem.lb = pl;
   problem.ub = pu;
   problem.options = optimset(...
      'algorithm','active-set',...
      'display','iter',...
      'maxiter',options.maxiter,...
      'maxfunevals',options.maxfunevals,...
      'GradObj','off',...
      'Hessian','off',...
      'LargeScale','off',...
      'tolfun',options.tolfun,...
      'tolx',options.tolx,...
      'typicalX',options.typicalx);

   % Call Optim Tbx solver.
   obj0 = realmax()*eps();
   if all(isinf(pl)) && all(isinf(pu))
      problem.solver = 'fminunc';
      [pstar,objstar,EXITFLAG,OUTPUT,Grad{1}(:,:),Hess{1}(:,:)] = fminunc(problem);
   else
      problem.solver = 'fmincon';
      [pstar,objstar,EXITFLAG,OUTPUT,Lambda,Grad{1}(:,:),Hess{1}(:,:)] = fmincon(problem);
   end

   % Compute contributions of prior distributions to gradient and hessian.
   if isprior
      [ans,gradprior,hessprior] = prior_(pstar);
      Grad{2}(:,:) = Grad{2}(:,:) + gradprior;
      Hess{2}(:,:) = Hess{2}(:,:) + hessprior;
   end
   % Compute contributions of penalty func to gradient and hessian.
   if ispenalty
      [ans,gradpenal,hesspenal] = penalty_(pstar);
      Grad{2}(:,:) = Grad{2}(:,:) + gradpenal;
      Hess{2}(:,:) = Hess{2}(:,:) + hesspenal;
   end

   % Assign estimated parameters,
   % and handle sstate and solve.
   if nargout > 4
      [m,npath] = updatemodel_(m,pstar,pindex,options);
   end
else
   % No parameters to be optimised.
   warning_(48);
end
   
%**************************************************************************
% Post mortem.

if options.domain == 1 && nargout > 4
   if nargout > 12
      % Smoothing requested.
      [ans,se2,F,pe,delta,Pdelta,void,pred,smooth] = loglik_(m);
   elseif nargout > 11
      % No smoothing requested.
      [ans,se2,F,pe,delta,Pdelta,void,pred] = loglik_(m);
      smooth = [];
   else
      % Neither smoothing nor prediction requested.
      [ans,se2,F,pe,delta,Pdelta] = loglik_(m); 
      pred = [];
      smooth = [];
   end
   % Scale stdevs by estimated factor, and assign out-of-lik params.
   % No need to re-compute steady state or solution.
   if loglikoptions.relative
      m = stdscale(m,sqrt(se2));
   end
   if ~isempty(loglikoptions.outoflik)
      m.assign(1,loglikoptions.outoflik) = delta(:)';
   end
   if loglikoptions.relative || ~isempty(loglikoptions.outoflik)
      m = refresh(m);
   end
   if strcmpi(output,'dbase')
      [F,pe,delta,Pdelta,pred,smooth] = outputdbase_( ...
         m,F,pe,delta,Pdelta,pred,smooth,range,loglikoptions);
   end
else
   [ans,se2] = loglik_(m);
   % Scale std errors by estimated factor.
   % No need to re-compute steady state or solution.
   m = stdscale(m,sqrt(se2));
   m = refresh(m);
end

if np == 0
   pstar_ = struct();
   obj_ = @(x) objstar;
else
   pstar_ = cell2struct(num2cell(pstar),plist,2);
   obj_ = @objfunc;
end

% End of function body.

%**************************************************************************
% Nested function objfunc().
   function varargout = objfunc(p,varargin)
      if nargin == 1
         % Assign parameters,
         % recompute steady state if requested,
         % solve model if requested,
         % refresh links if requested.
         [m,npath] = updatemodel_(m,p,pindex,options);
         if npath ~= 1
            if strcmp(options.nosolution,'error')
               failed(m,npath,'estimate');
            else
               varargout{1} = obj0 + 10*abs(obj0);
            end
         else
            varargout{1} = loglik_(m);
            if isprior
               mlogprior = zeros(size(p));
               mlogprior(priorindex) = -log(cellfun(@feval,prior(priorindex),num2cell(p(priorindex))));
               varargout{1} = varargout{1} + sum(mlogprior);
            end
            if ispenalty
               objpenal = penalty_(p);
               varargout{1} = varargout{1} + objpenal;
            end
            obj0 = varargout{1};
         end
         if nargout > 1
            varargout{2} = m;
         end
      elseif nargin == 0
         varargout{1} = objstar;
      elseif nargin > 1
         switch lower(varargin{1})
            case 'plist'
               varargout{1} = plist;
            case 'pstar'
               varargout{1} = pstar;
         end
      end
   end
% End of nested function objfunc().

%**************************************************************************
% Nested function prior_().
% Compute pdf, gradient and hessian for general prior distributions.

function [objprior,gradprior,hessprior] = prior_(p)
   tmp = p(priorindex);
      step = max([abs(tmp);ones(size(tmp))],[],1)*eps()^(1/3);
      tmpplus = tmp + step;
      tmpminus = tmp - step;    
      mlogprior = -log(cellfun(@feval,prior(priorindex),num2cell(tmp)));
      mlogpriorplus = -log(cellfun(@feval,prior(priorindex),num2cell(tmpplus)));
      mlogpriorminus = -log(cellfun(@feval,prior(priorindex),num2cell(tmpminus)));
      twosteps = tmpplus - tmpminus;
   objprior = sum(mlogprior);
   if nargout > 1
      gradprior = zeros(size(p));
      hessprior = zeros(np);
      gradprior(priorindex) = (mlogpriorplus - mlogpriorminus) ./ twosteps;
      hessprior(priorindex) = (mlogpriorplus - 2*mlogprior + mlogpriorminus) ./ (twosteps.^2);
      hessprior = spdiags(hessprior(:),0,np,np);  
   end
end
% End of nested function prior_().

   %***********************************************************************
   % Nested function penalty_().
   function [objpenal,gradpenal,hesspenal] = penalty_(p)
      objpenal = zeros(size(p));
      objpenal(penaltyindex) = ...
         sum(((p(penaltyindex) - pbar(penaltyindex)).^2).*weight((penaltyindex)));
      objpenal = sum(objpenal);
      if nargout > 1
         gradpenal = zeros(size(p));
         hesspenal = zeros(size(p));
         gradpenal(penaltyindex) = ...
            2*(p(penaltyindex) - pbar(penaltyindex)).*weight(penaltyindex);
         hesspenal(penaltyindex) = 2*weight(penaltyindex);
         hesspenal = spdiags(hesspenal(:),0,np,np);
      end
   end
   % End of nested function penalty_().

end
% End of primary function.

%**************************************************************************
% Subfunction body.

function [keep,plist,pindex,p0,pl,pu,...
   prior,priorindex,weight,penaltyindex,pbar,...
   notfound,invalidbounds,outofbounds] = optimparams_(m,d,penalty)

fields = fieldnames(d);
np = numel(fields);

pindex = [];
plist = {};
p0 = [];
pl = [];
pu = [];
prior = {};
priorindex = false([1,0]);
weight = [];
penaltyindex = false([1,0]);
pbar = [];
keep = false([1,np]);

notfound = {};
invalidbounds = {};
outofbounds = {};

for i = 1 : np
   name_ = fields{i};
   pindex_ = sum(m.nametype < 4) + find(strcmp(m.name(m.nametype == 4),name_));
   if isempty(pindex_)
      notfound{end+1} = name_;
      continue
   end
   aux = d.(name_);
   if isnumeric(aux)
      aux = num2cell(aux);
   end
   if ~isempty(aux) && isnumericscalar(aux{1})
      p0_ = aux{1};
   else
      p0_ = NaN;
   end
   if isnan(p0_)
      p0_ = m.assign(1,pindex_);
   end
   if length(aux) > 1 && isnumericscalar(aux{2})
      pl_ = aux{2};
   else
      pl_ = -Inf;
   end
   if length(aux) > 2  && isnumericscalar(aux{3})
      pu_ = aux{3};
   else
      pu_ = Inf;
   end
   if pl_ >= pu_
      invalidbounds{end+1} = name_;
      continue
   end
   if p0_ < pl_ || p0_ > pu_
      outofbounds{end+1} = name_;
      continue
   end

   % Prior distribution function (function_handle),
   % or penalty function ([weight] or [weight,centre]).
   prior_ = [];
   isprior_ = false;
   weight_ = 0;
   pbar_ = NaN;
   ispenalty_ = false;
   if length(aux) > 3
      if isfunc(aux{4})
         % Fourth entry is a prior distribution func.
         prior_ = aux{4};
         isprior_ = true;
      elseif isnumeric(aux{4}) && penalty > 0
         % Fourth entry is a penalty function.
         weight_ = aux{4}(1)*penalty;
         if length(aux{4}) == 1
            % Only weight specified. Centre of penalty func identical to
            % starting value.
            pbar_ = p0_;
         else
            % Both weight and centre specified.
            pbar_ = aux{4}(2);
         end
         if isnan(pbar_)
            pbar_ = m.assign(1,pindex_);
         end
         ispenalty_ = true;
      end
   end
   
   keep(i) = true;
   plist{end+1} = name_;
   pindex(end+1) = pindex_;
   p0(end+1) = p0_;
   pl(end+1) = pl_;
   pu(end+1) = pu_;
   prior{end+1} = prior_;
   priorindex(end+1) = isprior_;
   weight(end+1) = weight_;
   penaltyindex(end+1) = ispenalty_;
   pbar(end+1) = pbar_;

end

end
% End of primary function.