function [func,fcon,Pi] = forecast(m,init,range,cond,varargin)
% Type <a href="matlab: idoc model.forecast">idoc model.forecast</a> or click the link to get help on this function.

% The IRIS Toolbox.
% Copyright 2007-2010 Jaromir Benes.

options = passvalopt('model.forecast',varargin{:});

% Determine format of input and output data.
input = dataformat(init,true);
output = outputformat(input,options);

if ischar(options.dtrends)
   options.dtrends = ~options.deviation;
end

if isempty(cond)
  cond = struct();
end

%**************************************************************************
% Function body.

ny = size(m.solution{4},1);
nx = size(m.solution{1},1);
nb = size(m.solution{7},1);
nf = nx - nb;
ne = size(m.solution{2},2);
nalt = size(m.assign,3);
range = range(1) : range(end);
nper = length(range);

nanticipate = length(options.anticipate(:));
ndeviation = length(options.deviation(:));
ndtrends = length(options.dtrends(:));

% Get init cond (mean, MSE) for alpha vector.
% Initmse is [] if MSE is not available.
[ainitmean,xinitmean,naninit,ainitmse,xinitmse] = ...
   datarequest('init',m,init,range);
if ~isempty(naninit)
   error_(25,naninit);
end

ninit = size(ainitmean,3);

% Get structural conditions.
shock = datarequest('e',m,init,range);
nshock = size(shock,3);

% For bkw compatibility only.
% Add structural conditions from tune database.
shock1 = datarequest('e',m,cond,range);
isshock = any(shock(:) ~= 0);
isshock1 = any(shock1(:) ~= 0);
if isshock && isshock1
   % Structural conditions simultaneously in input database and in tune
   % database are not allowed.
   error_(74);
end
if isshock1
   shock = shock1;
   warning_(57);
end

% Get reduced-form conditions.
cond = datarequest('y',m,cond,range);
ncond = size(cond,3);

% Get user-supplied variances of shocks.
[options.varvec,options.anyvarvec,options.varvecindex] = ...
   std2varvec_(m,options.std,range);

% Total number of cycles.
nloop = max([nalt,nanticipate,ndeviation,ndtrends,ninit,ncond,nshock]);

% Pre-allocate output datapack.
func = struct();
func.mean_ = { ...
   nan([ny,1+nper,nloop]), ...
   nan([nx,1+nper,nloop]), ...
   nan([ne,1+nper,nloop]), ...
   [range(1)-1,range], ...
   };
if ~options.meanonly
   func.mse_ = { ...
      nan([ny,ny,1+nper,nloop]), ...
      nan([nx,nx,1+nper,nloop]), ...
      nan([ne,ne,1+nper,nloop]), ...
      [range(1)-1,range], ...
      };
end
if nargout > 1
   fcon = func; 
end

Pi = nan([1,nloop]); % test statistic

underdet = []; % index of underdetermined systems
nansolution = []; % index of NaN solutions
nanexpand = []; % index of NaN expansions

use = struct();

for iloop = 1 : nloop

   if iloop <= ndeviation
      use.deviation = options.deviation(iloop);
   end
   
   if iloop <= ndtrends
      use.dtrends = options.dtrends(iloop);
   end

   if iloop <= nanticipate
      use.anticipate = options.anticipate(iloop);
   end

   if iloop <= ncond
      % measurement conditions including detereministic trends
      use.conddet = cond(:,:,iloop);
      use.condindex = ~isnan(use.conddet);
      use.lastcond = max([0,find(any(use.condindex,1),1,'last')]); % last imposed tune
      use.condindex = use.condindex(:,1:use.lastcond);
      use.condindex = use.condindex(:)';
   end

   if iloop <= nalt
      % model solution
      [use.T,use.R,use.K,use.Z,use.H,use.D,use.U,use.Omega] = sspace_(m,iloop,true);
      % matrices for forward expansion
      use.expand = cell(size(m.expand));
      for i = 1 : length(m.expand)
         use.expand{i} = m.expand{i}(:,:,iloop);
      end
      % deterministic trends
      if options.dtrends
         use.W = dtrendsrequest_('range',m,range,iloop);
      end
   end

   if any(any(isnan(use.T)))
      nansolution(end+1) = iloop;
      continue
   end

   if any(any(isnan(use.expand{1})))
      nanexpand(end+1) = iloop;
      continue
   end

   if iloop <= nalt
      use.varvec = transpose(m.assign(1,end-ne+1:end,iloop).^2);
      use.varvec = use.varvec(:,ones([1,nper]));
      if options.anyvarvec
         use.varvec(options.varvecindex) = ...
            options.varvec(options.varvecindex);
      end
      use.activeshocks = use.varvec(:,1:use.lastcond) > 0;
      use.activeshocks = transpose(use.activeshocks(:));
   end

   if iloop <= nalt || iloop <= ncond
      % conditions adjusted for deterministic trends
      use.cond = use.conddet;
      if use.dtrends
         use.cond = use.cond - use.W;   
      end     
   end

   if iloop <= ninit
      % init condition mean and MSE
      use.initmean = ainitmean(:,1,iloop);
      if ~isempty(ainitmse) && ~strcmpi(options.initcond,'fixed')
         use.initmse = ainitmse(:,:,iloop); 
         use.activeinit = abs(diag(use.initmse)') > options.tolmse;
      elseif isnumeric(options.initcond)
         if iloop <= nalt
            error('iris:model', ...
               'Option |''initcod''| not implemented in |forecast| yet.'); 
         end
      else
         use.initmse = sparse(zeros(length(use.initmean)));
         use.activeinit = false([1,length(use.initmean)]);
      end
   end   
   
   if sum(use.condindex) > sum(use.activeinit) + sum(use.activeshocks)
      underdet(end+1) = iloop;
      continue
   end

   if iloop <= nshock
      use.shock = shock(:,:,iloop);
      use.lastshock = max([0,find(any(use.shock ~= 0),1,'last')]); % last imposed shock
   end

   % Furthest anticipated shock needed.
   if use.anticipate
      use.last = max([use.lastshock,use.lastcond]);   
      use.tplusk = use.last;
   else
      use.last = 0;
      use.tplusk = max([0,find(any(imag(use.shock) ~= 0),1,'last')]);
   end
 
   if ne > 0
      % Expansion available up to t+k0.
      if use.tplusk > size(use.R,2)/ne
         [use.R,use.expand{5}] = expand_(use.R,use.tplusk-1,use.expand{1:5});
      end
   end

   % Compute multipliers of initial condition, unanticipated and
   % anticipated shocks.
   if use.lastcond > 0
      if iloop <= nalt || iloop <= ncond
         % Multipliers of initial condition and unanticipated shocks.
         % This is needed whether forecast is anticipated or not.
         [use.DyDa0,use.DaDa0,use.DfDa0] = time_domain.multiplierinit(...
            use.T,use.R,use.K,use.Z,use.H,use.D,use.U,...
            use.lastcond,use.activeinit);
         [use.DyDeu,use.DaDeu,use.DfDeu] = time_domain.multipliereu(...
            use.T,use.R,use.K,use.Z,use.H,use.D,use.U,...
            use.lastcond,use.activeshocks);
         use.DfaDa0eu = [];
         for t = 1 : use.lastcond
            use.DfaDa0eu = [...
               use.DfaDa0eu;...
               use.DfDa0((t-1)*nf+(1:nf),:),use.DfDeu((t-1)*nf+(1:nf),:);...
               use.DaDa0((t-1)*nb+(1:nb),:),use.DaDeu((t-1)*nb+(1:nb),:);...
            ];
         end
      end
      if iloop <= nalt || iloop <= ncond || iloop <= nanticipate
         if use.anticipate
            % Mutlipliers of anticipated shocks.
            use.DyDea = time_domain.multiplierea(...
               use.T,use.R,use.K,use.Z,use.H,use.D,use.U,...
               use.lastcond,use.activeshocks);
         end
      end
   end

   % Structural conditions.

   [y,w] = time_domain.simulatemean(...
      use.T,use.R,use.K,use.Z,use.H,use.D,use.U,...
      use.initmean,use.shock,nper,use.anticipate,use.deviation,[]);
   xf = w(1:nf,:);
   a = w(nf+1:end,:);
   [Py,Pfa,Pe] = time_domain.simulatemse(...
      use.T,use.R,use.K,use.Z,use.H,use.D,use.U,use.varvec,...
      use.initmse,nper);

   % Store forecast with structural conditions.

   if use.dtrends
      func.mean_{1}(:,2:end,iloop) = y + use.W;
   else
      func.mean_{1}(:,2:end,iloop) = y;
   end
   func.mean_{2}(:,2:end,iloop) = [xf;a];
   func.mean_{2}(nf+1:end,1,iloop) = use.initmean;
   func.mean_{3}(:,2:end,iloop) = use.shock;
   if ~options.meanonly
      func.mse_{1}(:,:,2:end,iloop) = Py;
      func.mse_{2}(:,:,2:end,iloop) = Pfa;
      func.mse_{2}(nf+1:end,nf+1:end,1,iloop) = use.initmse;
      func.mse_{3}(:,:,2:end,iloop) = Pe;
   end
   
   % Reduced-form conditions.

   if use.lastcond > 0 && nargout > 1

      % Conditional mean.

      Z1 = use.DyDa0(use.condindex,:);
      if use.anticipate
         Z2 = use.DyDea(use.condindex,:);
      else
         Z2 = use.DyDeu(use.condindex,:);
      end
      pe = use.cond(use.condindex) - y(use.condindex);
      % P = blkdiag([initmse,0;0,diag(varvec)]) = [P1;P2]
      % Z = [Z1,Z2];
      P1 = use.initmse(use.activeinit,use.activeinit);
      P2 = sparse(diag(use.varvec(use.activeshocks)));
      P_Zt = [ % P_Zt = P*transpose(Z);
         P1*transpose(Z1)
         P2*transpose(Z2)
      ];
      F = [Z1,Z2] * P_Zt;
      M = P_Zt / F;
      gamma = [ % gamma := [a(0);e(1);...;e(lastcond)] both active and inactive
         use.initmean
         vec(use.shock(:,1:use.lastcond))
      ]; 
      active = [use.activeinit,use.activeshocks];
      gammahat = gamma;
      dgammahat = M * vec(pe); % only active entries
      gammahat(active) = gammahat(active) + dgammahat;

      % Simulate conditional mean with new init cond and new residuals.
      tmpinit = gammahat(1:nb);
      tmpshock = [reshape(gammahat(nb+1:end),[ne,use.lastcond]),use.shock(:,use.lastcond+1:end)];
      [yhat,what] = time_domain.simulatemean(...
         use.T,use.R,use.K,use.Z,use.H,use.D,use.U,...
         tmpinit,tmpshock,nper,use.anticipate,use.deviation,[]);
      xfhat = what(1:nf,:);
      ahat = what(nf+1:end,:);

      % Store conditional mean.

      if options.deviation
         fcon.mean_{1}(:,2:end,iloop) = yhat;
      else
         fcon.mean_{1}(:,2:end,iloop) = yhat + use.W;
      end
      fcon.mean_{2}(:,2:end,iloop) = [xfhat;ahat];
      fcon.mean_{2}(nf+1:end,1,iloop) = tmpinit;
      fcon.mean_{3}(:,2:end,iloop) = tmpshock;

      % Conditional MSE.

      if options.anticipate
         Z2 = use.DyDeu(use.condindex,:);
         P1 = use.initmse(use.activeinit,use.activeinit);
         P2 = sparse(diag(use.varvec(use.activeshocks)));
         P_Zt = [
            P1*transpose(Z1)
            P2*transpose(Z2)
         ];
         F = [Z1,Z2] * P_Zt;
         M = P_Zt / F;
      end
      P = blkdiag(P1,P2);
      V = zeros(nb+ne*use.lastcond); % V = MSE gammahat, i.e. both active and inactive
      V(active,active) = P - M*transpose(P_Zt);

      % test statistic
      if nargout > 2
         Pi(iloop) = dgammahat.' ...
            * blkdiag(pinv(P1),diag(1./(diag(P2)))) * dgammahat;
      end

      % MSE for y(t)
      % t = 1 .. lastcond
      X = [use.DyDa0,use.DyDeu];
      Vy = X*V(active,active)*transpose(X);

      % MSE for xf(t) and alpha(t)
      % t = 1 .. lastcond
      Vfa = use.DfaDa0eu*V(active,active)*transpose(use.DfaDa0eu);

      % MSE for e(t)
      % t = 1 .. lastcond
      Ve = V(nb+1:end,nb+1:end);

      % Project MSE.
      % t = lastcond+1 .. nper
      if ~options.meanonly
         [Vy2,Vfa2,Ve2] = time_domain.simulatemse(...
            use.T,use.R,use.K,use.Z,use.H,use.D,use.U,use.varvec(:,use.lastcond+1:end),...
            Vfa(end-nb+1:end,end-nb+1:end),nper-use.lastcond);
      end
         
      % Store conditional MSE.

      if ~options.meanonly
         for t = 1 : use.lastcond
            fcon.mse_{1}(:,:,1+t,iloop) = Vy((t-1)*ny+(1:ny),(t-1)*ny+(1:ny));
            fcon.mse_{2}(:,:,1+t,iloop) = Vfa((t-1)*nx+(1:nx),(t-1)*nx+(1:nx));
            fcon.mse_{3}(:,:,1+t,iloop) = Ve((t-1)*ne+(1:ne),(t-1)*ne+(1:ne));
         end
         fcon.mse_{2}(nf+1:end,nf+1:end,1,iloop) = V(1:nb,1:nb); % initial condition
         fcon.mse_{1}(:,:,1+(use.lastcond+1:nper),iloop) = Vy2;
         fcon.mse_{2}(:,:,1+(use.lastcond+1:nper),iloop) = Vfa2;
         fcon.mse_{3}(:,:,1+(use.lastcond+1:nper),iloop) = Ve2;
      end
         
   else

      if nargout > 1
         for i = 1 : 3
            fcon.mean_{i}(:,:,iloop) = func.mean_{i}(:,:,iloop);
            if ~options.meanonly
               fcon.mse_{i}(:,:,:,iloop) = func.mse_{i}(:,:,:,iloop);
            end
         end%for
      end%if

   end%if

end%for

%**************************************************************************
% Post-mortem.

% Transform alpha to xb.
func = alpha2xb_(m,func);
if nargout > 1
   fcon = alpha2xb_(m,fcon);
end

% Fix negative diagonal entries.
if ~options.meanonly
   for i = 1 : 3
      func.mse_{i} = time_domain.fixcov(func.mse_{i});
      if nargout > 1
         fcon.mse_{i} = time_domain.fixcov(fcon.mse_{i});
      end
   end
end

% Convert datapack to database.
if strcmp(output,'dbase')
   if options.meanonly
      func = dp2db(m,func.mean_);
   else
      func = dp2db(m,func);
   end
   if nargout > 1
      if options.meanonly
         fcon = dp2db(m,fcon.mean_);
      else
         fcon = dp2db(m,fcon);
      end
   end
end

% Underdetermined conditional forecast system.
if ~isempty(underdet)
   warning_(40,sprintf(' #%g',underdet));
end

% Expansion not avaiable.
if ~isempty(nansolution)
   warning_(44,sprintf(' #%g',nansolution));
end

% Expansion not available.
if ~isempty(nanexpand)
   warning_(45,sprintf(' #%g',nanexpand));
end

end
% End of primary function.