function varargout = get(this,varargin)
% <a href="matlab:edit(fullfile(irisroot(),'/class_library/@model/get.m'))">GET  Access model object properties.</a>
% 
% Syntax:
%   value = get(m,property)
%   [value1,value2,...] = get(m,property1,property2,...)
% 
% Input arguments:
% 
% * |m| [ model ] Model object.
% 
% * |property| [ char ] Name of the requested property.
% 
% Output arguments:
% 
% * |value| [ * ] Value of the requested property.
% 
% Description:
% 
% Below is the categorised list of properties that can be accessed by the
% |get| function. Note that letter 'y' is used in various contexts to
% denote measurement variables or equations, 'x' transition variables or
% equations, 'e' shocks, 'p' parameters, 'd' deterministic trend
% equations, 'l' dynamic links, and 'r' reporting equations. The property
% names are case insensitive.
% 
% Steady state:
% 
% You can |<a href="../misc/steady_state.html">learn more</a>| about how IRIS handles and
% describes the steady state.
% 
% * 'sstate' Returns [ struct ] a database with the steady states for all
%     model variables. The steady states are described by complex numbers in
%     which the real part is the level and the imaginary part is the growth rate.
% 
% * 'sstateLevel' Returns [ struct ] a database with the steady-state levels
%     for all model variables.
% 
% * 'sstateGrowth' Returns [ struct ] a database with steady-state growth
%     (first difference for linearised variables, gross rate of growth for
%     log-linearised variables) for all model variables.
% 
% * 'dtrends' Returns [ struct ] a database with the effect of the deterministic trends on
%     the measurement variables. The effect is described by complex numbers
%     the same way as the steady state.
% 
% * 'dtrendsLevel' Returns [ struct ] a database with the effect of the deterministic
%     trends on the steady-state levels of the measurement variables.
% 
% * 'dtrendsGrowth' Returns [ struct ] a database with the effect of
%     deterministic trends on steady-state growth of the measurement variables.
% 
% * 'sstate+dtrends' Returns [ struct ] the same as 'sstate' except that the measurement
%     variables are corrected for the effect of the deterministic trends.
% 
% * 'sstateLevel+dtrendsLevel' Returns [ struct ] the same as 'sstateLevel' except that
%     the measurement variables are corrected for the effect of the deterministic
%     trends.
% 
% * 'sstateGrowth+dtrendsGrowth' Returns [ struct ] the same as 'sstateGrowth' except
%     that the measurement variables are corrected for the effect of the
%     deterministic trends.
% 
% List of variables, parameters and equations:
% 
% * 'yList' Returns [ cellstr ] the list of the measurement variables in order of their
%     appearance in the model code declarations.
% 
% * 'xList' Returns [ cellstr ] the list of the transition variables in order of their
%     appearance in the model code declarations.
% 
% * 'eList' Returns [ cellstr ] the list of the shock in order of their
%     appearance in the model code declarations.
% 
% * 'pList' Returns [ cellstr ] the list of the parameters in order of their
%     appearance in the model code declarations.
% 
% * 'stdList' Returns [ cellstr ] the list of the standard deviations of the shocks in
%     order of the shocks' * appearance in the model code declarations.
% 
% * 'yEqtn' Returns [ cellstr ] the list of the measurement equations in order of their
%     appearance in the model code.
% 
% * 'xEqtn' Returns [ cellstr ] the list of the transition equations in order of their
%     appearance in the model code.
% 
% * 'dEqtn' Returns [ cellstr ] the list of the deterministic-trend equations in order of
%     the appearance of the respective measurement variables in the model code
%     declarations.
% 
% * 'lEqtn' Returns [ cellstr ] the list of the dynamic links in order of the
%     appearance in the model code.
% 
% * 'rEqtn' Returns [ cellstr ] the list of the reporting equations in order of their
%     appearance in the model code.
% 
% Tags and labels:
% 
% * 'tags' Returns [ struct ] a database with the tags each variable.
% 
% * 'yTags' Returns [ cellstr ] the tags associated with the measurement variables.
% 
% * 'xTags' Returns [ cellstr ] the tags associated with the transition variables.
% 
% * 'eTags' Returns [ cellstr ] the tags associated with the shocks.
% 
% * 'pTags' Returns [ cellstr ] the tags associated with the parameters.
% 
% * 'yLabels' Returns [ cellstr ] the labels associated with the measurement equations.
% 
% * 'xLabels' Returns [ cellstr ] the labels associated with the transition equations.
% 
% * 'dLabels' Returns [ cellstr ] the labels associated with the deterministic-trend equations.
% 
% * 'lLabels' Returns [ cellstr ] the labels associated with the dynamic links.
% 
% * 'rLabels' Returns [ cellstr ] the labels associated with the reporting equations.
% 
% Parameter values:
% 
% * 'parameters' Returns [ struct ] a database with the currently assigned parameters
%     values, including the std deviations of shocks.
% 
% * 'std' Returns [ struct ] a database with the currently assigned std deviations
%     of shocks.
% 
% Eigenvalues:
% 
% You can use the |<a href="eig.html">eig</a>| function to retrieve the vector(s) of all eigenvalues.
% 
% * 'stableRoots' Returns [ cell of numeric ] a vector of the model eigenvalues that are
%     smaller than one in magnitude (allowing for rounding errors around one).
% 
% * 'unitRoots' Returns [ cell of numeric ] a vector of the model eigenvalues that equal one
%     in magnitude (allowing for rounding errors around one).
% 
% * 'unstableRoots' [ cell of numeric ] A vector of the model eigenvalues that are greater
%     than one in magnitude (allowing for rounding errors around one).
% 
% Model structure and model solution:
% 
% You can use the |<a href="sspace.html">sspace</a>| function to retrieve the model's state-space
% representation.
% 
% * 'log' Returns [ struct ] a database with true for each log-linearised
%     variables, and false for each linearised variable.
% 
% * 'maxLag' Returns [ numeric ] the maximum lag in the model.
% 
% * 'maxLead' Returns [ numeric ] the maximum lead in the model.
% 
% * 'stationary' Returns [ struct ] a database with true for each stationary
%     variables, and false for each unit-root (non-stationary) variables (under
%     current solution).
% 
% * 'nonStationary' Returns [ struct ] a database with true for each
%     unit-root (non-stationary) varible, and false for each stationary variable
%     (under current solution). 
% 
% * 'stationaryList' Returns [ cellstr ] the list of stationary variables
%     (under current solution).
% 
% * 'nonStationaryList' Returns [ cellstr ] cell with the list of unit-root
%     (non-stationary) variables
%     (under current solution).
% 
% * 'initCond' Returns [ cellstr ] the list of the lagged transition
%     variables that need to be supplied as initial conditions in simulations and
%     forecasts. The list of the initial conditions is solution-specific as the
%     state-spece coefficients at some of the lags may evaluate to zero depending
%     on the current parameters.
% 
% * 'yVector' Returns [ cellstr ] the list of the measurement variables in
%     order of their appearance in the rows and columns of state-space matrices
%     (effectively identical to 'yList'). The state-space matrices can be
%     accessed by the |<a href="sspace.html">sspace</a>| function.
% 
% * 'xVector' Returns [ cellstr ] the list of the transition variables, and
%     their lags and leads, in order of their appearance in the rows and columns
%     of state-space matrices. The state-space matrices can be accessed by the
%     |<a href="sspace.html">sspace</a>| function.
% 
% * 'eVector' Returns [ cellstr ] the list of the shocks in order of their
%     appearance in the rows and columns of state-space matrices (effectively
%     identical to 'eList'). The state-space matrices can be accessed by the
%     |<a href="sspace.html">sspace</a>| function.
% 
% For more help on this function, type
%   <a href="matlab:idoc model.get">idoc model.get</a>

% The IRIS Toolbox.
% Copyright (c) 2007-2010 Jaromir Benes.
% End of header.

%**************************************************************************
% Function body.

% Store user input for reporting invalid properties.
userinput = varargin;

varargin = lower(strtrim(varargin));
% Alias fname, filename, file.
varargin = regexprep(varargin,'f(ile)?name','file');
% Alias dtrend, dtrends, dt.
varargin = regexprep(varargin,'dtrends?','dt');
% Alias ss, sstate, steadystate.
varargin = regexprep(varargin,'s(teady)?state','ss');
% Alias level, levels.
varargin = regexprep(varargin,'levels','level');
% Alias equation, equations, eqtn, eqtns.
varargin = regexprep(varargin,'eqtns?|equations?','eqtn');
% Alias list, name, names.
varargin = regexprep(varargin,'names?','list');
% Alias comment, comments, tag, tags.
varargin = regexprep(varargin,'comments?|tags','tag');
% Alias label, labels.
varargin = regexprep(varargin,'labels','label');
% Alias link, links.
varargin = regexprep(varargin,'links','link');
% Alias param, params, parameter, parameters.
varargin = regexprep(varargin,'param.*','param');
% Alias ss_dt, ss+dt.
varargin = regexprep(varargin,'_','+');

sslevel = [];
ssgrowth = [];
dtlevel = [];
dtgrowth = [];
level = [];
growth = [];
sstateList = { ...
   'ss','sslevel','level','ssgrowth','growth', ...
   'dt','dtlevel','dtgrowth', ...
   'ss_dt','sslevel_dtlevel','ssgrowth_dtgrowth', ...
};

varargout = {};
invalid = false(size(varargin));
for i = 1 : length(varargin)
   attrib = varargin{i};
   if isempty(sslevel) && any(strcmpi(attrib,sstateList))
      [sslevel,ssgrowth,dtlevel,dtgrowth,level,growth] = ...
         sstate_(this);
   end
   [varargout{i},flag] = get_(this,attrib, ...
      sslevel,ssgrowth,dtlevel,dtgrowth,level,growth);
   if ~flag
      invalid(i) = true;
   end
end

if any(invalid)
   error('iris:model', ...
      '\n*** Unrecognised model object attribute: ''%s''.', ...
      userinput{invalid});
end

end
% End of primary function.

%**************************************************************************
% Subfunction get_().
function [x,unrecognised] = get_(this,attrib, ...
   sslevel,ssgrowth,dtlevel,dtgrowth,level,growth)

   [ny,nx,nf,nb,ne,np,nalt] = size_(this);
   realsmall = getrealsmall();
   unrecognised = true;

   cell2struct_ = @(x,list) cell2struct( ...
      num2cell(permute(x,[2,3,1]),2), ...
      list(:),1);

   % check availability of solution
   chksolution = false;

   switch attrib

      case 'ss'
         x = cell2struct_(sslevel+1i*ssgrowth,this.name);

      case 'sslevel'
         x = cell2struct_(sslevel,this.name);

      case 'ssgrowth'
         x = cell2struct_(ssgrowth,this.name);

      case 'dt'
         index = this.nametype == 1;
         x = cell2struct_(dtlevel+1i*dtgrowth,this.name(index));

      case 'dtlevel'
         index = this.nametype == 1;
         x = cell2struct_(dtlevel,this.name(index));

      case 'dtgrowth'
         index = this.nametype == 1;
         x = cell2struct_(dtgrowth,this.name(index));

      case 'ss+dt'
         x = cell2struct_(level+1i*growth,this.name);

      case 'sslevel+dtlevel'
         x = cell2struct_(level,this.name);

      case 'ssgrowth+dtgrowth'
         x = cell2struct_(growth,this.name);

      case 'param'
         index = this.nametype == 4;
         x = cell2struct_( ...
            this.assign(1,index,:), ...
            this.name(index));

      case {'std'}
         [ans,stdvec] = omega_(this);
         index = find(this.nametype == 4);
         index = index(end-ne+1:end);         
         x = cell2struct_(stdvec,this.name(index));

      case {'eig','eigval','roots'}
         x = eig(this);

      case 'list'
         x = cell([1,4]);
         for i = 1 : 4
            x{i} = this.name(this.nametype == i);
         end

      case {'elist','xlist','ylist','plist'}
         type = find(attrib(1) == 'yxep');
         x = this.name(this.nametype == type);

      case 'stdlist'
         index = find(this.nametype == 4);
         index = index(end-ne+1:end);
         x = this.name(index);

      case 'rlist'
         x = {this.outside.lhs{:}};

      case {'ytag','xtag','etag','ptag'}
         index = find(attrib(1) == 'yxep');
         x = this.namelabel(this.nametype == index);

      case {'yeqtn','xeqtn'}
         index = find(attrib(1) == 'yx');
         x = this.eqtn(this.eqtntype == index);
         
      case {'deqtn'}
         x = this.eqtn(this.eqtntype == 3);
         x(cellfun(@isempty,x)) = [];
         
      case {'leqtn'}
         x = this.eqtn(this.eqtntype == 4);
         x = x(this.refresh);

      case 'reqtn'
         n = length(this.outside.rhs);
         x = cell([1,n]);
         for i = 1 : n
            x{i} = sprintf('%s=%s;', ...
               this.outside.lhs{i},this.outside.rhs{i});
         end
         % Remove references to database d from reporting equations.
         x = regexprep(x,'d\.([a-zA-Z])','$1');

      case 'rlabel'
         x = this.outside.label;

      case {'yvector','xvector','evector'}
         x = vector_(this,attrib(1));

      case {'log','islog'}
         x = struct();
         for i = find(this.nametype <= 3);
            x.(this.name{i}) = this.log(i);
         end
         
      case {'loglist'}
         x = this.name(this.log);

      case {'ylog','xlog','elog'}
         index = find(attrib(1) == 'yxe');
         x = this.log(this.nametype == index);

      case 'eqtn'
         x = cell([1,4]);
         for i = 1 : 2
            x{i} = this.eqtn(this.eqtntype == i);
         end
         nonemptyeqtn = ~cellfun(@isempty,this.eqtn);
         for i = 3 : 4
            x{i} = this.eqtn(this.eqtntype == i & nonemptyeqtn);
         end

      case 'label'
         x = cell([1,4]);
         for i = 1 : 2
            x{i} = this.eqtnlabel(this.eqtntype == i);
         end
         nonemptyeqtn = ~cellfun(@isempty,this.eqtn);
         for i = 3 : 4
            x{i} = this.eqtnlabel(this.eqtntype == i & nonemptyeqtn);
         end

      case {'xlabel','ylabel','dlabel','llabel'}
         index = find(attrib(1) == 'yxdl');
         if any(index == [1,2])
            x = this.eqtnlabel(this.eqtntype == index);
         else
            nonemptyeqtn = ~cellfun(@isempty,this.eqtn);
            x = this.eqtnlabel(this.eqtntype == index & nonemptyeqtn);
         end

      case 'tag'
         x = cell2struct(this.namelabel,this.name,2);

      case 'link'
         x = cell2struct(this.eqtn(this.eqtntype == 4),this.name,2);

      case {'diffuse','nonstationary','stationary','stationarylist','nonstationarylist'}
         chksolution = true;
         id = [this.solutionid{1:2}];
         t0 = imag(id) == 0;
         name = this.name(real(id(t0)));
         [ans,index] = isnan(this,'solution');
         status = nan([sum(t0),nalt]);
         for ialt = find(~index)
            % index of non-stationary x and y variables
            [dy,df,db] = isdiffuse(this.eigval(1,:,ialt),this.solution{4}(:,:,ialt),this.solution{1}(1:nf,:,ialt),this.solution{7}(:,:,ialt));
            d = [dy,df,db];
            if strncmp(attrib,'s',1)
               % Stationary.
               status(:,ialt) = transpose(double(~d(t0)));
            else
               % Non-stationary.
               status(:,ialt) = transpose(double(d(t0)));
            end
         end
         try
            status = logical(status);
         end
         if ~isempty(strfind(attrib,'list'))
            % List.
            if nalt == 1
               x = name(status == true | status == 1);
               x = x(:)';
            else
               x = cell([1,nalt]);
               for i = 1 : nalt
                  x{i} = name(status(:,i) == true | status(:,i) == 1);
                  x{i} = x{i}(:)';
               end
            end
         else
            % Database.
            x = cell2struct(num2cell(status,2),name(:),1);
         end

      case 'maxlag'
         x = min(imag(this.solutionid{2}(nf+1:end))) - 1;

      case {'icond','initcond','required'}
         key = this.solutionid{2}(nf+1:end);
         x = cell([1,nalt]);
         for ialt = 1 : nalt
            x{ialt} = vector_(this,key(this.icondix(1,:,ialt))-1i);
         end
         if nalt == 1
            x = x{1};
         end

      case {'forward'}
         ne = sum(this.nametype == 3);
         x = size(this.solution{2},2)/ne - 1;
         chksolution = true;

      case {'covmat','omega'}
         x = omega_(this);

      case {'stdvec'}
         [ans,x] = omega_(this);
         x = permute(x,[2,3,1]);

      case {'stableroots','unitroots','unstableroots'}
         switch attrib
            case 'stableroots'
               index = abs(this.eigval) < (1 - realsmall);
            case 'unstableroots'
               index = abs(this.eigval) > (1 + realsmall);
            case 'unitroots'
               index = abs(abs(this.eigval) - 1) <= realsmall;
         end
         x = nan(size(this.eigval));
         for ialt = 1 : nalt
            n = sum(index(1,:,ialt));
            x(1,1:n,ialt) = this.eigval(1,index(1,:,ialt),ialt);
         end
         x(:,all(isnan(x),3),:) = [];

      case {'nalt'}
         x = size(this.assign,3);

      case 'epsilon'
         x = this.epsilon;

      case {'torigin','baseyear'}
         x = this.torigin;

      case 'userdata'
         x = this.userdata;

      case {'activeshocks','inactiveshocks'}
         x = cell([1,nalt]);
         for ialt = 1 : nalt
            list = this.name(this.nametype == 3);
            stdvec = this.assign(1,end-sum(this.nametype == 3)+1:end,ialt);
            if attrib(1) == 'a'
               list(stdvec == 0) = [];
            else
               list(stdvec ~= 0) = [];
            end
            x{ialt} = list;
         end

      case 'file'
         x = this.fname;
         
      case 'nx'
         x = length(this.solutionid{2});
      case 'nb'
         x = size(this.solution{7},1);
      case 'nf'
         x = length(this.solutionid{2}) - size(this.solution{7},1);
      case 'ny'
         x = length(this.solutionid{1});
      case 'ne'
         x = length(this.solutionid{3});

      otherwise
         x = [];
         unrecognised = false;

   end

   if chksolution
      % Report solution(s) not available.
      [solutionflag,index] = isnan(this,'solution');
      if solutionflag
         warning_(44,sprintf(' #%g',find(index)));
      end
   end
end
% End of subfunction get_().

%**************************************************************************
% Subfunction sstate_().
function [sslevel,ssgrowth,dtlevel,dtgrowth,level,growth] = sstate_(this)
   [ny,nx,nf,nb,ne,np,nalt] = size_(this);
   islog = this.log;
   % Steady states.
   sslevel = real(this.assign);
   ssgrowth = imag(this.assign);
   % Fix missing (=zero) growth in steady states of log variables.
   ssgrowth(ssgrowth == 0 & islog(1,:,ones([1,nalt]))) = 1;
   % Retrieve dtrends.
   islog = islog(this.nametype == 1);
   [const,ttrend] = dtrendsrequest_('sstate',this);
   const = permute(const,[3,1,2]);
   ttrend = permute(ttrend,[3,1,2]);
   dtlevel = nan([1,ny,nalt]);
   dtlevel(1,~islog,:) = const(1,~islog,:);
   dtlevel(1,islog,:) = exp(const(1,islog,:));
   dtgrowth = nan([1,ny,nalt]);
   dtgrowth(1,~islog,:) = ttrend(1,~islog,:);
   dtgrowth(1,islog,:) = exp(ttrend(1,islog,:));
   % Steady state plus dtrends.
   index = find(this.nametype == 1);
   level = sslevel;
   level(1,index,:) = level(1,index,:) + dtlevel;
   growth = ssgrowth;
   index = find(this.log(this.nametype == 1));
   growth(1,index,:) = growth(1,index,:) .* dtgrowth(1,index,:);
   index = find(~this.log(this.nametype == 1));
   growth(1,index,:) = growth(1,index,:) + dtgrowth(1,index,:);
end
% End of subfunction sstate_().