function m = model(varargin)
% model:
% /class_library/@model/model.m
% ? Read model code file and create new model object.
% 
% Syntax:
%   this = model(fname,...)
% 
% Output arguments:
% 
% * |this| [ model ] A new model object based on one or more model code files.
% 
% Input arguments:
% 
% * |fname| [ char | cellstr ] Model code filename or a list of model code
%     filenames.
% 
% Optional input arguments:
% 
% * |'assign'| [ struct | *empty* ] Assign some or all of the parameter and/or
%     steady-state values from this database immediately when the model
%     objects is being created.
% * |'baseyear'| [ numeric | *2000* ] Base year for constructing
%     deterministic time trends.
% * |'epsilon'| [ numeric | *eps^(1/4)* ] Size of relative step for
%     numerical differentiation.
% * |'linear'| [ true | *false* ] Indicate linear models by setting this
%     option to true. More efficient solution procedures are use for linear
%     models.
% * |'std'| [ numeric | *1* for linear models | *0.01* for non-linear models ]
%     Default standard deviation for model shocks.
% * |'symbolic'| [ true | *false* ] Use the Symbolic Math Toolbox to
%     compute the Taylor expansion of model equations.
% * |'userdata'| [ anything | *empty* ] Attach any kind of user data to the
%     model object.
% 
% Description:
% 
% The |model| function reads the model code file |fname|, and translates the
% model code into a model object |m|. The model code file must be written in
% the <a href="../model_lang/index.html">model code language</a>.
% 
% If |fname| is a cell array of strings with more than one filenames, then
% all the files are merged together (in order of appearance), and the model
% object is based on such a combined model code.
% 
% Example 1:
% 
% Read in a model code file named |my.model|, and declare the model as
% linear:
% 
%   m = model('my.model','linear',true);
% 
% Example 2:
% 
% Read in a model code file named |my.model|, declare the model as linear,
% and assign some of the model parameters:
% 
%   m = model('my.model','linear',true,'assign',P);
% 
% Note that this is equivalent to
% 
%   m = model('my.model','linear',true);
%   m = assign(m,P);
% 
% unless some of the parameters passed in to the |model| fuction are needed
% to evaluate |<a href="if_else_end.html">if...else...end</a>| or |<a href="switch.html">switch...case...end</a>| expressions.
% 
% For more help on this function, type
%   <a href="matlab:idoc model.model">idoc model.model</a>

% The IRIS Toolbox.
% Copyright (c) 2007-2010 Jaromir Benes.
% End of header.

   % MODEL  Read model code file and create new model object.
   % For help on this function, type <a href="matlab:idoc('model.model')">idoc model.model</a>.

   % The IRIS Toolbox.
   % Copyright 2007-2010 Jaromir Benes.

%**************************************************************************
% Function body.

if nargin == 0
   m = empty_();
   m = class(m,'model',contained());
   return
end

if nargin == 1
   if ismodel(varargin{1})
      m = varargin{1};
      return
   elseif isstruct(varargin{1})
      m = empty_();
      list = fieldnames(m);
      for i = 1 : length(list)
         try
            m.(list{i}) = varargin{1}.(list{i});
         end
      end
      m = class(m,'model',contained());
      return
   end
end

if nargin > 0 && (ischar(varargin{1}) || iscellstr(varargin{1}))
   
   options = passvalopt('model.model',varargin{2:end});
   m = empty_();
   
   p = irisparser(varargin{1},options.assign,options.saveas);
   [m,a] = model(p,m,options);
   m = class(m,'model',contained());
   
   % Determine types of shocks.
   if any(isnan(m.shocktype))
      m = shocktype_(m);
   end
   
   % Differentiation step size.
   m.epsilon = options.epsilon;
   
   % Time origin (base year) for deterministic trends.
   if ~isempty(options.torigin)
      m.torigin = floor(options.torigin);
   end
   
   % Find recursive blocks.
   if ~m.linear
      m = reorder_(m);
   end
   
   % Creaete model-specific meta data.
   m = meta_(m,options);
   
   % Assign default stddevs.
   if ~isnan(options.std) && ~isempty(options.std)
      defaultstd = options.std;
   elseif m.linear
      defaultstd = 1;
   else
      defaultstd = 0.01;
   end
   
   % Pre-allocate solution matrices etc. Also assign zero steady states to
   % shocks and default stdevs.
   prealloc_();
   if ~isempty(a) ...
         && isstruct(a) ...
         && ~isempty(fieldnames(a))
      % Check number of alt params in input database. Exclude shocks.
      list = m.name(m.nametype ~= 3);
      maxlength = 1;
      for i = 1 : length(list)
         if isfield(a,list{i}) && isnumeric(a.(list{i}))
            a.(list{i}) = transpose(a.(list{i})(:));
            maxlength = max([maxlength,length(a.(list{i}))]);
         end
      end
      % Expand number of alt params if necessary.
      if maxlength > 1
         m = alter(m,maxlength);
      end
      m = assign(m,a);
   end
   
   % Convert the model's full equations to anonymous functions.
   m = eqtn2afcn_(m);
   
   % Pre-compute symbolic derivatives.
   if ~m.linear && options.symbolic && issymbolic()
      m = symbdiff(m,'simplify',options.simplify);
   end
   
   % Refresh dynamic links.
   if ~isempty(m.refresh)
      m = refresh(m);
   end
   
   return
end

error('Incorrect number or type of input argument(s).');

% End of function body.

%***********************************************************************
% Nested function empty_().
   function m = empty_()
      n = 1;
      m.fname = '';
      m.linear = false;
      m.assign = nan([1,0,n]);
      m.assign0 = m.assign;
      
      m.name = cell([1,0]);
      m.namelabel = cell([1,0]);
      m.nametype = zeros([1,0]);
      m.shocktype = zeros([1,0]);
      m.log = false([1,0]);
      
      m.eqtn = cell([1,0]);
      m.eqtnF = m.eqtn;
      m.eqtnS = m.eqtn;
      m.eqtnlabel = cell([1,0]);
      m.eqtntype = zeros([1,0]);
      m.deqtnF = cell([1,0]);
      m.outside = struct();
      m.refresh = zeros([1,0]);
      
      m.occur = sparse(false([0,0,n]));
      m.tzero = NaN;
      
      m.nameorder = zeros([1,0]);
      m.eqtnorder = zeros([1,0]);
      
      m.systemid = {cell([1,0]),cell([1,0]),cell([1,0])};
      m.metaderiv = struct();
      m.metasystem = struct();
      m.systemident = struct();
      m.metadelete = [];
      
      m.deriv0 = zeros([0,0,n]);
      m.system0 = struct();
      m.eigval = zeros([1,0,n]);
      % m.diffuse = {};
      m.epsilon = NaN;
      
      m.expand = {};
      m.solution = cell([1,7]);
      m.solution(1:7) = {zeros([0,0,n])};
      m.solutionid = cell([1,3]);
      m.solutionid(:) = {zeros([1,0,n])};
      m.icondix = false([1,0,n]);
      
      m.torigin = 2000;

   end
% End of nested function empty_().

%***********************************************************************
% Nested function prealloc_().
   function prealloc_()
      % Pre-allocate DERIV0 matrices.
      if issparse(m.occur)
         nt = size(m.occur,2)/length(m.name);
      else
         nt = size(m.occur,3);
      end
      nderiv = nt*sum(m.nametype <= 3);
      neqtn = sum(m.eqtntype <= 2);
      m.deriv0.c = zeros([neqtn,1]);
      m.deriv0.f = sparse(zeros([neqtn,nderiv]));
      
      % pre-allocate SYSTEM0 matrices
      ny = length(m.systemid{1});
      nx = length(m.systemid{2});
      ne = length(m.systemid{3});
      nf = sum(imag(m.systemid{2}) >= 0);
      nb = nx - nf;
      m.system0.K{1} = zeros([ny,1]);
      m.system0.K{2} = zeros([nx,1]);
      m.system0.A{1} = sparse(zeros([ny,ny]));
      m.system0.B{1} = sparse(zeros([ny,nb]));
      m.system0.E{1} = sparse(zeros([ny,ne]));
      m.system0.A{2} = sparse(zeros([nx,nx]));
      m.system0.B{2} = sparse(zeros([nx,nx]));
      m.system0.E{2} = sparse(zeros([nx,ne]));
      
      % Pre-allocate ASSIGN vectors.
      m.assign = nan([1,length(m.name)]);
      m.assign(m.nametype == 3) = 0;
      m.assign(end-sum(m.nametype == 3)+1:end) = defaultstd;
      m.assign0 = m.assign;
      
      % Pre-allocate SOLUTION and EXPAND matrices.
      ny = length(m.systemid{1});
      nx = length(m.systemid{2});
      nb = sum(imag(m.systemid{2}) < 0);
      nf = nx - nb;
      ne = length(m.systemid{3});
      fkeep = ~m.metadelete;
      nfkeep = sum(fkeep);
      m.solution{1} = nan([nfkeep+nb,nb]); % T
      m.solution{2} = nan([nfkeep+nb,ne]); % R
      m.solution{3} = nan([nfkeep+nb,1]); % K
      m.solution{4} = nan([ny,nb]); % Z
      m.solution{5} = nan([ny,ne]); % H
      m.solution{6} = nan([ny,1]); % D
      m.solution{7} = nan([nb,nb]); % U
      m.expand{1} = nan([nb,nf]);
      m.expand{2} = nan([nfkeep,nf]);
      m.expand{3} = nan([nf,ne]);
      m.expand{4} = nan([nf,nf]);
      m.expand{5} = nan([nf,nf]);
      
      % Preallocate EIGVAL and ICONDIX.
      m.eigval = nan([1,nx]);
      m.icondix = false([1,nb]);
   end
% End of nested function prealloc_().

end
% End of primary function.