function [m,deriv] = deriv_(m,eqselect,ialt)
% DERIV_  Compute first-order expansion of equations around current steady
% state for parameterisations.

% The IRIS Toolbox.
% Copyright 2007-2010 Jaromir Benes.

%**************************************************************************
% Function body.

realsmall = getrealsmall();
deriv = m.deriv0;
assign = m.assign(1,:,ialt);
nname = length(m.name);

if any(eqselect)
   nt = size(m.occur,2) / nname;
   nvar = sum(m.nametype <= 3);
   t = m.tzero;
   if ~isempty(m.deqtnF)
      % Symbolic derivatives available.
      numNeeded = false;
      sderiv_();
   else
      numNeeded = true;
   end
   % Numerical derivatives need to be computed.
   if numNeeded
      nderiv_();
   end
   % Normalise derivatives by largest number in non-linear models.
   if ~m.linear
      for eq = find(eqselect)
         index = deriv.f(eq,:) ~= 0;
         if any(index)
            norm = max(abs(deriv.f(eq,index)));
            deriv.f(eq,index) = deriv.f(eq,index) / norm;
         end
      end
   end   
end

if ialt == 1
   m.assign0(:) = m.assign(1,:,ialt);
   m.deriv0.c(:) = deriv.c;
   m.deriv0.f(:) = deriv.f;
end

% End of function body.





%**************************************************************************
% Nested function nderiv_().
% Numerical derivatives.
   function nderiv_() 
   
      mint = 1 - t;
      maxt = nt - t;
      tvec = mint : maxt;
      
      if m.linear
         init = zeros([1,nname]);
         init(1,m.nametype == 4) = real(assign(m.nametype == 4));
         init = init(1,:,ones([1,nt]));
         h = ones([1,nname,nt]);
      else
         init = trendarray_(m,1:nname,tvec,false,ialt);
         init = shiftdim(init,-1);
         h = abs(m.epsilon)*max([init;ones([1,nname,nt])],[],1);
      end
   
      plus = init + h;
      minus = init - h;
      step = plus - minus;
   
      if ~m.linear
         init(1,m.log,:) = exp(init(1,m.log,:));
         plus(1,m.log,:) = exp(plus(1,m.log,:));
         minus(1,m.log,:) = exp(minus(1,m.log,:));
      end
      
      % References to steady-state levels and growth rates.
      if ~m.linear
         L = init;
         G = imag(assign(1,:));
         G(m.log & G == 0) = 1;   
         G = G(:,:,ones([1,nt]));
      else
         L = [];
         G = [];
      end
   
      fgrid = cell([1,3]);
      fevaluate = cell([1,3]);
   
      for eq = find(eqselect)
         % Get occurences of variables in this equation.
         [tmocc,nmocc] = findoccur_(m,eq,'variables');
         % Find available symbolic derivatives.
         if isempty(m.deqtnF)
            symAvailableIndex = false(size(tmocc));
         else
            symAvailableIndex = ~cellfun(@isempty,m.deqtnF{eq});
         end
         if all(symAvailableIndex)
            continue,
         end
         tmocc = tmocc(~symAvailableIndex);
         nmocc = nmocc(~symAvailableIndex);
         % Total number of derivatives to be computed in this equation.
         n = length(nmocc);
         
         fgrid{2} = init;
         for i = [1,3]
            fgrid{i} = init(ones([1,n]),:,:);
         end
         for i = 1 : n
            fgrid{1}(i,nmocc(i),tmocc(i)) = minus(1,nmocc(i),tmocc(i));
            fgrid{3}(i,nmocc(i),tmocc(i)) = plus(1,nmocc(i),tmocc(i));
         end
   
         for i = [1,3]
            x = fgrid{i};
            fevaluate{i} = m.eqtnF{eq}(x,t,L,G);
         end
   
         % Constant in linear models.
         if m.linear
            x = fgrid{2};
            deriv.c(eq) = m.eqtnF{eq}(x,t,L,G);
         end
   
         value = zeros([1,n]);
         for i = 1 : n
            value(i) = (fevaluate{3}(i)-fevaluate{1}(i)) / step(1,nmocc(i),tmocc(i));
         end

         % Round numbers close to integers.
         roundIndex = abs(value - round(value)) <= realsmall;
         value(roundIndex) = round(value(roundIndex));

         % Assign values to the array of derivatives.
         index = (tmocc-1)*nvar + nmocc;
         deriv.f(eq,index) = value;
      end
   
   end
% End of nested function nderiv_().





%**************************************************************************
% Nested function sderiv_().
% Symbolic derivatives.
   function sderiv_() 
      
      
      if m.linear
         x = zeros([1,nname]);
         x(1,m.nametype == 4) = real(assign(m.nametype == 4));
         x = x(1,:,ones([1,nt]));
         % References to steady-state levels and growth rates.
         L = [];
         G = [];
      else
         mint = 1 - t;
         maxt = nt - t;
         tvec = mint : maxt;
         x = trendarray_(m,1:nname,tvec,true,ialt);
         x = shiftdim(x,-1);
         % References to steady-state levels and growth rates.
         L = x;
         G = imag(assign(1,:));
         G(m.log & G == 0) = 1;   
         G = G(:,:,ones([1,nt]));         
      end
      
      for eq = find(eqselect)
         % Get occurences of variables in this equation.
         [tmocc,nmocc] = findoccur_(m,eq,'variables');
         % Find available symbolic derivatives (NaN instead of function handle).
         symAvailableIndex = ~cellfun(@isempty,m.deqtnF{eq});
         % Indicate that some derivatives need to be computed numerically.
         if any(~symAvailableIndex)
            numNeeded = true;
         end
         if ~any(symAvailableIndex)
            continue,
         end   
         tmocc = tmocc(symAvailableIndex);
         nmocc = nmocc(symAvailableIndex);
         % Total number of derivatives to be computed in this equation.
         n = length(nmocc);
         % Constant in linear models. Becuase all variables are set to
         % zero, evaluating the equations gives the constant.
         if m.linear
            deriv.c(eq) = m.eqtnF{eq}(x,t,L,G);
         end
         % Evaluate all derivatives of the equation.
         value = cellfun(@(fcn) fcn(x,t,L,G), ...
            m.deqtnF{eq}(symAvailableIndex));
         % Assign values to the array of derivatives.
         index = (tmocc-1)*nvar + nmocc;
         deriv.f(eq,index) = value;
      end
      
   end  
% End of nested function sderiv_().

end
% End of primary function.