function [obj,varscalar,F,pe,delta,Pdelta,s,pred,smooth] = ...
   kalman_(m,data,range,ans,options)
% Kalman_  Kalman filter.

% The IRIS Toolbox.
% Copyright 2007-2010 Jaromir Benes.

[ny,nx,nf,nb,ne,np,nalt] = size_(m);
dopredict = nargout > 7;
dosmooth = nargout > 8;
dostoreped = nargout > 2;

% No need to compute k-step-ahead forecasts if they are not passed out.
% This can be the case if |estimate| is called with |'ahead'| option.
% K-step-ahead forecasts are then only computed once with estimated params.
if nargout < 4
   options.ahead = 1;
end


%**************************************************************************
% Function body.

realsmall = getrealsmall();

% User-supplied initial conditions.
if isstruct(options.initcond)
   [ainit,ans,ans,Painit] = datarequest('init',m,options.initcond,range);
end

% Out-of-lik params cannot be used with ~options.dtrends.
npout = length(options.outoflik);
if npout > 0 && ~options.dtrends
   error_(64);
end
s.npout = npout;

% Source data are always numeric arrays without initial condition.
% Add pre-sample initial condition.
ndata = size(data,3);
data = [nan([ny,1,ndata]),data];
range = range(1)-1 : range(end);
nper = length(range);
ahead = options.ahead;
if dosmooth
   if isempty(options.lastsmooth)
      s.lastsmooth = 1;
   else
      s.lastsmooth = round(options.lastsmooth - range(1)) + 1;
      if s.lastsmooth > 1
         % Include smoothing of pre-sample i/c.
         s.lastsmooth = s.lastsmooth - 1;
      elseif s.lastsmooth < 1
         s.lastsmooth = 1;
      end
   end
end

% The |'objectivesample'| options gives the range on which the objective
% function and out-of-lik parameters will be computed.
if any(isinf(options.objectivesample))
   options.objectivesample = [false,true([1,nper-1])];
else
   tmpstart = round(options.objectivesample(1) - range(1) + 1);
   if tmpstart < 2
      tmpstart = 2;
   end
   tmpend = round(options.objectivesample(end) - range(1) + 1);
   if tmpend > nper
      tmpend = nper;
   end
   options.objectivesample = false([1,nper]);
   options.objectivesample(tmpstart : tmpend) = true;
end

% Total number of cycles.
nloop = max([ndata,nalt]);

% Pre-allocation.
obj = nan([1,nloop]);
varscalar = nan([1,nloop]);
if dostoreped
   F = cell([1,2]);
   F(:) = {nan([ny,ny,nper-1,nloop])};
   pe = nan([ny,nper-1,ahead,nloop]);
else
   F = [];
   pe = [];
end
delta = nan([1,npout,nloop]);
Pdelta = nan([npout,npout,nloop]);
nandata = false([1,nloop]);

% Pre-allocate output datapacks.
if dopredict
   pred.mean_ = { ...
      nan([ny,nper,ahead,nloop]), ...
      nan([nx,nper,ahead,nloop]), ...
      nan([ne,nper,nloop]), ...
      range ,...
   };
   if ~options.meanonly
      pred.mse_ = { ...
         nan([ny,ny,nper,nloop]), ...
         nan([nx,nx,nper,nloop]), ...
         nan([ne,ne,nper,nloop]), ...
         range, ...
      };
   end
   if dosmooth
      nsmooth = nper - s.lastsmooth + 1;
      smooth.mean_ = { ...
         nan([ny,nsmooth,nloop]), ...
         nan([nx,nsmooth,nloop]), ...
         nan([ne,nsmooth,nloop]), ...
         range(s.lastsmooth:end), ...
      };
      if ~options.meanonly
         smooth.mse_ = { ...
            nan([ny,ny,nsmooth,nloop]), ...
            nan([nx,nx,nsmooth,nloop]), ...
            nan([ne,ne,nsmooth,nloop]), ...
            range(s.lastsmooth:end), ...
         };
      end
   end
end

%**************************************************************************
% Main loop.

for iloop = 1 : nloop
   
% Next model solution.

   if iloop <= nalt
      [s.T,s.R,s.k,s.Z,s.H,s.d,s.U] = sspace_(m,iloop,false);
      s.abseigval = abs(ordeig(s.T(nf+1:end,:)));
      s.nunit = sum(abs(s.abseigval - 1) <= realsmall);
      s.Tf = s.T(1:nf,:);
      s.Rf = s.R(1:nf,1:ne); % cutt off any forward expansion
      s.Ta = s.T(nf+1:end,:);
      s.Ra = s.R(nf+1:end,1:ne); % Cutt off any forward expansion.
      s.Tat = s.Ta.';
      s.Tft = s.Tf.';
      s.Zt = s.Z.';
      s.Rat = s.Ra.';
      s.Rft = s.Rf.';
      s.Ht = s.H.';
      if options.deviation
         s.k = [];
         s.ka = [];
         s.kf = [];
         s.d = [];
      else
         s.d = m.solution{6}(:,:,iloop);
         s.k = m.solution{3}(:,1,iloop);
         s.kf = s.k(1:nf);
         s.ka = s.k(nf+1:end);
      end

      % Get shock variances for each period by combining model and
      % user-supplied std database. Check if variances remain equal between
      % periods. The user-supplied |varvec| matrix is pre-processed in
      % |loglikopt_| and added to |options|.
      s.varvec = transpose(m.assign(1,end-ne+1:end,iloop).^2);
      % Set stdevs of non-linear shocks to zero.
      s.varvec(m.shocktype == 0) = 0;
      s.varvec = s.varvec(:,ones([1,nper]));
      if options.anyvarvec
         s.varvec(options.varvecindex) = options.varvec(options.varvecindex);
      end
      if any(any(isinf(s.varvec)))
         [s.varvec,s.excludeinf] = excludeinf_(s.varvec,s.H);
      else
         s.excludeinf = false([ny,nper]);
      end
      s.varveceq = [false, ...
         all(s.varvec(:,2:end) == s.varvec(:,1:end-1),1)];   
      % Deterministic trends.
      % y(t) - D(t) - X(t)*delta = Z*a(t) + H*e(t)      
      s.X = zeros([ny,npout,nper]);
      s.D = [];
      if npout > 0 || options.dtrends
         [s.D,s.X] = delta_(m,range,options.outoflik,iloop);
      end
      if ~options.dtrends
         s.D = [];
      end
      init_();
   end

   % Next data.
   if iloop <= ndata
      s.data = data(:,:,iloop);
   end
   
   % Adjust measurement variables for deterministic trends.
   s.y1 = s.data;
   if ~isempty(s.D)
      s.y1 = s.y1 - s.D;
   end
   s.y1(s.excludeinf) = NaN;
   s.yindex = ~isnan(s.y1);
   s.jyeq = [false,all(s.yindex(:,2:end) == s.yindex(:,1:end-1),1)];   
   nandata(iloop) = all(all(~s.yindex));
   
   % Number of initial conditions to be optimised.
   if strcmpi(options.initcond,'optimal')
      s.ninit = nb;
   else
      % Estimate diffuse initial conditions only if there is at least one
      % diffuse measurement variable.
      if any(any(abs(s.Z(:,1:s.nunit)) > realsmall))
         s.ninit = s.nunit;
      else
         s.ninit = 0;
      end
   end
   
   % Store prediction step either if user requests it as output, or if
   % user request k-step ahead forecasts.
   dostorepredict = dopredict || options.ahead > 1;
   
   % Run prediction error decomposition and evaluate user-requested
   % objective function.
   [obj(iloop),varscalar(iloop),s] = ...
      ped1_(s,options,dostoreped,dostorepredict,dosmooth);
   
   % Correct prediction step for estimated initial conditions and dtrends
   % parameters.
   if dostoreped
      s = correctpe_(s);
   end
   if dostorepredict && (s.ninit > 0 || npout > 0)
      s = correct_(s);
   end
   
   % Calculate predictin step for fwl variables.
   if dopredict
      s = filterxf_(s);
   end
   
   % K-step-ahead predictions.
   if options.ahead > 1 && dostorepredict
      s = ahead_(s,options,dopredict);
   end
      
   % Smoothing step for all variables.
   if dosmooth
      s.lastobs = find(any(s.yindex,1),1,'last');
      if ~options.meanonly
         s = smoothmse_(s);
      end
      s = smoothmean_(s);
      % Correct estimates of NaN observations by the effect of estimated
      % pouts.
      if npout > 0
         for t = s.lastobs : -1 : s.lastsmooth
            j = s.yindex(:,t);
            if any(~j)
               s.y2(~j,t) = s.y2(~j,t) + s.ydelta(~j,t);
            end
         end
      end      
   end

% Output arguments.

   if dopredict
      % Note that s.y0 includes all k-sted-ahead predictions.
      % Mean.
      pred.mean_{1}(:,2:end,:,iloop) = s.y0(:,2:end,:);
      if ~isempty(s.D)
         pred.mean_{1}(:,2:end,:,iloop) = ...
            pred.mean_{1}(:,2:end,:,iloop) + s.D(:,2:end,ones([1,ahead]));
      end
      pred.mean_{2}(1:nf,2:end,:,iloop) = s.f0(:,2:end,:);
      pred.mean_{2}(nf+1:end,:,:,iloop) = s.a0(:,:,:);
      pred.mean_{3}(:,2:end,iloop) = 0;
      if ~options.meanonly
         % MSE matrices for measurement variables.
         pred.mse_{1}(:,:,2:end,iloop) = s.F(:,:,2:end);
         % MSE matrices for transition variables.
         pred.mse_{2}(1:nf,1:nf,2:end,iloop) = s.Pf0(:,:,2:end);
         pred.mse_{2}(1:nf,nf+1:end,2:end,iloop) = s.Pfa0(:,:,2:end);
         pred.mse_{2}(nf+1:end,1:nf,2:end,iloop) = ...
            permute(s.Pfa0(:,:,2:end),[2,1,3]);
         pred.mse_{2}(nf+1:end,nf+1:end,:,iloop) = s.Pa0;
         % MSE matrices for shocks.
         pred.mse_{3}(:,:,:,iloop) = s.Pe0;
         if options.relative
            for i = 1 : 3
               pred.mse_{i}(:,:,:,iloop) = ...
                  pred.mse_{i}(:,:,:,iloop)*varscalar(iloop);
            end
         end
      end
         
      if dosmooth
         % Mean.
         t = s.lastsmooth+1 : nper;
         T = s.lastsmooth : nper;
         smooth.mean_{1}(:,2:end,iloop) = s.y2(:,t);
         if ~isempty(s.D)
            smooth.mean_{1}(:,2:end,iloop) = ...
               smooth.mean_{1}(:,2:end,iloop)+ s.D(:,t);
         end
         smooth.mean_{2}(1:nf,2:end,iloop) = s.f2(:,t);
         smooth.mean_{2}(nf+1:end,:,iloop) = s.a2(:,T);
         smooth.mean_{3}(:,2:end,iloop) = s.e2(:,t);
         if ~options.meanonly
            % MSE matrices for measurement variables.
            smooth.mse_{1}(:,:,2:end,iloop) = s.Py2(:,:,t);
            % MSE matrices for transition variables.
            smooth.mse_{2}(1:nf,1:nf,2:end,iloop) = s.Pf2(:,:,t);
            smooth.mse_{2}(1:nf,nf+1:end,2:end,iloop) = s.Pfa2(:,:,t);
            smooth.mse_{2}(nf+1:end,1:nf,2:end,iloop) = ...
               permute(s.Pfa2(:,:,t),[2,1,3]);
            smooth.mse_{2}(nf+1:end,nf+1:end,:,iloop) = s.Pa2(:,:,T);
            % MSE matrices for shocks.
            if options.relative
               for i = 1 : 3
                  smooth.mse_{i}(:,:,:,iloop) = ...
                     smooth.mse_{i}(:,:,:,iloop)*varscalar(iloop);
               end
            end
         end
      end
   end
   
   if dostoreped
      % |pe| includes all k-step-ahead prediction errors.
      pe(:,:,:,iloop) = s.pe(:,2:end,:);
      F{1}(:,:,:,iloop) = s.F(:,:,2:end);
      F{2}(:,:,:,iloop) = s.Fi(:,:,2:end);
      delta(1,:,iloop) = s.delta;
      Pdelta(:,:,iloop) = s.Pdelta;
      if options.relative
         F{1}(:,:,:,iloop) = F{1}(:,:,:,iloop)*varscalar(iloop);
         F{2}(:,:,:,iloop) = F{2}(:,:,:,iloop)/varscalar(iloop);
         Pdelta(:,:,iloop) = Pdelta(:,:,iloop)*varscalar(iloop);
      end      
   end

end
% End of main loop.

%**************************************************************************
% Post-mortem.

% Convert alpha to xb.
if dopredict
   pred = alpha2xb_(m,pred);
   if dosmooth
      smooth = alpha2xb_(m,smooth);
   end
end

% Fix negative variances occuring because of numerical inaccuracy.
if ~options.meanonly && dopredict
   pred.mse_{2}(:,:,:,:) = time_domain.fixcov(pred.mse_{2});
   if dosmooth
      smooth.mse_{2}(:,:,:,:) = time_domain.fixcov(smooth.mse_{2});
   end
end

if any(nandata)
   warning_(51,sprintf(' #%g',find(nandata)));
end

% End of function body.





%**************************************************************************
% Nested function init_().
   function init_()
      s.Painit = zeros(nb);      
      tmpstable = [false([1,s.nunit]),true([1,nb-s.nunit])];
      % Initialise MSE matrix.
      if isstruct(options.initcond)
         % User-supplied initial condition.
         s.Painit = Painit(:,:,1,min([end,iloop]));
      elseif nb > s.nunit && strcmpi(options.initcond,'stochastic')
         % Asymptotic initial condition.
         Rastable = s.Ra(tmpstable,:);
         Sa = Rastable*diag(s.varvec(:,1))*Rastable.';
         if sum(tmpstable) == 1
            Pa0stable = Sa / (1 - s.Ta(tmpstable,tmpstable)^2);
         else
            Pa0stable = time_domain.lyapunov(s.Ta(tmpstable,tmpstable),Sa);
            Pa0stable = (Pa0stable + Pa0stable.')/2;
         end
         s.Painit(tmpstable,tmpstable) = Pa0stable;
      end
      % Initialise mean.
      s.ainit = zeros([nb,1]);      
      if isstruct(options.initcond)
         % User-supplied initial condition.
         s.ainit = ainit(:,1,min([end,iloop]));
      elseif ~isempty(s.ka)
         % Asymptotic initial condition.
         s.ainit(tmpstable,1) = ...
         (eye(nb-s.nunit) - s.Ta(tmpstable,tmpstable)) ...
            \ s.ka(tmpstable,1);
      end
   end
% End of nested function init_().

end
% End of primary function.



%**************************************************************************
% Subfunction correctpe_().
% Correct prediction errors for estimated init and delta.
function s = correctpe_(s)
   est = [s.delta(:);s.init(:)];
   nper = size(s.y1,2);
   for t = 2 : nper
      j = s.yindex(:,t);
      s.pe(j,t) = s.pe(j,t) - s.M(j,:,t)*est;
   end
end
% End of subfunction correctpe_().





%**************************************************************************
% Subfunction correct_().
% Correct alpha, y, and F\pe for estimated init and delta.
function s = correct_(s)
   [ny,nper] = size(s.y1);
   init = s.init(:);
   delta = s.delta(:);
   est = [delta;init];
   s.ydelta = nan([ny,nper]);
   s.a0(:,1) = s.a0(:,1) + s.Q(:,:,1)*est;
   for t = 2 : nper
      j = s.yindex(:,t);
      s.Fipe(j,t) = s.Fi(j,j,t)*s.pe(j,t,1);
      correct = s.Q(:,:,t)*est; % s.Q1(:,:,t)*delta + s.Q2(:,:,t)*init;
      s.a0(:,t) = s.a0(:,t) + correct;
      s.ydelta(:,t) = s.X(:,:,t)*delta;
      % s.pe(jy,t,1) = s.pe(jy,t,1) - s.Z(jy,:)*correct - s.ydelta(jy,t);
   end
   s.y0 = s.Z*s.a0 + s.ydelta;
   if ~isempty(s.d)
      s.y0 = s.y0 + s.d(:,ones([1,nper]));
   end
end
% End of nested function correct_().





%**************************************************************************
% Subfunction ahead_().
% K-step ahead predictions and prediction errors for K>2
% when requested by caller. This function must be called after
% correction for diffuse initial conditions and/or out-of-lik params
% has been made.
function s = ahead_(s,options,dopredict)
   ahead = options.ahead;
   s.a0 = cat(3,s.a0,nan([size(s.a0),ahead-1]));
   s.pe = cat(3,s.pe,nan([size(s.pe),ahead-1]));
   s.y0 = cat(3,s.y0,nan([size(s.y0),ahead-1]));
   if dopredict
      % |s.f0| exists and needs to be calculated only if |pred| data are
      % requested.
      s.f0 = cat(3,s.f0,nan([size(s.f0),ahead-1]));
   end
   nper = size(s.a0,2);
   for k = 2 : min([options.ahead,nper-1]);
      t = 1+k : nper;
      tmpindex = ones([1,numel(t)]);
      s.a0(:,t,k) = s.Ta*s.a0(:,t-1,k-1);
      if ~isempty(s.ka)
         s.a0(:,t,k) = s.a0(:,t,k) + s.ka(:,tmpindex);
      end
      s.y0(:,t,k) = s.Z*s.a0(:,t,k);
      if ~isempty(s.d)
         s.y0(:,t,k) = s.y0(:,t,k) + s.d(:,tmpindex);
      end
      if dopredict
         s.f0(:,t,k) = s.Tf*s.a0(:,t-1,k-1);
         if ~isempty(s.kf)
            s.f0(:,t,k) = s.f0(:,t,k) + s.kf(:,tmpindex);
         end
      end
   end
   if s.npout > 0
      s.y0(:,:,2:end) = s.y0(:,:,2:end) + s.ydelta(:,:,ones([1,ahead-1]));   
   end
   s.pe(:,:,2:end) = s.y1(:,:,ones([1,ahead-1])) - s.y0(:,:,2:end);
end
% Subfunction ahead_().




%**************************************************************************
% Subfuntion geninv_().
function X = geninv_(A,k)
   if isempty(A)
     X = [];  
     return
   end
   m = size(A,1);   
   [U,S,V] = svd(A);
   s = diag(S);
   tol = max([k,m*eps(s(1))]);
   rank = sum(s > tol);
   if rank == 0
      X = zeros(m);
   else
      S = diag(1./s(1:rank));
      X = V(:,1:rank)*S*U(:,1:rank).';
   end
end
% End of subfunction geninv_().






%**************************************************************************
% Subfunction ped1_().
% Prediction error decomposition and evaluation of objective function.
function [obj,v,s] = ...
   ped1_(s,options,dostoreped,dostorepredict,dosmooth)

   % Size of system matrices.
   nb = size(s.Ta,1);
   ny = size(s.Z,1);
   nper = size(s.y1,2);
   npout = s.npout;
   ninit = s.ninit;
   nf = size(s.Rf,1);
   ne = size(s.Ra,2);
   
   Ta = s.Ta;
   Tat = s.Tat;
   ka = s.ka;
   d = s.d;
   Ra = s.Ra;
   Rat = s.Rat;
   Rf = s.Rf;
   H = s.H;
   Ht = s.Ht;
   jy = false([ny,1]);
   Z = s.Z(jy,:);
   X = s.X;

   varvec = s.varvec;
   varveceq = s.varveceq;
   y1 = s.y1;
   yindex = s.yindex;
   a = s.ainit;
   P = s.Painit;
   G = zeros([nb,0]);
   K = zeros([nb,0]);
   pe = zeros([0,1]);
   F = zeros(0);
   ZP = zeros([0,nb]);
   Fi = zeros(0);
   
   % Initialise objective function components.
   sumpeFipe = 0;
   sumlogdetF = 0;
   % Effect of outofliks and fixed init states on a(t).
   Q1 = zeros([nb,npout]);
   Q2 = eye([nb,ninit]);
   % Effect of outofliks and fixed init states on pe(t).
   M1 = zeros([0,npout]);
   M2 = zeros([0,ninit]);
   MtFi = [];
   sumMtFiM = 0;
   sumMtFipe = 0;
   
   % Initialise flags.
   reuse = false;
   jyeq = s.jyeq;
   include = false([0,1]);
   anypout = npout > 0;
   anyinit = ninit > 0;
   anyestimate = anypout || anyinit;
   anyexclude = true;
   
   % Initialise matrices that are to be stored.
   if dostoreped
      s.F = nan([ny,ny,nper]);
      s.Fi = nan([ny,ny,nper]);
      s.Fx = nan([ny,ny,nper]);
      s.Fd = nan([1,nper]);
      s.pe = nan([ny,nper]);
      s.M = nan([ny,npout+ninit,nper]);
      s.invsumMtFiM = nan(npout+ninit);
      s.MtFi = nan([npout+ninit,ny,nper]);
      if dostorepredict
         s.a0 = nan([nb,nper]);
         s.a0(:,1) = s.ainit;
         s.Pa0 = nan([nb,nb,nper]);
         s.Pa0(:,:,1) = s.Painit;
         s.Pa1 = s.Pa0;
         s.Pe0 = nan([ne,ne,nper]);
         s.y0 = nan([ny,nper]);
         s.G = nan([nb,ny,nper]);
         s.K = nan([nb,ny,nper]);
         s.ZtFi = nan([nb,ny,nper]);
         s.Fipe = nan([ny,nper]);
         s.Q = zeros([nb,npout+ninit,nper]);
         s.Q(:,npout+1:end,1) = Q2;
         s.Omg = nan([ne,ne,nper]);
         s.RaOmg = nan([nb,ne,nper]);
         s.HOmg = nan([ny,ne,nper]);
         if nf > 0
            s.RfOmg = nan([nf,ne,nper]);
         end
         if dosmooth
            s.L = nan([nb,nb,nper]);
            s.L(:,:,1) = Ta;
         end
      end
   end
   
   % Initialise reduced-form shock covariance.
   Omg = diag(varvec(:,1));
   Sa = Ra*Omg*Rat;
   Sy = H*Omg*Ht;
   
   % Determine tolerance on Kalman filter convergence.
   if ischar(options.tolmse)
      sing = svd(P);
      if sing == 0
         sing = 1e-2;
      end
      tolmse =  eps(sing(1))^(2/3);
   else
      tolmse = options.tolmse;
   end

   % Main loop.
   for t = 2 : nper

      % Effect of outofliks on a(t). This step must be made before updating
      % |include|.
      if anypout
         Q1 = Ta*Q1 - G(:,include)*M1;
      end
      
      % Effect of fixed init states on a(t). This step must be made before
      % updating |include|.
      if anyinit
         Q2 = Ta*Q2 - G(:,include)*M2;
      end
      
      % Prediction a(t|t-1) based on a(t-1|t-2), prediction error pe(t-1),
      % and the Kalman gain G(t-1).
      a = Ta*a + G*pe;
      if ~isempty(ka)
         a = a + ka;
      end
      
      % Index of available observations.
      jy = yindex(:,t);
      % Update index of observations included in loglik if needed.
      if ~jyeq(t)
         include = ~options.exclude(jy);
         anyexclude = any(~include);
      end

      % Update reduced-form shock covariance if needed. We cannot re-use
      % F, P, G in that case.
      if ~varveceq(t)
         Omg = diag(varvec(:,t));
         Sa = Ra*Omg*Rat;
         Sy = H*Omg*Ht;
         reuse = false;
      end      

      if ~reuse || ~jyeq(t)
         % Prediction MSE P(t|t-1) based on P(t-1|t-2) and the Kalman gain
         % G(t-1).
         P_ = P;
         P = (Ta*P - G*ZP)*Tat + Sa;
         % Make sure P is symmetric and does not explode over time.
         P = (P+P.')/2;
         % Time-varying Z matrix.
         Z = s.Z(jy,:);
         % Prediction MSE for observables F(t).     
         ZP = Z*P;
         PZt = ZP.';         
         F = Z*PZt + Sy(jy,jy);
         Fi = inv(F);
         K = PZt*Fi;
         G = Ta*K;
      end
      
      if ~reuse || ~jyeq(t)
         if anyexclude
            Fx = F(include,include);
            Fd = log(det(Fx));
         else
            Fd = log(det(F));
         end         
         reuse = false;
      end

      % Prediction error pe(t). The size of pe changes with jy.
      pe = y1(jy,t) - Z*a;
      if ~isempty(d)
         pe = pe - d(jy);
      end         

      % Test if F, P, G can be re-used. This can only work if the
      % composition of observables is the same across time.
      if ~reuse && jyeq(t)
         Pdiff = max(max(abs(P - P_)));
         reuse = Pdiff < tolmse;
      end
      
      if anyestimate
         % Update effect of outofliks and fixed init states on pe(t).
         M1 = Z(include,:)*Q1 + X(include,:,t);
         M2 = Z(include,:)*Q2;
         M = [M1,M2];
         Mt = M.';
      end
      
      if options.objectivesample(t)
         % Compute components of objective functions if user wants to
         % include this period of observations.
         if options.objective == 1
            % Likelihood function.
            if anyexclude
               % If some observations excluded, we must divide pred errors by a
               % submatrix of F.
               pex = pe(include);
               pext = pex.';
               sumpeFipe = sumpeFipe + pext*(Fx\pex);
            else
               % If all observations included, use existing Fi.
               pet = pe.';
               sumpeFipe = sumpeFipe + pet*Fi*pe;
            end
            sumlogdetF = sumlogdetF + Fd;
            % Estimation of pout and delta.
            if anyestimate
               if anyexclude
                  MtFi = Mt/Fx;
                  sumMtFipe = sumMtFipe + MtFi*pex;
               else
                  MtFi = Mt*Fi;
                  sumMtFipe = sumMtFipe + MtFi*pe;
               end
               sumMtFiM = sumMtFiM + MtFi*M;
            end
         elseif options.objective == 2
            % Weighted sum of prediction errors.
            pex = pe(include);
            pext = pe.';
            W = options.weighting(include,include);
            sumpeFipe = sumpeFipe + pext*W*pex;
            % Estimation of pout and delta.
            if anyestimate
               MtFi = Mt*W;
               sumMtFipe = sumMtFipe + MtFi*pex;
               sumMtFiM = sumMtFiM + MtFi*M;
            end
         end
      end
      
      if dostoreped
         if dostorepredict
            % Compute MSE for all measurement variables, not only for the
            % currently observed ones when predict data are returned.
            s.F(:,:,t) = s.Z*P*s.Z.' + Sy;
         else
            s.F(jy,jy,t) = F;
         end
         s.Fi(jy,jy,t) = Fi;
         s.pe(jy,t) = pe;
         if anyestimate
            s.M(jy,:,t) = M;
         end
         if dostorepredict
            storepredict_();
         end
      end

   end
   % End of main loop.

   % Total number of observations included in loglik.
   nobs = sum(sum(...
      yindex(:,2:end) & ~options.exclude(:,ones([1,nper-1]))...
      ));
   s.nobs = nobs;
   
   % Estimate outofliks and fixed init states, and adjust objective
   % function components.
   if anyestimate
      invsumMtFiM = geninv_(sumMtFiM,0);
      est = (invsumMtFiM * sumMtFipe).';
      sumpeFipe = sumpeFipe - est*sumMtFipe;
      s.delta = est(1:npout);
      s.Pdelta = invsumMtFiM(1:npout,1:npout);
      s.init = est(npout+1:end);
      if dostoreped
         s.invsumMtFiM = invsumMtFiM;
      end
   else
      s.delta = zeros([1,0]);
      s.init = zeros([1,0]);
      s.Pdelta = zeros(0);
   end
   
   % Estimate variance scalar. Can be only done when objective function is
   % likelihood.
   v = 1;
   if options.relative && options.objective == 1
      if nobs > 0
         v = sumpeFipe / nobs;
         sumpeFipe = nobs; % == sumpeFipe / v;
         sumlogdetF = sumlogdetF + nobs*log(v);  
      else
         sumpeFipe = 0;
      end
   end

   % Evaluate objective function.
   if options.objective == 1
      obj = (nobs*log(2*pi) + sumlogdetF + sumpeFipe)/2;
   else
      obj = sumpeFipe;
   end

   % End of subfunction ped1_() body.
   
   %***********************************************************************
   % Nested function storepredict_().
   % Store all information on prediction step.
   function storepredict_()
      s.a0(:,t) = a;
      s.Pa0(:,:,t) = P;
      s.Pa1(:,:,t) = P - K*ZP;
      s.Pe0(:,:,t) = Omg;
      s.y0(:,t) = s.Z*a;
      if ~isempty(d)
         s.y0(:,t) = s.y0(:,t) + d;
      end
      s.G(:,jy,t) = G;
      s.K(:,jy,t) = K;
      Zt = Z.';
      s.ZtFi(:,jy,t) = Zt*Fi;
      s.Fipe(jy,t) = Fi*pe;
      s.Q(:,:,t) = [Q1,Q2];
      s.Omg(:,:,t) = Omg;
      s.RaOmg(:,:,t) = Ra*Omg;
      s.HOmg(:,:,t) = H*Omg;
      if nf > 0
         s.RfOmg(:,:,t) = Rf*Omg;
      end
      if options.objectivesample(t) && anyestimate
         s.MtFi(:,jy,t) = MtFi;
      end
      if anyexclude
         s.Fx(include,include,t) = Fx;
      end
      s.Fd(t) = Fd;
      if dosmooth
         s.L(:,:,t) = Ta - G*Z;
      end
   end
   % End of nested function storepredict_().

end
% End of subfunction ped1_().





%**************************************************************************
% Subfunction filterxf_().
function s = filterxf_(s)
% Filter mean and MSE matrices of fwl transition variables.
   nper = size(s.y1,2);
   [nx,nb] = size(s.T);
   nf = nx - nb;
   % Pre-allocate state vectors.
   s.f0 = nan([nf,nper]);
   s.Pfa0 = nan([nf,nb,nper]);
   s.Pf0 = nan([nf,nf,nper]);
   if nf == 0
      return
   end
   for t = 2 : nper
      j = s.yindex(:,t);
      s.f0(:,t,1) = s.Tf*(s.a0(:,t-1) + s.K(:,j,t-1)*s.pe(j,t-1));
      if ~isempty(s.kf)
         s.f0(:,t,1) = s.f0(:,t,1) + s.kf;
      end
      TfPa1 = s.Tf*s.Pa1(:,:,t-1);
      s.Pfa0(:,:,t) = TfPa1*s.Tat + s.RfOmg(:,:,t)*s.Rat;
      s.Pf0(:,:,t) = TfPa1*s.Tft + s.RfOmg(:,:,t)*s.Rft;
      s.Pf0(:,:,t) = (s.Pf0(:,:,t) + s.Pf0(:,:,t).')/2;
   end
end
% End of subfunction filterxf_().





%**************************************************************************
% Subfunction smoothmse_().
function s = smoothmse_(s)
   [nx,nb] = size(s.T);
   nf = nx - nb;
   nper = size(s.y1,2);
   % Pre-allocation.
   s.Pa2 = s.Pa0;
   s.Pf2 = s.Pf0;
   s.Py2 = s.F;
   if nf > 0
      s.Pfa2 = s.Pfa0;
   else
      s.Pfa2 = zeros([nf,nb,nper]);
   end
   % Index of available observations.
   lastobs = find(any(s.yindex,1),1,'last');
   N = 0;
   for t = lastobs : -1 : s.lastsmooth
      j = s.yindex(:,t);
      N = s.ZtFi(:,j,t)*s.Z(j,:) + s.L(:,:,t).'*N*s.L(:,:,t);
      Pa0NPa0 = s.Pa0(:,:,t)*N*s.Pa0(:,:,t);
      s.Pa2(:,:,t) = s.Pa0(:,:,t) - Pa0NPa0;
      s.Pa2(:,:,t) = (s.Pa2(:,:,t)+s.Pa2(:,:,t).')/2;
      if nf > 0
         % Fwl transition variables.
         Pfa0N = s.Pfa0(:,:,t)*N;
         s.Pf2(:,:,t) = s.Pf0(:,:,t) - Pfa0N*s.Pfa0(:,:,t).';
         s.Pfa2(:,:,t) = s.Pfa0(:,:,t) - Pfa0N*s.Pa0(:,:,t);
         s.Pf2(:,:,t) = (s.Pf2(:,:,t)+s.Pf2(:,:,t).')/2;
         % Measurement variables.
         s.Py2(:,:,t) = s.F(:,:,t) - s.Z*Pa0NPa0*s.Z.';
         s.Py2(:,:,t) = (s.Py2(:,:,t)+s.Py2(:,:,t).')/2;
         s.Py2(j,:,t) = 0;
         s.Py2(:,j,t) = 0;
      end
   end
end
% End of subfunction smoothmse_().





%**************************************************************************
% Subfunction smoothmean_().
function s = smoothmean_(s)
   [nx,nb] = size(s.T);
   nf = nx - nb;
   ne = size(s.R,2);
   nper = size(s.y1,2);
   % Pre-allocation.
   s.a2 = s.a0;
   s.f2 = s.f0;
   s.e2 = zeros([ne,nper]);
   s.y2 = s.y1;
   % Last available observation.
   % Do not run the smoother beyond that point.
   lastobs = find(any(s.yindex,1),1,'last');
   s.y2(:,lastobs+1:end) = s.y0(:,lastobs+1:end,1);
   r = zeros([nb,1]);
   for t = lastobs : -1 : s.lastsmooth
      j = s.yindex(:,t);
      if any(j)
         s.e2(:,t) = s.e2(:,t) + s.HOmg(j,:,t).'*(s.Fipe(j,t) - s.G(:,j,t).'*r);
      end
      r = s.Zt(:,j)*s.Fipe(j,t) + s.L(:,:,t).'*r;
      s.a2(:,t) = s.a0(:,t,1) + s.Pa0(:,:,t)*r;
      s.e2(:,t) = s.e2(:,t) + s.RaOmg(:,:,t).'*r;
      % Back out NaN observables.
      if any(~j)
         s.y2(~j,t) = s.Z(~j,:)*s.a2(:,t) + s.H(~j,:)*s.e2(:,t);
         if ~isempty(s.d)
            s.y2(~j,t) = s.y2(~j,t) + s.d(~j,1);
         end
      end
   end
   % Fwl transition variables.
   if nf > 0
      s.f2(:,2:nper) = s.Tf*s.a2(:,1:nper-1) + s.Rf(:,:)*s.e2(:,2:nper);
      if ~isempty(s.kf)
         s.f2(:,2:nper) = s.f2(:,2:nper) + s.kf(:,ones([1,nper-1]));
      end
   end
end
% End of subfunction smoothmean_().