function data = resample(m,source,range,ndraw,varargin)
% To get help for this IRIS function
% * type <a href="matlab: idoc model.resample">idoc model.resample</a>, or
% * open <a href="matlab: helpbrowser">Matlab Product Help</a> and go the The IRIS Toolbox at the bottom of the Contents pane,

% The IRIS Toolbox.
% Copyright 2007-2010 Jaromir Benes.

if ~ismodel(m) || ~isnumeric(range) || ~isnumeric(ndraw) || ~iscellstr(varargin(1:2:end))
   error('Incorrect type of input argument(s).');
end

options = passvalopt('model.resample',varargin{:});
input = dataformat(source,true);
output = outputformat(input,options);

%********************************************************************
% Function body..

range = range(1) : range(end);
nper = length(range);
realsmall = getrealsmall();
[ny,nx,nf,nb,ne,np,nalt] = size_(m);
if nalt > 1
   error_(47,'<<matlab:idoc(''model.resample'') resample>>');
end

[T,R,K,Z,H,D,U,Omega,stdvec] = sspace_(m,1,false);
nunit = sum(abs(abs(m.eigval)-1) <= realsmall);
nstable = nb - nunit;
Ta = T(nf+1:end,:);
Ra = R(nf+1:end,:);
Ta2 = Ta(nunit+1:end,nunit+1:end);
Ra2 = Ra(nunit+1:end,:);
stdvec = permute(stdvec,[2,3,1]);

% Describe the distribution of initial conditions.
if strcmp(options.method,'montecarlo')
   % Monte Carlo.
   if strcmp(input,'struct')
      % User-supplied distribution.
      [Ea,Ex,ans,Pa] = datarequest('init',m,source,range);
      Ea1 = Ea(1:nunit,1);
      Ea2 = Ea(nunit+1:end,1);      
      Fa = time_domain.factorise(Pa);
   else
      % Asymptotic distribution.
      Ea = asymptoticMean_();
      Fa = zeros([nb,nb]);
      Pa2 = time_domain.acovf(Ta2,Ra2, ...
         [],[],[],[],[],Omega,m.eigval(nunit+1:end),0);
      Fa(nunit+1:end,nunit+1:end) = time_domain.factorise(Pa2);
      Ea1 = Ea(1:nunit,1);
      Ea2 = Ea(nunit+1:end,1);
   end
else
   % Bootstrap.
   if ~options.wild
   % Efron boostrap.
      sourceAlpha = datarequest('alpha',m,source,range);
   else   
   % Wild bootstrap.
      sourceAlpha0 = datarequest('init',m,source,range);  
      Ea = asymptoticMean_();
      Ea1 = Ea(1:nunit,1);
      Ea2 = Ea(nunit+1:end,1);
   end
end

% Describe the distribution of shocks.
if strcmp(options.method,'montecarlo')
   % Monte Carlo.
   stdvecnper = stdvec(:,ones([1,nper]));
else
   % Bootstrap.
   sourceE = datarequest('e',m,source,range);
end

% Pre-allocate output data.
data = { ...
   nan([ny,nper,ndraw]), ...
   nan([nx,nper,ndraw]), ...
   nan([ne,nper,ndraw]), ...
   range(1)-1:range(end), ...
};

% Pre-llocate initial conditions.
init = nan([nx,1,ndraw]);

if ~options.deviation
   W = dtrendsrequest_('range',m,range);
end

% Distinguish between transition and measurement residuals.
rindex = any(abs(R(:,1:ne)) > 0,1);
hindex = any(abs(H(:,1:ne)) > 0,1);

% Create a command-window progress bar.
if options.progress
   progress = progressBar('IRIS model.resample progress');
end

for idraw = 1 : ndraw
   data{3}(:,:,idraw) = getresiduals_();
   a0 = getinitcond_();
   % Transition variables.
   data{2}(:,1,idraw) = T*a0 + R(:,rindex)*data{3}(rindex,1,idraw);
   if ~options.deviation
      data{2}(:,1,idraw) = data{2}(:,1,idraw) + K;
   end
   for t = 2 : nper
      data{2}(:,t,idraw) = T*data{2}(nf+1:end,t-1,idraw) + R(:,rindex)*data{3}(rindex,t,idraw);
      if ~options.deviation
         data{2}(:,t,idraw) = data{2}(:,t,idraw) + K;
      end
   end
   % Measurement variables.
   data{1}(:,:,idraw) = Z*data{2}(nf+1:end,:,idraw) + H(:,hindex)*data{3}(hindex,:,idraw);
   if ~options.deviation
      data{1}(:,:,idraw) = data{1}(:,:,idraw) + D(:,ones([1,nper])) + W;
   end
   % Store initial condition.
   init(nf+1:end,1,idraw) = a0;
   % Update the progress bar.
   if options.progress
      progress = update(progress,idraw/ndraw);
   end
end

% Add pre-sample init cond.
data{1} = [nan([ny,1,ndraw]),data{1}];
data{2} = [init,data{2}];
data{3} = [nan([ne,1,ndraw]),data{3}];

% Convert alpha to xb;
data = alpha2xb_(m,data);

% Convert datapack to database if requested.
if strcmp(output,'dbase')
   data = dp2db(m,data);
end

% End of function body.

   %***********************************************************************
   % Nested function asymptoticMean_().
   function Ea = asymptoticMean_()
      Ea = zeros([nb,1]);
      if ~options.deviation
         Ka2 = K(nf+nunit+1:end);
         Ea(nunit+1:end) = (eye(nstable) - Ta2) \ Ka2;
      end
   end
   % End of nested function asymptoticMean_().

   %***********************************************************************
   % Nested function getresiduals_().
   function e = getresiduals_()
      % Resample residuals.
      if strcmp(options.method,'bootstrap')
         if options.wild
            % Wild bootstrap.
            draw = randn([1,nper]);
            % To reproduce input sample: draw = ones([1,nper]);
            e = sourceE.*draw(ones([1,ne]),:);
         else
            % Standard Efron bootstrap.
            % draw is uniform on [1,nper].
            draw = randi([1,nper],[1,nper]);
            % To reproduce input sample: draw = 0 : nper-1;
            e = sourceE(:,draw);
         end
      else
         % Draw shocks from standardised normal,
         % and scale them by std.
         e = stdvecnper.*randn([ne,nper]);
      end
   end
   % End of nested function getresiduals_().

   %***********************************************************************
   % Nested function getinitcond_().
   function a0 = getinitcond_()
      % Randomise initial condition for stable alpha.
      if strcmp(options.method,'empirical')
         % Bootstrap from empirical distribution.
         if options.randomise
            if options.wild
               % Wild-bootstrap initial condition for alpha from given sample
               % initial condition.
               a0 = [ ...
                  sourceAlpha0(1:nunit,1); ...
                  Ea2 + randn()*(sourceAlph0(nunit+1:end,1) - Ea2); ...
               ];
            else
               % Efron-bootstrap init cond for alpha from sample.
               draw = randi([1,nper],[1,nper]);
               a0 = sourceAlpha(:,draw);
            end
         else
            % Fix init cond to given pre-sample init cond.
            a0 = sourceAlpha0;
         end
      else
         % Gaussian Monte Carlo from theoretical distribution.
         if options.randomise
            a0 = Ea + Fa*randn([nb,1]);
         else
            % Fix initial conditions to mean.
            a0 = Ea;
         end
      end
   end
   % End of nested function getinitcond_().

end
% End of primary function.