function [data,nonlinflag] = simulate(m,data,range,varargin)
% To get help for this IRIS function
% * type <a href="matlab: idoc model.simulate">idoc model.simulate</a>, or
% * open <a href="matlab: helpbrowser">Matlab Product Help</a> and browse the The IRIS Toolbox documentation found in the Contents pane.

% The IRIS Toolbox.
% Copyright 2007-2010 Jaromir Benes.

p = inputParser();
p.addRequired('m',@ismodel);
p.addRequired('data',@(x) isstruct(x) || iscell(x));
p.addRequired('range',@isnumeric);
p.parse(m,data,range);

options = passvalopt('model.simulate',varargin{:});

if ischar(options.dtrends)
   options.dtrends = ~options.deviation;
end

% Determine output data format.
input = dataformat(data,true);
output = outputformat(input,options);

% Tunes.
istune = isplan(options.plan) && ~isempty(options.plan,'tunes');
% Nonlinear simulation.
isnonlin = isplan(options.plan) ...
   && ~isempty(options.plan,'nonlin') ...
   && options.maxiter > 0 ...
   && options.tolerance < Inf ...
   && any(options.lambda > 0);

%**************************************************************************
% Function body.

ny = size(m.solution{4},1);
nx = size(m.solution{1},1);
nb = size(m.solution{7},1);
nf = nx - nb;
ne = size(m.solution{2},2);
nalt = size(m.assign,3);
range = range(1) : range(end);
nper = length(range);

nanticipate = length(options.anticipate(:));
nignore = length(options.ignoreresiduals(:));

% Get initial condition for alpha.
% alpha is always expanded to match nalt within |datarequest|.
[ainit,xinit,naninit] = datarequest('init',m,data,range);
if ~isempty(naninit)
   if isnan(options.missing)
      naninit = unique(naninit);
      error_(25,naninit);
   else
      ainit(isnan(ainit)) = options.missing;
   end
end
ninit = size(ainit,3);

% Get y, current dates for [xf;xb], and e.
data = {...
   datarequest('y',m,data,range),...
   datarequest('x',m,data,range),...
   datarequest('e',m,data,range),...
};
ndata = size(data{2},3);

% Fix the 3rd dimension for measurement vars and shocks.
% This is because if there are no measurements and shocks,
% |datarequest| returns 0-by-nper-by-1 matrices.
if size(data{1},3) < ndata
   data{1} = cat(3,data{1},data{1}(:,:,end*ones([1,ndata-size(data{1},3)])));
end
if size(data{3},3) < ndata
   data{3} = cat(3,data{3},data{3}(:,:,end*ones([1,ndata-size(data{3},3)])));
end

% TODO: Add check n... == 1 or nloop.
nloop = max([nalt,ninit,ndata,nanticipate,nignore]);

% Expand initial conditions in 3rd dim to match nloop.
if ninit < nloop
   ainit(:,1,end+1:nloop) = ainit(:,1,end*ones([1,nloop-ndata]));
   xinit(:,1,end+1:nloop) = xinit(:,1,end*ones([1,nloop-ndata]));
end

% Expand data in 3rd dim to match nloop.
if ndata < nloop
   for i = 1 : 3
      data{i} = data{i}(:,:,[1:end,end*ones([1,nloop-ndata])]);
   end
end

if istune
   [yAnchors,xAnchors,eAnchorsReal,eAnchorsImag] = ...
      getanchors(options.plan,range);
   nanexog = chkexog_();
   % Some of the variables are exogenised to NaNs.
   if ~isempty(nanexog)
      error_(26,nanexog);
   end
else
   % Create empty anchors in case ~istune but isnonlin.
   yAnchors = false([ny,nper]);
   xAnchors = false([nx,nper]);
   eAnchorsReal = false([ne,nper]);
   eAnchorsImag = false([ne,nper]);
end

nonlinflag = true([1,nloop]);
if isnonlin
   % Settings for non-linear simulations.
   % * Passed in through the non-linear plan:
   % nonlinx = options.plan.nonlinx;
   nonline = options.plan.nonline;
   nonlinfunc = options.plan.nonlinfunc;
   nonlinlabel = options.plan.nonlinlabel;
   nonlinperiods = options.plan.nonlinperiods;
   % * Passed in through the |simulate| options:
   nonlinmaxiter = options.maxiter;
   nonlinlambda = options.lambda;
   nonlintol = options.tolerance;
   nonlindisplay = options.display;
   % * Initialise the following for reporting:
   % nonlinxtuneindex = [];
   % * Initialise the step-back variables.
   nonlinlambda = options.lambda(1);
   % maxdiscrep0 = 0;
end

% The 'contributions' option cannot be used with the 'plan' option
% or with multiple parameterisations.
if options.contributions
   if istune || isnonlin
      error_(51);
   end
   if nalt > 1
      error_(47,'<<matlab:idoc(''model.simulate'') simulate>> with ''contributions''');
   end
   % Expand data in 3rd dimension; will be filled with ne+1 contributions.
   for i = 1 : 3
      data{i} = data{i}(:,:,ones([1,ne+1]));
   end
end

% Find position of last logical true, or return zero.
lastOrZero_ = @(x) max([0,find(any(x,1),1,'last')]);

% Index of NaN solutions.
[flag,nansolution] = isnan(m,'solution');
% Index of NaN expansions.
[ans,nanexpansion] = isnan(m,'expansion');
% List of expansions not available, reported at the end.
cannotExpand = [];

%**************************************************************************
% Main loop.

use = struct();
for iloop = 1 : nloop

   if iloop <= nanticipate
      use.anticipate = options.anticipate(iloop);
   end
   
   % Anchors need to be reset because 'anticipate' can vary, and/or
   % nonlinear simulations can have changed them.
   if istune || isnonlin
      use.yanchors = yAnchors;
      use.xanchors = xAnchors;
      if use.anticipate
         use.eanchorsA = eAnchorsReal;
         use.eanchorsU = eAnchorsImag;
      else
         use.eanchorsA = eAnchorsImag;
         use.eanchorsU = eAnchorsReal;
      end         
      % Positions of
      % * last endogenised anticipated shock,
      % * last endogenised unanticipated shock,
      % * last exogenised variable.
      lastendogA = lastOrZero_(use.eanchorsA);
      lastendogU = lastOrZero_(use.eanchorsU);
      lastexog = lastOrZero_([use.yanchors;use.xanchors]);
      % If all endogenised shocks are unanticipated, and the number of them
      % equals the number of exogenised variables in each period, use fast
      % conditional simulation.
      use.fast = lastendogA == 0 ...
         && all(sum([use.yanchors;use.xanchors],1) == sum(use.eanchorsU,1));
   else
      lastendogA = 0;
      lastendogU = 0;
      lastexog = 0;
      use.fast = false;
   end
   
   if iloop <= nignore
      use.ignoreresiduals = options.ignoreresiduals(iloop);
   end
   
   if iloop <= nalt
      T = m.solution{1}(:,:,iloop);
      R = m.solution{2}(:,:,iloop);
      K = m.solution{3}(:,:,iloop);
      Z = m.solution{4}(:,:,iloop);
      H = m.solution{5}(:,:,iloop);
      D = m.solution{6}(:,:,iloop);
      U = m.solution{7}(:,:,iloop);
      for i = 1 : length(m.expand)
         use.expand{i} = m.expand{i}(:,:,iloop);
      end
      % Compute deterministic trends if requested.
      if options.dtrends
         W = dtrendsrequest_('range',m,range,iloop);
      end
      use.nanexpansion = nanexpansion(iloop);
      use.nansolution = nansolution(iloop);
   end

   % Nonlinear simulations.
   if isnonlin && options.deviation
      use.xbar = trendarray_(m, ...
         m.solutionid{2},0:nonlinperiods,false,min([iloop,nalt]));
   end

   % Where to place results in data{}.
   % Contributions simulation produces ne+1 pages.
   if options.contributions
      placeInData = 1 : ne+1;
   else
      placeInData = iloop;
   end
   
   % Solution not available.
   if use.nansolution
      data{1}(:,:,placeInData) = NaN;
      data{2}(:,:,placeInData) = NaN;
      continue
   end

   % Initial condition.
	a0 = ainit(:,1,iloop);
   
   % Get residuals,
   % and position of last anticipated residual.
   if use.ignoreresiduals
      e = zeros([ne,nper]);
   else
      e = data{3}(:,:,iloop);
   end
      
   if istune || isnonlin
      % Tunes on measurement variables.
      ytune = data{1}(:,:,iloop);
      if options.dtrends
         ytune = ytune - W;
      end
      % Tunes on transition variables.
      xtune = data{2}(:,:,iloop);
   end
   
   % Mutliplier matrices -- impact of shocks on the measurement and
   % transition variables.
   % * Anticipated shocks:
   MYA = zeros([ny*lastexog,ne*lastendogA]);
   MXA = zeros([nx*lastexog,ne*lastendogA]);   
   % * Unanticipated shock:.
   MYU = zeros([ny*lastexog,ne*lastendogU]);
   MXU = zeros([nx*lastexog,ne*lastendogU]);
   
   % While loop for non-linear simulations.
   nonlinstop = 0;
   nonlincount = 0;
   while nonlinstop == 0
      
      if use.anticipate
         use.lastResidA = lastOrZero_(real(e) ~= 0);
      else
         use.lastResidA = lastOrZero_(imag(e) ~= 0);
      end
      
      % Expand solution forward.
      if ne > 0
         % Current expansion is to t+k0.
         k0 = size(R,2)/ne - 1;
         % Expansion needed to t+k.
         k = max([1,use.lastResidA,lastendogA]) - 1;
         % Expand solution forward to t+k.
         if k > k0
            [R,use.expand{5}] = expand_(R,k,use.expand{1:5});
         end
      end
      if lastexog > 0
         if use.fast
            % Recursive (fast) conditional simulation.
            % Mutlipliers are computed inside |simulatemean|.
            f = struct();
            f.yanchors = use.yanchors;
            f.xanchors = use.xanchors;
            f.eanchorsU = use.eanchorsU;
            f.ytune = ytune;
            f.xtune = xtune;
            [y,w,e] = time_domain.simulatemean(...
               T,R,K,Z,H,D,U,...
               a0,e,nper,use.anticipate,options.deviation,f);
         else
            % Non-recursive conditional simulation.
            % Plain simulation first.
            [y,w] = time_domain.simulatemean(...
               T,R,K,Z,H,D,U,...
               a0,e,lastexog,use.anticipate,options.deviation,[]);
            neA = sum(sum(use.eanchorsA));
            neU = sum(sum(use.eanchorsU));
            % Compute multiplier matrices.
            % Size of MXA may change after first iteration in non-linear
            % simulations (but not after any later one).
            if nonlincount <= 1
               [MYA,MXA] = impact_(true,lastendogA);
               [MYU,MXU] = impact_(false,lastendogU);
            end
            % Back out residuals.
            [addeA,addeU] = exogenise_();
            % Add anticipated residuals.
            indexA = addeA ~= 0;
            indexU = addeU ~= 0;
            if use.anticipate
               e(indexA) = e(indexA) + addeA(indexA);
               e(indexU) = e(indexU) + addeU(indexU)*1i;
            else
               e(indexA) = e(indexA) + addeA(indexA)*1i;
               e(indexU) = e(indexU) + addeU(indexU);
            end
            % Re-simulate with residuals added.
            [y,w] = time_domain.simulatemean(...
               T,R,K,Z,H,D,U,...
               a0,e,nper,use.anticipate,options.deviation,[]);
         end
      else
         if ~options.contributions
            % Plain simulation.
            [y,w] = time_domain.simulatemean(...
               T,R,K,Z,H,D,U,...
               a0,e,nper,use.anticipate,options.deviation,[]);
         else
            % Compute contributions of shocks and init.cond.+constant
            e0 = e;
            e = zeros([size(e0),ne+1]);
            y = nan([ny,nper,ne+1]);
            w = nan([nx,nper,ne+1]); % := [xf;a]
            for i = 1 : ne
               e(i,:,i) = e0(i,:);
               [y(:,:,i),w(:,:,i)] = time_domain.simulatemean(...
                  T,R,K,Z,H,D,U,...
                  zeros(size(a0)),e(:,:,i),nper,use.anticipate,true,[]);
            end
            [y(:,:,ne+1),w(:,:,ne+1)] = time_domain.simulatemean(...
               T,R,K,Z,H,D,U,...
               a0,e(:,:,ne+1),nper,use.anticipate,options.deviation,[]);
         end
      end
      
      % Non-linear simulations.
      if isnonlin
         nonlinear_();
      else
         nonlinstop = 1;
      end
      
   end
   % End of nonlinear while.
   
   if nonlinstop == 3
      warning_(56);
   end
   
   % Add measurement detereministic trends.
   if options.dtrends
      % Add to simulation.
      % When contributions == true, add to last simulation.
      y(:,:,end) = y(:,:,end) + W;
   end
   
   % Store results.
   data{1}(:,:,placeInData) = y;
   data{2}(:,:,placeInData) = w;
   data{3}(:,:,placeInData) = e;
  
   % Store exit flag.
   nonlinflag(iloop) = nonlinstop;
   
end
% End of main loop.

%**************************************************************************
% Post-mortem.

% Convert alpha to xb.
data = alpha2xb_(m,data);

% Add initial conditions for xb to the datapack.
if options.contributions
   tmpinit = zeros([size(xinit),ne+1]);
   tmpinit(:,1,end) = xinit;
   data{1} = [nan([ny,1,ne+1]),data{1}];
   data{2} = [[nan([nf,1,ne+1]);tmpinit],data{2}];
   data{3} = [nan([ne,1,ne+1]),data{3}];
else
   data{1} = [nan([ny,1,nloop]),data{1}];
   data{2} = [[nan([nf,1,nloop]);xinit],data{2}];
   data{3} = [nan([ne,1,nloop]),data{3}];
end
data{4} = [range(1)-1,range];

% Convert datapack to database.
if strcmp(output,'dbase')
   data = dp2db(m,data);
   % Add shock names to contributions simulation.
   if options.contributions
      aux = [m.name(m.nametype == 3),{'Initial condition'}];
      for i = find(m.nametype <= 3)
         data.(m.name{i}) = comment(data.(m.name{i}),aux);
      end    
   end
   for i = find(m.nametype == 4)
      data.(m.name{i}) = permute(m.assign(1,i,:),[1,3,2]);
   end
end

% Solution not available.
if flag
   warning_(44,sprintf(' #%g',find(nansolution)));
end

% Expansion not available.
if ~isempty(cannotExpand)
   warning_(45,sprintf(' #%g',cannotExpand));
end

% End of function body.

%**************************************************************************
% Nested function chkexog_().
   function nanexog = chkexog_()
      % Check for NaNs in exogenised variables.
      nanexog = {};
      index1 = [yAnchors;xAnchors];
      index2 = [any(isnan(data{1}),3);any(isnan(data{2}),3)];
      index = any(index1 & index2,2);
      if any(index)
         temp = [vector_(m,'y'),vector_(m,'x')];
         nanexog = temp(index);
      end
   end  
% End of nested function chkexog_().

%**************************************************************************
% Nested function impact_().
   function [MY,MX] = impact_(anticipate,lastendog)
     MY = zeros([ny*lastexog,ne*lastendog]);
     MX = zeros([nx*lastexog,ne*lastendog]);
     if lastexog == 0 || lastendog == 0
        return
     end
     if anticipate
       RR = R(:,1:ne*lastendog);
       MX(1:nx,:) = RR;
     else
       RR = R(:,1:ne);
       MX(1:nx,1:ne) = RR;
     end
     for t = 2 : lastexog
       nonzero = find(any(MX((t-2)*nx+(1:nx),:),1));
       MX((t-1)*nx+(1:nx),nonzero) = T*MX((t-2)*nx+(nf+1:nx),nonzero);
       if anticipate
         RR = RR(:,1:end-ne);
         MX((t-1)*nx+(1:nx),(t-1)*ne+1:end) = MX((t-1)*nx+(1:nx),(t-1)*ne+1:end) + RR;
       elseif t <= lastendog
         MX((t-1)*nx+(1:nx),(t-1)*ne+(1:ne)) = MX((t-1)*nx+(1:nx),(t-1)*ne+(1:ne)) + RR;
       end
     end
     for t = 1 : lastexog
       MY((t-1)*ny+(1:ny),:) = Z*MX((t-1)*nx+(nf+1:nx),:);
       if t <= lastendog
          MY((t-1)*ny+(1:ny),(t-1)*ne+(1:ne)) = MY((t-1)*ny+(1:ny),(t-1)*ne+(1:ne)) + H;
       end
       nonzero = find(any(MX((t-1)*nx+(nf+1:nx),:),1));
       MX((t-1)*nx+(nf+1:nx),nonzero) = U*MX((t-1)*nx+(nf+1:nx),nonzero);
     end
   end
% End of nested function impact_().

%**************************************************************************
% Nested function exogenise_().
   function [addeA,addeU] = exogenise_()
      yindex = vec(use.yanchors(:,1:lastexog));
      xindex = vec(use.xanchors(:,1:lastexog));
      eindexA = vec(use.eanchorsA(:,1:lastendogA));
      eindexU = vec(use.eanchorsU(:,1:lastendogU));      
      % Convert [xf;a] vector to [xf;xb] vector.
      x = w;
      x(nf+1:end,:) = U*x(nf+1:end,:);
      % Compute "prediction errors".
      ype = [];
      xpe = [];
      for t = 1 : lastexog
         ype = [ype;ytune(use.yanchors(:,t),t)-y(use.yanchors(:,t),t)];
         xpe = [xpe;xtune(use.xanchors(:,t),t)-x(use.xanchors(:,t),t)];
      end
      M = [...
         MYA(yindex,eindexA),MYU(yindex,eindexU);...
         MXA(xindex,eindexA),MXU(xindex,eindexU)...
      ];
      adde = M \ [ype;xpe];
      addeA = zeros([ne,lastendogA]);
      addeU = zeros([ne,lastendogU]);
      neA = sum(eindexA);
      addeA(eindexA) = adde(1:neA);
      addeU(eindexU) = adde(neA+1:end);
   end
% End of nested function exogenise_().

%**************************************************************************
% Nested function nonlinear_().
   function nonlinear_()
      tt = 1 : nonlinperiods;
      xx = [[nan([nf,1]);a0],w(:,tt)];
      xx(nf+1:end,:) = U*xx(nf+1:end,:);
      %xactual = xx(nonlinx,1+tt);
      if options.deviation
         xx = xx + use.xbar;
      end
      ee = [zeros([ne,1]),real(e(:,tt))+imag(e(:,tt))];
      ee(nonline,:) = 0;
      pp = m.assign(1,m.nametype == 4,1);
      nn = length(nonlinfunc);
      xtarget = zeros([nn,nonlinperiods]);
      for i = 1 : nn
         xtarget(i,:) = nonlinfunc{i}(xx,ee,pp,1+tt,[],[]);
      end
      %if options.deviation
      %   xtarget = xtarget - use.xbar(nonlinx,1+tt);
      %end
      
      % Maximum discrepancy and max add factor.
      discrep = abs(xtarget); %abs(xactual - xtarget);   
      maxdiscrep2 = max(discrep,[],2);
      maxdiscrep = max(maxdiscrep2);
      maxdiscrepeqtn = findnaninf(maxdiscrep2,maxdiscrep,1,'first');
      maxaddfactor2 = max(abs(e(nonline,tt)),[],2);
      maxaddfactor = max(maxaddfactor2);
      maxaddfactoreqtn = findnaninf(maxaddfactor2,maxaddfactor,1,'first');
           
      if maxdiscrep <= nonlintol
         nonlinstop = 1;
      elseif nonlincount >= nonlinmaxiter;
         nonlinstop = 2;
      elseif isnan(maxdiscrep) || isinf(maxdiscrep)
         nonlinstop = 3;
      else
         use.fast = false;
         lastendogA = nper; nonlinperiods;
         % lastOrZero_(use.eanchorsA);
         % lastexog = nonlinperiods; % lastOrZero_([use.yanchors;use.xanchors]);
         % Tune non-lin LHS variables to the weighted average of the
         % previous simulation and the value of the non-linearised RHS.
         %{
         success = maxdiscrep < maxdiscrep0;
         if success
            nonlinlambda = min([options.lambda(2),nonlinlambda*1.5]);
         else
            nonlinlambda = max([options.lambda(1),nonlinlambda/1.5]);
         end
         xtune(nonlinx,tt) = ...
            (1-nonlinlambda)*xactual + nonlinlambda*xtarget;         
         maxdiscrep0 = maxdiscrep;
         % Tune only those data points where previous simulation differs
         % significantly from non-lin value.
         %}
         %nonlinxtuneindex = abs(xtarget - xactual) > nonlintol;
         %{
         use.xanchors(nonlinx,tt) = nonlinxtuneindex;
         use.eanchorsA(nonline,tt) = nonlinxtuneindex;
         %}
         temp = nonlinlambda*(-xtarget);% - xactual);
         %{
         nfill = nper - nonlinperiods;
         rho = temp(:,end) ./ temp(:,end-1);
         rho(:,2) = 0.8;
         rho = min(rho,[],2);
         for j = 2 : nfill
            rho(:,j) = rho(:,1).*rho(:,j-1);
         end
         %}
         if use.anticipate
            e(nonline,tt) = e(nonline,tt) + 1*temp;
         else
            e(nonline,tt) = e(nonline,tt) + 1i*temp;
         end
         %{
         e(nonline,nonlinperiods+1:end) = ...
            e(nonline,nonlinperiods*ones([1,nfill])).*rho;
         %}
      end
      % Report this iteration if requested.
      if nonlindisplay
         nonlinreport_( ...
            nonlincount,nonlinlambda,maxdiscrep,maxaddfactor,...
            nonlinlabel{maxdiscrepeqtn},...
            nonlinlabel{maxaddfactoreqtn},...
            nonlinstop);
      end       
      nonlincount = nonlincount + 1;
   end  
% End of nested function nonlinear_().


end
% End of primary function.

%**************************************************************************
% Subfunction nonlinreport_().
% Report nonlin simulation iteration.
function nonlinreport_(count,lambda,maxdiscrep,maxaddfactor,...
   maxdiscrepeqtn,maxaddfactoreqtn,nonlinstop)
   if count == 0
      fprintf('\t Iter\t\t Lambda\t Max discrep\tEqtn\t\t\t  Max add-factor\tEqtn\n');
   end         
   fprintf(...
      '\t%5g\t%10g\t%12g\t%s\t%12g\t%s\n',...
      count,lambda,maxdiscrep,padlabel_(maxdiscrepeqtn,16),...
      maxaddfactor,padlabel_(maxaddfactoreqtn,16));
   if nonlinstop ~= 0
      fprintf('\n');
   end
end
% End of subfunction nonlinreport_().

%**************************************************************************
% Subfunction padlabel_().
% Cut eqtn labels or add padding spaces to eqtn labels.
function label = padlabel_(label,n)
   nlabel = length(label);
   if nlabel > n-3
      label = [label(1:n-3),'...'];
   else
      label = [label,char(32*ones([1,n-nlabel]))];
   end
end
% End of subfunction padlabel_().