function m = symbdiff(m,varargin)
% SYMBDIFF  Evaluate symbolic derivatives for model equations.

% The IRIS Toolbox.
% Copyright 2007-2010 Jaromir Benes.

options = passvalopt('model.symbdiff',varargin{:});

%**************************************************************************
% Function body.

% Symbolic math toolbox not available.
if ~issymbolic()
   warning_(37);
   return
end

% Symbolic derivatives of
% * full measurement and transition equations wrt variables;
% * full dtrend equations wrt parameter.
% No derivatives computed for dynamic links.
neqtn = length(m.eqtn);
valid = true([1,neqtn]);
m.deqtnF = cell([1,neqtn]);
m.deqtnF(1:end) = {{}};
for eq = find(m.eqtntype <= 3)
   eqtn = func2str(m.eqtnF{eq});
   eqtn = strrep(eqtn,'@(x,t,L,G)','');
   eqtn = strrep(eqtn,'@(x,t,ttrend)','');
   eqtn = strrep(eqtn,'.*','*');
   eqtn = strrep(eqtn,'./','/');
   eqtn = strrep(eqtn,'.^','^');
   eqtn = strrep(eqtn,'+-','-'); % Symbolic Tbx does not like +-
   eqtn = strrep(eqtn,'-+','-'); % Symbolic Tbx does not like -+
   eqtn = strrep(eqtn,'++','+'); % Symbolic Tbx does not like ++
   eqtn = strrep(eqtn,'--','+'); % Symbolic Tbx does not like --
   if m.eqtntype(eq) <= 2
      % Measurement or transition equations.
      % Differentiate equations w.r.t. variables.
      % [tmocc,nmocc] = find(permute(m.occur(eq,m.nametype <= 3,:),[3,2,1]));
      [tmocc,nmocc] = findoccur_(m,eq,'variables');
   elseif m.eqtntype(eq) == 3
      % Deterministic trends.
      % Differentiate dtrends w.r.t. parameters.
      % [tmocc,nmocc] = find(permute(m.occur(eq,m.nametype == 4,m.tzero),[3,2,1]));
      [tmocc,nmocc] = findoccur_(m,eq,'parameters');
      nmocc = nmocc + sum(m.nametype <= 3);
      tmocc(:) = m.tzero;
   end
   nocc = length(nmocc);
   m.deqtnF{eq} = cell([1,nocc]);
   % Replace x(:,n,t+k) with xN, xNpK, or xNmK.
   eqtn = regexprep(eqtn,'([xLG])\(:,(\d+),t\)','$1$2');
   eqtn = regexprep(eqtn,'([xLG])\(:,(\d+),t\+(\d+)\)','$1$2p$3');
   eqtn = regexprep(eqtn,'([xLG])\(:,(\d+),t-(\d+)\)','$1$2m$3');
   for i = 1 : nocc
      if tmocc(i) == m.tzero
         % Time index == 0: replace x(1,23,t) with x23.
         unknown = sprintf('x%g',nmocc(i));
      elseif tmocc(i) > m.tzero
         % Time index >= 0: replace x(1,23,t+0) with x23p0.
         unknown = sprintf('x%gp%g',nmocc(i),round(tmocc(i)-m.tzero));
      else
         % Time index < 0: replace x(1,23,t-1) with x23m1.
         unknown = sprintf('x%gm%g',nmocc(i),round(m.tzero-tmocc(i)));
      end
      try
         deqtn = diff(eqtn,unknown);
         if m.log(nmocc(i))
            deqtn = deqtn * sym(unknown);
         end
         if length(char(deqtn)) > options.simplify
            deqtn = char(simple(horner(simple(deqtn))));
         else
            deqtn = char(deqtn);
         end
         % If the symbolic expression contains "diff(...)" or "[1]"
         % (depending on the version) it means that Symbolic Tbx does
         % not know the analytical derivates of some function(s)
         % occuring in the equation.
         if ~isempty(regexp(deqtn,'\<diff\>\(','once')) ...
               || ~isempty(strfind(deqtn,'['))
            % Throw an error to move to the catch block.
            error('Symbolic derivative cannot be evaluated.');
         end
         deqtn = strrep(deqtn,'*','.*');
         deqtn = strrep(deqtn,'/','./');
         deqtn = strrep(deqtn,'^','.^');   
         % Replace xN, xNpK, or xNmK back with x(:,N,t+/-K).
         deqtn = regexprep(deqtn,'([xLG])(\d+)p(\d+)','$1(:,$2,t+$3)');
         deqtn = regexprep(deqtn,'([xLG])(\d+)m(\d+)','$1(:,$2,t-$3)');
         deqtn = regexprep(deqtn,'([xLG])(\d+)','$1(:,$2,t)');
         % Store strings. These will be coverted to anonymous functions at
         % the end of this file.
         m.deqtnF{eq}{i} = deqtn;
      catch
         % Symbolic derivative not available.
         % Assign an empty numeric.
         m.deqtnF{eq}{i} = [];
         valid(eq) = false;
      end
   end
end

% Convert string derivatives to anonymous functions.
% Do this in a subfunction so that the anonymous functions' workspace does
% not include any variables created in the primary function. This could be
% a problem when there is a function name used in the model equations that
% conficts with some of those variables.
[m,valid] = deriv2afcn_(m,valid);

if any(~valid)
   warning_(52,m.eqtn(~valid));
end

end
% End of primary function.

%**************************************************************************
% Subfunction deriv2afcn_().
function [m,valid___] = deriv2afcn_(m,valid___)

% Symbolic derivatives of full equations wrt variables.
for i = find(m.eqtntype <= 2)
   for j = 1 : length(m.deqtnF{i})
      if ~ischar(m.deqtnF{i}{j})
         continue
      else
         if isempty(m.deqtnF{i}{j})
            m.deqtnF{i}{j} = @(x,t,L,G) 0;
         else
            try
               m.deqtnF{i}{j} = ...
                  eval(['@(x,t,L,G) ',m.deqtnF{i}{j}]);
            catch
               m.deqtnF{i}{j} = [];
               valid___(i) = false;
            end
         end
      end
   end
end

% Symbolic derivatives of full dtrends wrt parameters.
for i = find(m.eqtntype == 3)
   for j = 1 : length(m.deqtnF{i})
      if ~ischar(m.deqtnF{i}{j})
         continue
      else
         if isempty(m.deqtnF{i}{j})
            m.deqtnF{i}{j} = @(x,t,ttrend) 0;
         else
            try
               m.deqtnF{i}{j} = ...
                  eval(['@(x,t,ttrend) ',m.deqtnF{i}{j}]);
            catch
               m.deqtnF{i}{j} = [];
               valid___(i) = false;
            end
         end
      end
   end
end

end
% End of subfunction deriv2afcn_().