function forblocks = compile(s,outputfile,varargin)

if numel(varargin) == 1 && isstruct(varargin{1})
   options = varargin{1};
else
   options = passvalopt('sstate.compile',varargin{:});
end

options.outputfile = outputfile;
options.inputfile = s.fname;
clear('functions');

%**************************************************************************
% Function body.

% Blocks to evaluate.
if ischar(options.start)
   startIndex = find(strcmp(options.start,s.label),1);
   if isempty(startIndex)
      sstate.error(1,sprintf('''%s''',options.start));
   end
else
   startIndex = options.start;
   if startIndex ~= round(startIndex) ...
         || startIndex < 1 || startIndex > s.nblock
      sstate.error(1,sprintf('%g',options.start));
   end
end
if ischar(options.end)
   endIndex = find(strcmp(options.end,s.label),1);
   if isempty(endIndex)
      sstate.error(2,sprintf('''%s''',options.end));
   end
else
   % Use the |min| function to permit using Inf (for the last block).
   endIndex = min([options.end,s.nblock]);
   if endIndex ~= round(endIndex) ...
         || endIndex < 1 || endIndex > s.nblock
      sstate.error(2,sprintf('%g',options.end));
   end
end
forblocks = startIndex : endIndex;

% Use numerical solution instead of symbolic if Symbolic Math Tbx not
% installed or if requested by user.
index = strcmp('symbolic',s.type);
if any(index) && (~issymbolic || ~options.symbolic)
   s.type(index) = {'numerical'};   
   if options.symbolic
      warning('iris:sstate',...
         '*** The Symbolic Math Tbx not installed. Numerical solution will be used instead.');
   end
end

% Create subfunctions for each block.
subfun = cell([1,s.nblock]);
for i = forblocks
   % Write subfunction heading.
   code = sprintf('function varargout = block%g(varargin)\n',i);
   % Assign the i-th block's input values. Enclose in a try-end statement
   % so that an error is only generated when a missing input parameter is
   % really needed to evaluate the subsequent equations.
   for j = 1 : length(s.input{i})
      code = [code, ...
         sprintf('try\n   %s = varargin{1}.%s(varargin{2});\nend\n', ...
         s.input{i}{j},s.input{i}{j}), ...
      ];
   end
   % Assign growth rates.
   for j = 1 : length(s.growth{i})
      code = [code, sprintf('%s\n',s.growth{i}{j})];
   end
   switch s.type{i}
      case 'assignment'
         code = [code, ...
            assignment_(s.eqtn{i},s.solvefor{i})];        
      case 'symbolic'
         % Solve block symbolically.       
         [tmpcode,errormsg] = ...
            symbolic_(i,s.eqtn{i},s.solvefor{i},s.allbut(i),s.logs{i},options);
         if ~isempty(errormsg)
            warning('iris:sstate', ...
               '\n*** Error solving symbolically block #%g.\n*** %s\n*** Block will be solved numerically.', ...
               i,errormsg);
            tmpcode = ...
               numerical_(s.eqtn{i},s.solvefor{i},s.allbut(i),s.logs{i});
            s.type{i} = 'numerical';
         end   
         code = [code,tmpcode];
      case 'numerical'
         % Solve block numerically.
         code = [code, ...
            numerical_(s.eqtn{i},s.solvefor{i},s.allbut(i),s.logs{i})];
   end  
   % Write end of subfunction.
   code = [code,sprintf('end\n')];   
   subfun{i} = code;
end

% Write primary function.
code = '';
add(sprintf('function [P,discrep,exitflag] = %s(P,varargin)',regexprep(outputfile,'\..*','')));
add(sprintf('%% Steady-state file based on |%s|.',s.fname));
add(timestamp_());

% Create optimset.
add(sprintf('nblock = %g;',length(forblocks)));
add('optim = optimset(varargin{:});');
if ~isempty(s.input{1})
   list = sprintf('''%s'',',s.input{1}{:});
   list = list(1:end-1);
else
   list = '';
end

% List of input parameters.
add(['input = {',list,'};']);

% If input argument is a model object, create a database and pull out all
% input parameters.
add();
add('if ismodel(P)')
add('   outputformat = ''model'';');
add('   m = P;');
add('   P = struct();');
add('   for i = 1 : length(input)');
add('      P.(input{i}) = m.(input{i});');
add('   end');
add('else');
add('   outputformat = ''dbase'';');
add('end');

add();
add('[P,nalt] = getmaxlength(P,input);');
add('discrep = cell([1,nblock]);');
add('discrep(:) = {cell([1,nalt])};');
add('exitflag = cell([1,nblock]);');
add('exitflag(:) = {nan([1,nalt])};');
add('for i = 1 : nalt');
   
% Call individual blocks.
for i = forblocks
   if strcmp(s.type{i},'numerical')
      % Numerical.
      add(sprintf('   [P,discrep{%g}{i},exitflag{%g}(i)] = block%g(P,i,optim);',i,i,i));
   else
      % Symbolic or assignment.
      add(sprintf('   P = block%g(P,i);',i));
   end
end

% Write end of for loop.
add('end');

% If output format is model, assign P to the input model object, and return
% the model object as output.
add();
add('if strcmpi(outputformat,''model'')');
add('   P = assign(m,P);');
add('end');

% Write end of primary function.
add();
add('end');
add();
add();

div = sprintf('%%**************************************************************************\n');

% Write block-specific subfunctions.
for i = forblocks
   code = [code,div];
   code = [code,sprintf('%s\n',subfun{i})];
end

% Write |getmaxlength| helper function.
code = [code,div];
code = [code, ...
   grabtext('=== START OF GETMAXLENGTH ===','=== END OF GETMAXLENGTH ===')];

% Save output function.
char2file(code,outputfile);
rehash();

   function add(line)
      if nargin == 0
         line = '';
      end
      code = [code,line,sprintf('\n')];
   end

end
% End of primary function.

%**************************************************************************
% Subfunction assignment_().

function code = assignment_(eqtn,solvefor)
   % Write equations.
   code = '';
   for i = 1 : length(eqtn)
      code = [code,sprintf('%s;\n',eqtn{i})];
   end
   code = [code,sprintf('varargout{1} = varargin{1};\n')];
   % Output LHS variables.
   for i = 1 : length(solvefor)
      code = [code,sprintf('varargout{1}.%s(1,varargin{2}) = %s;\n',solvefor{i},solvefor{i})];
   end
end
% End of subfunction assignment_().

%**************************************************************************
% Subfunction symbolic_().

function [code,errormsg] = symbolic_(block,eqtn,solvefor,allbut,logs,options)

code = '';
errormsg = '';
lastwarn('','');
nsolvefor = numel(solvefor);

%aux = sprintf(',%s',solvefor{:});
%aux(1) = '';
[ans,ftitle] = fileparts(options.outputfile);
fname = sprintf('%s_block%g',ftitle,block);
positive = {};
for i = 1 : nsolvefor
   if sstate.islog(solvefor{i},logs,allbut)
      positive{end+1} = solvefor{i};
   end
end
savesymbolic_(block,fname,eqtn,solvefor,positive,options);
rehash();
func = eval(['@',fname]);
try
   x = func();
catch Error
   errormsg = sprintf('Matlab says ''%s''', ...
      strrep(Error.message,char(10),' '));
end
if options.deletesymbolicmfiles
   delete([fname,'.m']);
end
if ~isempty(errormsg)
   return
end
if ~isempty(lastwarn())
   errormsg = sprintf('Matlab says ''%s''', ...
      strrep(lastwarn(),char(10),' '));
   return
end

% Number of solutions returned.
nvar = numel(solvefor);
nsol = numel(x.(solvefor{1}));

% Convert sym solutions to char solutions.
s = cell([1,nsol]);
for i = 1 : nsol
   s{i} = cell([1,nvar]);
   for j = 1 : nvar
      tmp = x.(solvefor{j})(i);
      if numel(char(tmp)) >= options.simplify
         tmp = simple(horner(simple(tmp)));
      end
      s{i}{j} = char(tmp);
   end
end

% Exclude all-zero solutions.
if nsol > 1 && options.excludezero
   remove = false([1,nsol]);
   for i = 1 : nsol
      remove(i) = all(cellfun(@(x) strcmp(x,'0'),s{i}));
   end
   if any(remove)
      s(remove) = [];
      nsol = length(s);
      warning('iris:sstate', ...
         '\n*** A total of %g all-zero solution(s) discarded in block #%g.', ...
         sum(remove),block);
   end
end
   
if nsol == 0
   errormsg = sprintf( ...
      'Other than all-zero solutions not found for block #%g', ...
      block);
   return
end

if nsol > 1
   choose = multisolutions_(s,solvefor,block);
else
   choose = 1;
end
s = s{choose};

code = sprintf('varargout{1} = varargin{1};\n');
code = [code, ...
   writesolution_(s,solvefor,sprintf('\n'),'varargout{1}','(1,varargin{2})')];

end
% End of subfunction symbolic_().

%**************************************************************************
% Subfunction numerical_().

function code = numerical_(eqtn,solvefor,allbut,logs)

neqtn = length(eqtn);
nsolvefor = length(solvefor);

% Write initial values.
code = sprintf('X__ = zeros([1,%g]);\n',nsolvefor);
islog = false([1,nsolvefor]);
for i = 1 : nsolvefor
   islog(i) = sstate.islog(solvefor{i},logs,allbut);
   code = [code,sprintf('try \n')];
   if islog(i)
      code = [code, ...
         sprintf('  X__(%g) = log(varargin{1}.%s(1,varargin{2}));\n',i,solvefor{i})];
   else
      code = [code, ...
         sprintf('  X__(%g) = varargin{1}.%s(1,varargin{2});\n',i,solvefor{i})];
   end
   code = [code,sprintf('catch\n')];
   code = [code,sprintf('   try\n')];
   if islog(i)
      code = [code, ...
         sprintf('     X__(%g) = log(varargin{1}.%s(1,varargin{2}-1));\n',i,solvefor{i})];
   else
      code = [code, ...
         sprintf('     X__(%g) = varargin{1}.%s(1,varargin{2}-1);\n',i,solvefor{i})];
   end   
   code = [code,sprintf('   end\n')];
   code = [code,sprintf('end\n')];
end
code = [code,sprintf('varargout{1} = varargin{1};\n')];
code = [code,sprintf('[X__,varargout{2},varargout{3}] = fsolve(@solve__,X__,varargin{3});\n')];
for i = 1 : nsolvefor
   if islog(i)
      code = [code,sprintf('varargout{1}.%s(1,varargin{2}) = exp(X__(%g));\n',solvefor{i},i)];
   else
      code = [code,sprintf('varargout{1}.%s(1,varargin{2}) = X__(%g);\n',solvefor{i},i)];
   end
end

% Nested function called by |fsolve|.
code = [code,sprintf('\tfunction varargout = solve__(varargin)\n')];
for i = 1 : nsolvefor
   if islog(i)
      code = [code,sprintf('\t%s = exp(varargin{1}(%g));\n',solvefor{i},i)];
   else
      code = [code,sprintf('\t%s = varargin{1}(%g);\n',solvefor{i},i)];
   end
end

% Compute discrepancies in individual equations.
code = [code,sprintf('\tvarargout{1} = [\n')];
for i = 1 : neqtn
   aux = regexprep(eqtn{i},'(.*?)=(.*)','$1-($2)');
   code = [code,sprintf('\t%s\n',aux)];
end
code = [code,sprintf('\t];\n')];

% Write end of nested function
code = [code,sprintf('\tend\n')];

end
% End of subfunction numerical_().

%**************************************************************************
% Subfunction multisolutions_().

function choose = multisolutions_(s,solvefor,block)

html = grabtext('=== START OF HTML ===','=== END OF HTML ===');
html = strrep(html,'### BLOCK NUMBER HERE ###',sprintf('%g',block));

nsol = length(s);
code = '';
for i = 1 : nsol
   code = [code,sprintf('<div><div style="margin-top: 1em; font-size: large; font-weight: bold;">Solution #%g</div><pre style="font-family: Consolas,Monospace;">',i)];
   code = [code,writesolution_(s{i},solvefor,'<br/>','')];
   code = [code,sprintf('</pre></div>')];
end

html = strrep(html,'### SOLUTIONS HERE ###',code);
[ans,h] = web('-new');
set(h,'htmlText',html);

ask = true;
while ask
   choose = input(sprintf('Choose solution # (1 to %g) or press Ctrl+C to escape: ',nsol));
   ask = choose ~= round(choose) || choose < 1 || choose > nsol;
end
close(h);

end
% End of subfunction multisolutions_().

%**************************************************************************
% Subfunction writesolution_().
function code = writesolution_(s,solveFor,newLine,prefix,sufix)
   if nargin > 3 && ~isempty(prefix)
      prefix = [prefix,'.'];
   else
      prefix = '';
   end
   if nargin < 5
      sufix = '';
   end
   code = '';
   for i = 1 : length(solveFor)
      code = [code, ...
         sprintf('%s%s%s = %s;%s',prefix,solveFor{i},sufix,s{i},newLine)];
   end
end
% End of subfunction writesolution_().

%**************************************************************************
% Subfunction savesymbolic_().
function savesymbolic_(block,fname,eqtn,solvefor,positive,options)
   code = sprintf('function varargout = %s()\n',fname);
   code = [code,sprintf(...
      '%% Symbolic steady state block #%g of |%s|.\n', ...
      block,options.inputfile)];
   code = [code,timestamp_()];
   if ~isempty(positive)
      code = [code,'syms ', ...
         sprintf('%s ',positive{:}), ...
         'positive;',sprintf('\n')];
   end
   code = [code,sprintf('varargout{1} = solve( ...\n')];
   for i = 1 : numel(eqtn)
      code = [code,sprintf('''%s'', ...\n',eqtn{i})];
   end
   for i = 1 : numel(solvefor)
      code = [code,sprintf('''%s'',',solvefor{i})];
   end
  code = [code(1:end-1),sprintf(' ...\n);\n')];
  code = [code,sprintf('end\n')];
  char2file(code,[fname,'.m']);
end
% End of subfunction savesymbolic_().

%**************************************************************************
% Subfunction timestamp_().
function x = timestamp_()
   x = sprintf( ...
      '%% Automatically generated by The IRIS Toolbox, %s.\n', ...
      datestr(now()));
end
% End of subfunction timestamp_().

%{
=== START OF HTML ===
<html>
<body>
<div style="font-size: large; font-weight: bold; color: #900;">
Multiple solutions found in block ### BLOCK NUMBER HERE ###.</br>
Choose one of them by typing its number in command prompt.
</div>
### SOLUTIONS HERE ###
</body>
</html>
=== END OF HTML ===
%}

%{
=== START OF GETMAXLENGTH ===
function [P,maxlength] = getmaxlength(P,list)
nlist = numel(list);
n = nan([1,nlist]);
for i = 1 : nlist
   try
      P.(list{i}) = transpose(P.(list{i})(:));
      n(i) = length(P.(list{i}));
   end
end
maxlength = max([1,n(~isnan(n))]);
index = 1 : nlist;
index(n == maxlength) = [];
for i = index
   try
      P.(list{i}) = ...
         [P.(list{i}),P.(list{i})(end)*ones([1,maxlength-n(i)])];
   end
end
end
=== END OF GETMAXLENGTH ===
%}
