function x = arf(x,A,z,range,varargin)
% ARF  Run autoregressive filter on time series.

% The IRIS Toolbox.
% Copyright 2007-2009 Jaromir Benes.

if nargin < 4
   range = Inf;
end

if ~istseries(x) ...
      || ~isnumeric(A) ...
      || (~isnumeric(z) && ~istseries(z)) ...
      || ~isnumeric(range)
   error('Incorrect type of input argument(s).');
end

options = passvalopt('tseries.arf',varargin{:});

%**************************************************************************
% Function body.

A = A(:).';
order = length(A) - 1;

% Work out range (includes pre/post-sample initial condition).
xfirst = x.start;
xlast = x.start + size(x.data,1) - 1;
if isequal(range,Inf)
   range = xfirst : xlast;
else
   if strcmp(options.time,'forward')
      range = range(1)-order : range(end);
   else
      range = range(1) : range(end)+order;
   end
end

% Get endogenous data.
xdata = rangedata(x,range);
xsize = size(xdata);
xdata = xdata(:,:);

% Do noting if effective range is empty.
nper = length(range);
if nper <= order
   return
end

% Get exogenous (z) data.
if istseries(z)
   zdata = getdata_(z,range);
   zdata = zdata(:,:);
   % expand zdata in 2nd dimension if needed
else
   if isempty(z)
      z = 0;
   end
   zdata = z(ones([1,nper]),:);
end
if size(zdata,2) == 1 && size(xdata,2) > 1
   zdata = zdata(:,ones([1,size(xdata,2)]));
end

% Normalise polynomial vector.
if A(1) ~= 1
   zdata = zdata / A(1);
   A = A / A(1);
end

% Run AR.
if strcmp(options.time,'forward')
   shifts = -1 : -1 : -order;
   timevec = 1+order : nper;
else
   shifts = 1 : order;
   timevec = nper-order : -1 : 1;
end
for i = 1 : size(xdata,2)
   for t = timevec
      xdata(t,i) = -A(2:end)*xdata(t+shifts,i) + zdata(t,i);
   end
end

% Update output series.
xdata = reshape(xdata,[size(xdata,1),xsize(2:end)]);
x = setdata_(x,range,xdata);

end
% End of primary function.