function [x,t] = bpass(x,band,range,varargin)
%
% <a href="matlab: edit tseries/bpass">BPASS</a>  Christiano-Fitzgerald band-pass filter.
%
% Syntax:
%   x = bpass(x,range,band,...)
% Output arguments:
%   x [ tseries ] Band-pass filtered output series.
% Required input arguments:
%   x [ tseries ] Input series to be filtered.
%   range [ numeric | Inf ] Time range, i.e. <a href="dates.html">IRIS serial date numbers</a>.
%   band [ numeric ] Band of periodicities to be retained in output series: [low,high].
% <a href="options.html">Optional input arguments:</a>
%   'detrend' [ <a href="default.html">true</a> | false ] Remove linear time trend before filtering.
%   'log' [ true | <a href="default.html">false</a> ] Filter evaluated on log(x).
%   'ttrend' [ <a href="default.html">true</a> | false ] Add linear time trend back to filtered output series (only if band includes Inf).
%   'unitroot' [ <a href="default.html">true</a> | false ] Assume unit root in time series.
%
% Christiano, L.J. and T.J.Fitzgerald (2003). The Band Pass Filter.
% International Economic Review, 44(2), 435--465.
%
% See also the <a href="http://www.clevelandfed.org/research/model/bandpass/bpassm.txt">original Christiano-Fitzgerald m-file</a>.

% The IRIS Toolbox.
% Copyright 2007-2009 Jaromir Benes.

if nargin < 3
   range = Inf;
end
if ~isnumeric(band) || ~isnumeric(range)
   error('Incorrect type of input argument(s).');
end
if length(band) ~= 2 && length(range) == 2
   [range,band] = deal(band,range);
end
range = setrange(range,x.start:x.start+size(x.data,1)-1);

%**************************************************************************
% Function body.

if isempty(range) || isnan(x.start)
   x = empty(x);
   return
end

tmpsize = size(x.data);
data = getdata_(x,range);
data = data(:,:);
[data,tdata] = bpass_(data,band,varargin{:});
x.data = reshape(data,[size(data,1),tmpsize(2:end)]);
x.start = range(1);
tdata = reshape(tdata,[size(tdata,1),tmpsize(2:end)]);
t = replace(x,tdata);
x = cut_(x);
t = cut_(t);

end
% End of primary function.