function [y,range,freq,per] = fft(x,range,varargin)
% Discrete Fourier transform of time series.
% 
% Syntax:
%   [y,range,freq,per] = fft(x,range)
% Output arguments:
%   y [ numeric ] Fourier transform (column-wise).
%   range [ numeric ] Time range actually used.
%   freq [ numeric ] Frequencies corresponding to FFT vector elements.
%   per [ numeric ] Periodicities corresponding to FFT vector elements.
% Required input arguments:
%   x [ tseries ] Times series to be transformed.
%   range [ numeric | Inf ] Time range, i.e. <a href="dates.html">IRIS serial date numbers</a>.

% The IRIS Toolbox.
% Copyright 2007-2009 Jaromir Benes.

options = passvalopt('tseries.fft',varargin{:});

if nargin < 2
   range = Inf;
end

%**************************************************************************
% Function body.

tmpsize = size(x.data);

if isempty(range)
   y = zeros([0,tmpsize(2:end)]);
   return
end

if isequal(range,Inf)
   data = x.data;
   range = x.start + (0 : tmpsize(1)-1);
else
   range = range(1) : range(end);
   data = rangedata(x,range);
end
nper = length(range);

% Run Fourier.
y = fft(data(:,:));

% Back out frequencies.
freq = 2*pi*(0:nper-1) / nper;

% Convert frequencies to periodicities.
index = freq == 0;
per = nan(size(freq));
per(~index) = 2*pi./freq(~index);
per(index) = Inf;

if ~options.full
   % Return only data points within [0,pi].
   index = freq <= pi;
   freq = freq(index);
   y = y(index,:);
end
y = reshape(y,[size(y,1),tmpsize(2:end)]);

end
% End of primary function.