function [x,F] = hwfsf(x,band,range,varargin)
% <a href="matlab: edit tseries/hwfsf">HWFSF</a>  Iacobucci-Noullez H-windowed frequency-selective filter.
%
% Syntax:
%   [x,F] = hwbpf(x,range,band,...)
% Output arguments:
%   x [ tseries ] Filtered time series.
%   F [ numeric ] Spectral window weights.
% Required input arguments:
%   x [ tseries ] Time series to be filtered.
%   range [ numeric ] Time range, i.e. <a href="dates.html">IRIS serial date numbers</a>.
%   band [ numeric ] Spectral band to be retained.
% <a href="options.html">Optional input arguments:</a>
%   'detrend' [ <a href="default.html">true</a> | false ] Remove deterministic time trend.
%   'log' [ true | <a href="default.html">false</a> ] Filter evaluated on log(x).
%   'addtrend' [ <a href="default.html">true</a> | false ] Add detereministic trend to output series if band contains Inf and/or seasonal frequency.
%   'season' [ true | <a href="default.html">false</a> ] Include zero-sum seaonals into deterministic trend.
%   'window' [ <a href="default.html">'hamming'</a> | 'hanning' | 'none' ] Type of spectral window.
%
% Iacobucci, A. & A. Noullez (2005). A Frequency Selective Filter for Short-Length Time Series.
% Computational Economics, 25, 75--102.

% The IRIS Toolbox.
% Copyright 2007-2009 Jaromir Benes.

if nargin < 3
   range = Inf;
end

if ~isnumeric(band) || ~isnumeric(range)
  error('Incorrect type of input argument(s).');
end

if numel(band) ~= 2 && numel(range) == 2
   [band,range] = deal(range,band);
end


%**************************************************************************
% Function body.

if isempty(range)
  x = empty(x);
  F = [];
  return
end

tmpsize = size(x.data);

if isequal(range,Inf)
   data = x.data(:,:);
   range = x.start + (0 : tmpsize(1)-1);
else
   range = range(1) : range(end);
   data = rangedata(x,range);
   data = data(:,:);
end

[data,F] = hwfsf_(data,datfreq(x.start),band,varargin{:});
x.data = reshape(data,[size(data,1),tmpsize(2:end)]);
x.start = range(1);
x = cut_(x);

end
% End of primary function.