function x = interpmatch_(x,range1,freq1,freq2,options)
% Called from within tseries/convert. Convert low-frequency series to
% high-frequency series by interpolating data points while matching sum or
% average.

% The IRIS Toolbox.
% Copyright 2007-2009 Jaromir Benes.

%**************************************************************************
% Function body.

if isnan(x.start) && isempty(x.data)
   return
end
if isempty(range1)
   x = empty(x);
   return
end
if ~any(isinf(range1))
   range1 = range1(1) : range1(end);
end

n = freq2/freq1;
if n ~= round(n)
   error('iris:tseris',...
      'Source and target frequencies incompatible for ''%s'' interpolation.',...
      options.method);
end

[xdata,range1] = getdata_(x,range1);
xsize = size(xdata);
xdata = xdata(:,:);

[startyear1,startper1] = dat2ypf(range1(1));
[endyear1,endper1] = dat2ypf(range1(end));

startyear2 = startyear1;
endyear2 = endyear1;
% Find the earliest freq2 period contained (at least partially) in freq1
% start period.
startper2 = 1 + floor((startper1-1)*freq2/freq1);
% Find the latest freq2 period contained (at least partially) in freq1 end
% period.
endper2 = ceil((endper1)*freq2/freq1);
range2 = datcode(startyear2,startper2,freq2) : datcode(endyear2,endper2,freq2);

[xdata2,flag] = evaluate_(xdata,n);
if ~flag
   warning('iris:tseries',...
      'Cannot compute ''%s'' interpolation for series with within-sample NaNs.',...
      options.method);
end
if strcmpi(options.method,'quadavg')
   xdata2 = xdata2*n;
end

xdata2 = reshape(xdata2,[size(xdata2,1),xsize(2:end)]);
x.start = range2(1);
x.data = xdata2;
x = cut_(x);

end
% End of primary function.

%**************************************************************************
% Subfunction evaluate_().
function [y2,flag] = evaluate_(y1,n)

[nobs,ny] = size(y1);
y2 = nan([nobs*n,ny]);

t1 = (1 : n)';
t2 = (n+1 : 2*n)';
t3 = (2*n+1 : 3*n)';
M = [...
   n, sum(t1), sum(t1.^2);...
   n, sum(t2), sum(t2.^2);...
   n, sum(t3), sum(t3.^2);...
];

flag = true;
for i = 1 : ny
   y1i = y1(:,i);
   [samplei,flagi] = getsample(y1i');
   flag = flag && flagi;
   if ~any(samplei)
      continue
   end
   y1i = y1i(samplei);
   nobsi = numel(y1i);
   yy = [ y1i(1:end-2), y1i(2:end-1), y1i(3:end) ]';
   b = nan([3,nobsi]);
   b(:,2:end-1) = M \ yy;
   y2i = nan([n,nobsi]);
   for t = 2 : nobsi-1
      y2i(:,t) = b(1,t)*ones([n,1]) + b(2,t)*t2 + b(3,t)*t2.^2;
   end
   y2i(:,1) = b(1,2) + b(2,2)*t1 + b(3,2)*t1.^2;
   y2i(:,end) = b(1,end-1) + b(2,end-1)*t3 + b(3,end-1)*t3.^2;
   samplei = samplei(ones([1,n]),:);
   samplei = samplei(:);
   y2(samplei,i) = y2i(:);
end

end
% End of subfunction evaluate_().