function x = setdata_(x,s,y)

% Simplified syntax:
% setdata_(x,dates,y)
if ~isstruct(s)
   dates = s;
   s = struct();
   s.type = '()';
   s.subs{1} = dates;
end

% Expand LHS tseries object data with NaNs to comply with references.
% Remove the rows from dates that do not pass the frequency test.
[x,s,dates,freqtest] = expand_(x,s);

% Get RHS tseries object data.
if istseries(y)
   y = getdata_(y,dates);
end

% If RHS has only one row but multiple cols (or size > 1 in other dims),
% tseries is multivariate, and assigned are multiple dates, then expand RHS
% in 1st dimension.
xsize = size(x.data);
ysize = size(y);
if length(y) > 1 && size(y,1) == 1 ...
      && length(s.subs{1}) > 1 && any(xsize(2:end) > 1)
   n = length(s.subs{1});
   y = reshape(y(ones([1,n]),:),[n,ysize(2:end)]);
end

% Remove the rows from RHS that do not pass the frequency test.
ysize = size(y);
if ~all(freqtest) && ysize(1) == length(freqtest)
   y = y(freqtest,:);
   y = reshape(y,[size(y,1),ysize(2:end)]);
end

x.data = subsasgn(x.data,s,y);
if isempty(y)
   % If RHS is empty, then some of the columns have been probably deleted,
   % and the comments must be adjusted accordingly.
   x.comment = subsasgn(x.comment,s,y);
end
x = cut_(x);

end
% End of primary function.

%**************************************************************************
function [x,s,dates,freqtest] = expand_(x,s)

if length(s.subs) == 1
   % Replace x(dates) with x(dates,:,...,:).
   s.subs(2:ndims(x.data)) = {':'};
end

% Inf and ':' produce the entire tseries range.
% Convert subscripts in 1st dimension from dates to indices.
if isequal(s.subs{1},':') || isequal(s.subs{1},Inf)
   s.subs{1} = ':';
   if isnan(x.start)
      % LHS is empty.
      dates = [];
   else
      dates = x.start + (0 : size(x.data,1)-1);
   end
   freqtest = true(size(dates));
elseif isnumeric(s.subs{1}) && ~isempty(s.subs{1})
   dates = s.subs{1};
   if ~isempty(dates)
      if isnan(x.start)
         % If LHS series is empty set startdate to min(dates).
         x.start = min(dates);
      end
      f1 = x.start - floor(x.start);
      f2 = dates - floor(dates);
      freqtest = abs(f1 - f2) < 1e-2;
      dates(~freqtest) = [];
      s.subs{1} = round(dates - x.start + 1);
   end
else
   dates = [];
   freqtest = [];
end

% Add NaNs to data when user indices go beyond the data size.
% Add NaNs to 1st dimension when user indices are non-positive.
% Add empty strings for comments to comply with new size.
% This modifies standard Matlab matrix assignment, which produces zeros.
nsubs = length(s.subs);
for i = find(~strcmp(':',s.subs))
   % Non-positive index in 1st dimension.
   if i == 1 && any(s.subs{1} < 1)  
      n = 1 - min(s.subs{1});
      currentsize = size(x.data);
      currentsize(1) = n;
      x.data = [nan(currentsize);x.data];
      x.start = x.start - n;
      s.subs{1} = s.subs{1} + n;
   end
   % Index greater that the current size.
   if any(s.subs{i} > size(x.data,i))
      currentsize = size(x.data);
      currentsize(end+1:nsubs) = 1;
      addsize = currentsize;
      addsize(i) = max(s.subs{i}) - addsize(i);
      x.data = cat(i,x.data,nan(addsize));
      if i > 1
         % Add an appropriate empty cellstr to comments if tseries data are
         % expanded in 2nd or higher dimensions.
         comment = cell([1,addsize(2:end)]);
         comment(:) = {''};
         x.comment = cat(i,x.comment,comment);
      end
   end
end

end
% End of subfunction expand_().