function saved = dbsave(d,fname,dates,varargin)
% DBSAVE  Save database as CSV file.
%
% Syntax:
%   list = dbsave(d,fname)
%   list = dbsave(d,fname,dates,...)
% Output arguments:
%   list [ cellstr ] List of database entries that failed to be saved.
% Required input arguments:
%   d [ struct ] <a href="databases.html">Database</a> to be saved.
%   fname [ char ] CSV file name (including extension).
%   dates [ numeric | <a href="default.html">Inf</a> ] Time dates, i.e. <a href="dates.html">IRIS serial date numbers</a>.
% <a href="options.html">Optional input arguments:</a>
%  'class' [ <a href="default.html">true</a> | false ] Include row with Matlab class specification.
%  'comment' [ <a href="default.html">true</a> | false ] Include comments (time series fields only).
%  'decimal' [ numeric | <a href="default.html">empty</a> ] Number of decimal places (overrides option 'format').
%  'format' [ char | <a href="default.html">'%.8e'</a> ] Numeric format.
%  'freqletters' [ char | 'YZQBM' ] Letters to represent date frequencies (annual, semi-annual, quarterly, bimonthly, monthly).
%  'nan' [ char | <a href="default.html">'NaN'</a> ] Representation of NaNs.

% The IRIS Toolbox.
% Copyright 2007-2010 Jaromir Benes.

% Parse options.
options = passvalopt('data.dbsave',varargin{:});

% Run dates/datedefaults_ to substitute the default (irisget) date format
% options for 'config'.
options = datedefaults_(options);

% Remove double quotes from the date format string. This is because the
% double quotes are used to delimit the CSV cells.
[flag,options.dateformat] = strfun.findremove(options.dateformat,'"');
if flag
   warning('iris:data', ...
      '\n*** Double quotes removed from date format string.');
end

% Set up the formatting string.
if isempty(options.decimal)
   format = options.format;
else
   format = ['%.',sprintf('%g',options.decimal),'e'];
end

%**************************************************************************
% Function body..

% Allow both dbsave(d,fname) and dbsave(fname,d).
if ischar(d) && isstruct(fname)
   [d,fname] = deal(fname,d);
end

if nargin < 3 || isequal(dates,Inf)
   dates = dbrange(d);
else
   dates = dates(:)';
end

list = fieldnames(d)';

saved = {};
data = nan([length(dates),1]);
nameRow = {};
classRow = {};
commentRow = {};
savedIndex = false(size(list));

for i = 1 : numel(list)
   if istseries(d.(list{i}))
      tmpdata = d.(list{i})(dates);
      tmpcomment = comment(d.(list{i}));
      savedIndex(i) = true;
      tmpclass = 'tseries';
   elseif isnumeric(d.(list{i}))
      tmpdata = d.(list{i});
      tmpcomment = {''};
      savedIndex(i) = true;
      tmpclass = class(d.(list{i}));
   else
      continue
   end
   tmpdata = double(tmpdata);
   tmpsize = size(tmpdata);
   tmpdata = tmpdata(:,:);
   [tmprows,tmpcols] = size(tmpdata);
   if tmpcols == 0
      continue
   elseif tmpcols > 1
      tmpcomment(end+1:tmpcols) = {''};
   end
   % Add data, expand first dimension if necessary.
   nRows = size(data,1);
   if nRows < tmprows
      data(end+1:tmprows,:) = NaN;
   elseif size(data,1) > tmpsize(1)
      tmpdata(end+1:nRows,:) = NaN;
   end
   data = [data,tmpdata];
   nameRow{end+1} = list{i};
   classRow{end+1} = [tmpclass,sprintf('[%g]',tmpsize)];
   commentRow(end+(1:tmpcols)) = tmpcomment;
   if tmpcols > 1
      nameRow(end+(1:tmpcols-1)) = {''};
      classRow(end+(1:tmpcols-1)) = {''};
   end
end
data(:,1) = [];

if isempty(data)
   return
end
saved = list(savedIndex);

% Create an empty buffer.
c = '';
nl = sprintf('\n');

% Write name row.
c = [c,'""',sprintf(',"%s"',nameRow{:})];

% Write comments.
if options.comment
   c = [c,nl,'Comments',sprintf(',"%s"',commentRow{:})];
end

% Write classes.
if options.class
   c = [c,nl,'Class[Size]',sprintf(',"%s"',classRow{:})];
end

% Create the ceall array of date strings.
dates = dat2str(dates(:),options);
ndates = length(dates);

% Repeat the formatting strings and separate them by commas.
format = [',',format];
format = repmat(format,[1,size(data,2)]);

% Write the data row by row.
for i = 1 : size(data,1)
   if i <= ndates
      datestring = ['"',dates{i},'"'];
   else
      datestring = '""';
   end
   c = [c,nl,datestring,sprintf(format,data(i,:))];
end

% Save the buffer.
char2file(c,fname);

end
% End of primary function.