function string = dat2str(dat,varargin)
% DAT2STR  Convert series date numbers to cell array of strings.
%
% Syntax:
%   s = dat2str(dat,...)
% Output arguments:
%   s [ cellstr ] Cellstr with text representation of serial date number(s).
% Required input arguments:
%   dat [ numeric ] <a href="dates.html">IRIS serial date number(s).</a>
% <a href="options.html">Options:</a>
%   'dateformat' [ char | 'YYYYFP' ] Requested date format.
%   'freqletters' [ char | 'YZQBM' ] Letters to represent individual frequencies (annual,semi-annual,quarterly,bimontly,monthly).

% The IRIS Toolbox.
% Copyright 2007-2010 Jaromir Benes.

if nargin > 1 && isstruct(varargin{1})
   options = varargin{1};
   config = irisget();
else
   % Parse options.
   options = passvalopt('dates.dat2str',varargin{1:end});
   % Run dates/datedefaults_ to substitute the default (irisget) date format
   % options for 'config'.
   [options,config] = datedefaults_(options);
end

%**************************************************************************
% Function body.

if ischar(options.dateformat)
   options.dateformat = {options.dateformat};
end
ndateformat = numel(options.dateformat);

[year,per,freq] = dat2ypf(dat);
string = cell(size(year));
n = numel(year);

for k = 1 : n
   if k <= ndateformat
      dateformat = options.dateformat{k};
   end
   string{k} = buildstring_(year(k),per(k),freq(k),dateformat, ...
      options.freqletters,options.months,options.standinmonth, ...
      config.highcharcode);
end

end
% End of primary function.

%**************************************************************************
% Subfunction buildstring_()
function s = buildstring_(year,per,freq,dateformat, ...
   freqletters,months,standinmonth,offset)

if freq == 0
   varyear = '';
   longyear = '';
   shortyear = '';
else
   varyear = sprintf('%g',year);
   longyear = sprintf('%04g',year);
   if length(longyear) > 2
      shortyear = longyear(end-1:end);
   else
      shortyear = longyear;
   end
end

switch freq
   case 0
      freqletter = '';
      shortarabper = sprintf('%g',per);
      longarabper = sprintf('%02g',per);
      numericmonth = NaN;
      romanper = '';
   case 1
      freqletter = freqletters(1);
      shortarabper = '';
      longarabper = '';
      numericmonth = per2month_(per,1,standinmonth);
      romanper = '';
   case 2
      freqletter = freqletters(2);
      shortarabper = sprintf('%g',per);
      longarabper = sprintf('%02g',per);
      numericmonth = per2month_(per,2,standinmonth);
      romanper = roman_(per);
   case 4
      freqletter = freqletters(3);
      shortarabper = sprintf('%g',per);
      longarabper = sprintf('%02g',per);
      numericmonth = per2month_(per,4,standinmonth);
      romanper = roman_(per);
   case 6
      freqletter = freqletters(4);
      shortarabper = sprintf('%g',per);
      longarabper = sprintf('%02g',per);
      numericmonth = per2month_(per,6,standinmonth);
      romanper = roman_(per);
   case 12
      freqletter = freqletters(5);
      shortarabper = sprintf('%02g',per);
      longarabper = sprintf('%02g',per);
      numericmonth = per;
      romanper = roman_(per);
   otherwise
      freqletter = '?';
      shortarabper = sprintf('%g',per);
      longarabper = sprintf('%02g',per);
      numericmonth = NaN;
      romanper = '';
end

if isempty(numericmonth) || isnan(numericmonth) || ~isnumeric(numericmonth)
   longmonth = '';
   shortmonth = '';
else
   longmonth = months{numericmonth};
   shortmonth = longmonth(1:min([3,end]));
end
romanmonth = roman_(numericmonth);
varnumericmonth = sprintf('%g',numericmonth);
numericmonth = sprintf('%02g',numericmonth);

subs = { ...
   '%Y','Y'; ...
   '%P','P'; ...
   '%F','F'; ...
   '%f','f'; ...
   '%R','R'; ...
   '%r','r'; ...
   '%Q','Q'; ...
   '%q','q'; ...
   '%M','M'; ...
   '%m','m'; ...
   'YYYY',longyear; ...
   'YY',shortyear; ...
   'Y',varyear; ...
   'PP',longarabper; ...
   'P',shortarabper; ...
   'Q',romanmonth; ...
   'q',lower(romanmonth); ...
   'R',romanper; ...
   'r',lower(romanper); ...
   'F',upper(freqletter); ...
   'f',lower(freqletter); ...
   'Mmmm',longmonth; ...
   'Mmm',shortmonth; ...
   'mmmm',lower(longmonth); ...
   'mmm',lower(shortmonth); ...
   'MMMM',upper(longmonth); ...
   'MMM',upper(shortmonth); ...
   'MM',numericmonth; ...
   'mm',numericmonth; ...
   'M',varnumericmonth; ...
   'm',varnumericmonth; ...
   };

s = dateformat;
for i = 1 : size(subs,1)
   s = strrep(s,subs{i,1},char(offset+i));
end
for i = 1 : size(subs,1)
   s = strrep(s,char(offset+i),subs{i,2});
end

end
% End of subfunction buildstring_().

%**************************************************************************
% Subfunction roman_().
function x = roman_(x)
romans = { ...
   'I','II','III','IV','V','VI', ...
   'VII','VIII','IX','X','XI','XII', ...
};
try
   x = romans{x};
catch
   x = '';
end
end
% End of subfunction roman_().
