function dat = str2dat(string,varargin)
% <a href="dates/str2dat">STR2DAT</a>  Convert cell array of string(s) to IRIS serial date number(s).
%
% Syntax:
%   dat = str2dat(s,...)
% Output arguments:
%   dat [ numeric ] IRIS serial date numbers.
% Required input arguments:
%   s [ cellstr ] Cell array of strings with dates.
% <a href="options.html">Options:</a>
%   'dateformat' [ char | <a href="default.html">'YYYYFPP'</a> ] Format of date strings.
%   'freq' [ numeric \ <a href="default.html">empty</a> ] Enforce frequency.
%   'freqletters' [ char | <a href="default.html">'YZQBM'</a> ] Letters to represent frequencies: annual, semi-annual, quarterly, bimonthly, monthly.

% The IRIS Toolbox.
% Copyright 2007-2010 Jaromir Benes.

options = passvalopt('dates.str2dat',varargin{:});

% If the following options are empty numerics, the default setting from
% irisconfig is used.
config = irisget();
if isequal(options.dateformat,'config')
   options.dateformat = config.dateformat;
end
if isequal(options.freqletters,'config')
   options.freqletters = config.freqletters;
end
if isequal(options.months,'config')
   options.months = config.months;
end

%**************************************************************************
% Function body.

longmonthlist = sprintf('%s|',options.months{:});
longmonthlist(end) = '';
tmp = regexp(options.months,'\w{1,3}','match','once');
shortmonthlist = sprintf('%s|',tmp{:});
shortmonthlist(end) = '';
romanlist = 'xii|xi|x|ix|viii|vii|vi|v|iv|iii|ii|i|iv|v|x';
offset = config.highcharcode;

if ischar(string)
   string = {string};
end

if isempty(string)
   dat = nan(size(string));
   return
end

pattern = pattern_();
tokens = regexpi(string,pattern,'names','once');
[year,per,freq] = parsedates_(tokens,options);
dat = datcode(year,per,freq);

% Try indeterminate frequency.
index = find(isnan(dat(:).'));
for i = index
   aux = round(str2double(string{i}));
   if ~isempty(aux)
      dat(i) = aux;
   end
end

% End of function body.

   %***********************************************************************
   % Nested function pattern_().
   function x = pattern_()
      x = upper(options.dateformat);
      x = regexprep(x,'[\.\+\{\}\(\)]','\\$0');
      x = regexprep(x,'(?<!%)\*','.*');
      x = regexprep(x,'(?<!%)\?','.');
      subs = { ...
         '(?<!%)YYYY','(?<longyear>\\d{4})'; ...
         '(?<!%)YY','(?<shortyear>\\d{2})'; ...
         '(?<!%)Y','(?<longyear>\\d{0,4})'; ...
         '(?<!%)PP','(?<longperiod>\\d{2})'; ...
         '(?<!%)P','(?<shortperiod>\\d*)'; ...
         '(?<!%)F',sprintf('(?<freqletter>[%s])',options.freqletters); ...
         '(?<!%)MMMM',['(?<month>',longmonthlist,')']; ...
         '(?<!%)MMM',['(?<month>',shortmonthlist,')']; ...
         '(?<!%)MM','(?<numericmonth>\\d{1,2})'; ...
         '(?<!%)Q',['(?<romanmonth>',romanlist,')']; ...
         '(?<!%)R',['(?<romanperiod>',romanlist,')']; ...
         '(?<!%)I','(?<indeterminate>\\d+)'; ...
      };
      for i = 1 : size(subs,1)
         x = regexprep(x,subs{i,1},char(offset+i));
      end
      for i = 1 : size(subs,1)
         x = regexprep(x,char(offset+i),subs{i,2});
      end
      x = regexprep(x,'%([YFPMQRI])','$1');
   end
   % End of nested function pattern_().

end
% End of primary function.

%********************************************************************
% Subfunction parsedates_().

function [year,per,freq] = parsedates_(tokens,options)
   [thisyear,ans] = datevec(now());
   thiscentury = 100*floor(thisyear/100);
   freqvec = [1,2,4,6,12];
   freq = nan(size(tokens));
   % Set period to 1 by default so that e.g. YPF is correctly matched with
   % 2000Y.
   per = ones(size(tokens));
   year = nan(size(tokens));
   for i = 1 : length(tokens)
      if length(tokens{i}) ~= 1
         continue
      end
      if isfield(tokens{i},'indeterminate') ...
            && ~isempty(tokens{i}.indeterminate)
         freq(i) = 0;
         per(i) = str2num(tokens{i}.indeterminate);
         continue
      end
      if ~isempty(options.freq) && ( ...
            (isfield(tokens{i},'longmonth') && ~isempty(tokens{i}.longmonth)) ...
            || (isfield(tokens{i},'shortmonth') && ~isempty(tokens{i}.shortmonth)) ...
            || (isfield(tokens{i},'numericmonth') && ~isempty(tokens{i}.numericmonth)) )
         freq(i) = 12;
      end
      if isfield(tokens{i},'freqletter') && ~isempty(tokens{i}.freqletter)
         freqi = freqvec(upper(options.freqletters) == upper(tokens{i}.freqletter));
         if ~isempty(freqi)
            freq(i) = freqi;
         end
      end
      try
         yeari = str2double(tokens{i}.shortyear);
         yeari = yeari + thiscentury;
         if yeari - thisyear > 20
            yeari = yeari - 100;
         elseif yeari - thisyear <= -80
            yeari = yeari + 100;
         end
         year(i) = yeari;
      end
      try
         yeari = str2double(tokens{i}.longyear);
         if ~isempty(yeari)
            year(i) = yeari;
         end
      end
      try
         per(i) = str2double(tokens{i}.shortperiod);
      end
      try
         per(i) = str2double(tokens{i}.longperiod);
      end
      try
         per(i) = roman2num_(tokens{i}.romanperiod);
      end
      month = NaN;
      try
         month = roman2num_(tokens{i}.romanmonth);
      end
      try
         month = str2double(tokens{i}.numericmonth);
      end
      try
         index = strncmpi(tokens{i}.month,options.months,length(tokens{i}.month));
         if any(index)
            month = find(index,1);
         end
      end      
      if ~isnumeric(month) || length(month) ~= 1 || isinf(month)
         month = NaN;
      end
      if ~isempty(options.freq)
         freq(i) = options.freq;
      end      
      if ~isnan(month)
         if ~isnan(freq(i)) && freq(i) ~= 12
            %aux = 12 / freq(i);
            %per(i) = 1 + floor(round(month-1) / round(aux));
            per(i) = ceil(month.*freq(i)./12);
         else
            per(i) = month;
            freq(i) = 12;
         end
      end
      % Disregard periods for annul dates. This is now also consistent with
      % the YY function.
      if freq(i) == 1
         per(i) = 1;
      end
   end
   % Try to guess freq by the highest period found in all the dates passed
   % in.
   if all(isnan(freq))
      maxper = max(per(~isnan(per)));
      if ~isempty(maxper)
         index = find(maxper <= freqvec,1,'first');
         if ~isempty(index)
            freq(:) = freqvec(index);
         end
      end
   end
end
% End of subfunction parsedates_().

%**************************************************************************
% Subfunction roman2num_().
function per = roman2num_(romanper)
   per = 1;
   list = {'i','ii','iii','iv','v','vi','vii','viii','ix','x','xi','xii'};
   index = strcmpi(romanper,list);
   if any(index)
      per = find(index,1);
   end
end
% End of subfunction roman2num_().