function [options,varargout] = passvalopt(spec,varargin)
% PASSVALOPT  Pass in and validate optional arguments. Initialise and
% permanently store default options for IRIS functions. If called with two
% output arguments, it passes out unused option names-values and does not
% throw a warning.

% The IRIS Toolbox.
% Copyright 2007-2010 Jaromir Benes.

persistent Default;

if nargin == 0 || isempty(Default)
   % Initialise default options, and store them as persistent.
   munlock();
   Default = struct();
   Default.data = default_data();
   Default.dates = default_dates();
   Default.FAVAR = default_FAVAR();
   Default.latex = default_latex();
   Default.model = default_model();
   % Default.plan = default_plan();
   Default.quickreport = default_quickreport();
   Default.tseries = default_tseries();
   Default.VAR = default_VAR();
   Default.utils = default_utils();
   Default.report = struct();
   Default.sstate = default_sstate();
   Default.iris = default_IRIS();
   Default.irisparser = default_irisparser();
   mlock();
   if nargout == 0
      return
   end
end

%**************************************************************************
% Function body.

if ischar(spec)
   index = findstr(spec,'.');
   spec = Default.(spec(1:index-1)).(spec(index+1:end));
end

defaultName = spec(1:3:end);
defaultValue = spec(2:3:end);
validator = spec(3:3:end);

% Return list of unused options.
varargout{1} = {};

if ~isempty(varargin)
   
   if iscellstr(varargin(1:2:end))
      % Called passopt(spec,'name',value). This is the preferred way.
      userName = varargin(1:2:end);
      userValue = varargin(2:2:end);
   elseif nargin == 2 && isstruct(varargin{1})
      % Called passvalopt(spec,struct).
      userName = fieldnames(varargin{1});
      userValue = struct2cell(varargin{1})';
   elseif nargin == 2 && iscell(varargin{1});
      % Called passvalopt(spec,{'name',value}).
      userName = varargin{1}(1:2:end);
      userValue = varargin{1}(2:2:end);
   else
      error('iris:options', ...
         '\n*** Incorrect list of user options.');
   end
   
   if length(userName) > length(userValue)
      error('iris:options', ...
         '\n*** Option ''%s'' has no value assigned.',varargin{end});
   end

   changed = false([1,length(defaultName)]);
   for i = 1 : length(userName)
      index = strcmpi(userName{i},defaultName);
      if ~any(index)
         % Try to match the name excluding the last character. This is to
         % allow for both singulars and plurals in options like
         % 'deviation(s)', 'constraint(s)', etc.
         index = strncmpi(userName{i}(1:end-1), ...
            defaultName,length(userName{i})-1);
      end
      if any(index)
         defaultValue{index} = userValue{i};
         changed(index) = true;
      else
         varargout{1}{end+1} = userName{i};
         varargout{1}{end+1} = userValue{i};
      end
   end
   
   if nargout == 1 && ~isempty(varargout{1})
      %{
      warning('iris:options', ...
         '\n*** Invalid or obsolete option ''%s''. Option not used.', ...
         varargout{1}{1:2:end});
      %}
      error('iris:options', ...
         '\n*** Invalid or obsolete option ''%s''. Option not used.', ...
         varargout{1}{1:2:end});
   end
   
   % Validate the user-supplied options.
   invalid = {};
   for i = find(changed)
      if ~isempty(validator{i})
         flag = validator{i}(defaultValue{i});
         if ~flag
            invalid{end+1} = defaultName{i};
            invalid{end+1} = func2str(validator{i});
         end
      end
   end
   
   if ~isempty(invalid)
      error('iris:options', ...
         '\n*** Value assigned to optional argument ''%s'' does not pass validation ''%s''.\n', ...
         invalid{:});
   end
   
end

options = cell2struct(defaultValue,defaultName,2);

end
% End of primary function.





%**************************************************************************
% Subfunction default_data().
function default = default_data()

default = struct();

config = irisget();
dateformat = { ...
   'dateformat','config',config.validate.dateformat, ...
   'freqletters','config',config.validate.freqletters, ...
   'months','config',config.validate.months, ...
   'standinmonth','config',config.validate.standinmonth, ...   
};

default.dbbatch = {
  'append',true,@islogical, ...
  'classfilter',Inf,@(x) (isnumeric(x) && isinf(x)) || ischar(x), ...
  'namefilter',Inf,@(x) (isnumeric(x) && isinf(x)) || ischar(x), ...
  'namelist',{},@(x) ischar(x) || iscellstr(x), ...
  'fresh',false,@(x) islogical(x) || isstruct(x), ...
};

default.dbload = { ...
   'case','',@(x) isempty(x) || any(strcmpi(x,{'lower','upper'})), ...
   'changecase','',@(x) isempty(x) || any(strcmpi(x,{'lower','upper'})), ...
   'commentrow',{'comment','comments'},@(x) ischar(x) || iscellstr(x), ...
   'delimiter',',',@(x) ischar(x) && length(sprintf(x)) == 1, ...
   'freq',[],@(x) isempty(x) || (ischar(x) && strcmpi(x,'daily')) || (length(x) == 1 && isnan(x)) || (isnumeric(x) && length(x) == 1 && any(x == [0,1,2,4,6,12])), ...
   'nan',{'na','#n/a','n.a.'},@(x) ischar(x) || iscellstr(x), ...
   'skiprows','',@(x) isempty(x) || ischar(x) || iscellstr(x), ...
   'leadingrow','',@ischar, ...
   dateformat{:}, ...
};

default.dbnames = { ...
   'classfilter',Inf,@(x) isequal(x,Inf) || ischar(x), ...
   'namefilter',Inf,@(x) isequal(x,Inf) || ischar(x), ...
};

default.dbplot = { ...
   'highlight',[],@isnumeric, ...
   'round',Inf,@(x) isnumericscalar(x) && x > 0 && x == round(x), ...
   'plotfunc',@plot,@isfunc, ...
   'style',[],@(x) isempty(x) || isstruct(x), ...
   'subplot','auto',@(x) strcmpi(x,'auto') || (isnumeric(x) && length(x) == 2), ...
   'title','',@(x) isempty(x) || iscellstr(x) || ischar(x) || isfunc(x), ...
   'tight',false,@islogical, ...
   'transform',[],@(x) isempty(x) || isa(x,'function_handle'), ...
   'xlabel','',@(x) ischar(x) || iscellstr(x), ...
   'ylabel','',@(x) ischar(x) || iscellstr(x), ...
   'zlabel','',@(x) ischar(x) || iscellstr(x), ...
   'zeroline',false,@islogicalscalar, ...
};

default.dbsave = {
   'class',true,@islogicalscalar, ...
   'comment',true,@islogicalscalar, ...
   'decimal',[],@(x) isempty(x) || (length(x) == 1 && isnumeric(x)), ...
   'format','%.8e',@ischar, ...
   'nan','NaN',@ischar, ...
   dateformat{:}, ...
};

end
% End of subfunction default_data().





%**************************************************************************
% Subfunction default_dates().
function default = default_dates()

default = struct();

config = irisget();

default.convert = {
   'standinmonth','config',config.validate.standinmonth, ...
};

default.dat2str = {
   'dateformat','config',config.validate.dateformat, ...
   'freqletters','config',config.validate.freqletters, ...
   'months','config',config.validate.months, ...
   'standinmonth','config',config.validate.standinmonth, ...
};

default.str2dat = {
   'dateformat','config',config.validate.dateformat, ...
   'freq',[],@(x) isempty(x) ...
      || (isnumericscalar(x) && any(x == [0,1,2,4,6,12])), ... 
   'freqletters','config',config.validate.freqletters, ...
   'months','config',config.validate.months, ...
};

end
% End of subfunction default_dates().





%**************************************************************************
% Subfunction default_FAVAR().
function default = default_FAVAR()

default = struct();

default.estimate = { ...
   'cross',true,@islogicalscalar, ...
   'method','auto',@(x) isequal(x,'auto') || isequal(x,1) || isequal(x,2), ...
   'order',1,@(x) isnumericscalar(x), ...
   'output','auto',@(x) ischar(x) && any(strcmpi(x,{'auto','dbase','tseries','array'})), ...
   'rank',Inf,@(x) isnumericscalar(x), ...
   'tolerance','auto',@(x) strcmpi(x,'auto') || isnumericscalar(x), ...
   'ynames',@(n) ['y',sprintf('%g',n)],@(x) iscellstr(x) || isfunc(x), ...
};

default.filter = { ...
   'cross',true,@islogicalscalar, ...
   'meanonly',false,@islogicalscalar, ...
   'output','auto',@(x) ischar(x) && any(strcmpi(x,{'auto','dbase','tseries','array'})), ...
   'tolerance','auto',@(x) strcmpi(x,'auto') || isnumericscalar(x), ...
};

default.forecast = { ...
   'cross',true,@islogicalscalar, ...
   'meanonly',false,@islogicalscalar, ...
   'output','auto',@(x) ischar(x) && any(strcmpi(x,{'auto','dbase','tseries','array'})), ...
   'tolerance','auto',@(x) strcmpi(x,'auto') || isnumericscalar(x), ...
};

end
% End of subfunction default_FAVAR().





%**************************************************************************
% Subfunction default_IRIS().
function default = default_IRIS()

default = struct();

default.irisremove = { ...
   'display',true,@islogical, ...
   'removeroot',false,@islogical, ...
};

end
% End of subfunction default_IRIS().





%**************************************************************************
% Subfunction default_irisparser().
function default = default_irisparser()

default = struct();

default.clone = { ...
   'assign',[],@(x) isempty(x) || isstruct(x), ...
};

default.preparse = { ...
   'assign',[],@(x) isempty(x) || isstruct(x), ...
};

end
% End of subfunction default_irisparser().





%**************************************************************************
% Subfunction default_latex().
function default = default_latex()

default = struct();
config = irisget();

default.publish = {...
   'author','',@ischar, ...
   'abstract','',@ischar, ...
   'date','\today',@ischar, ...
   'deletelatex',true,@islogicalscalar, ...
   'display',true,@islogicalscalar, ...
   'evalcode',true,@islogicalscalar, ...
   'filecontents',true,@islogicalscalar, ...
   'linenumbers',true,@islogicalscalar, ...
   'maketitle',false,@islogicalscalar, ...
   'orientation','portrait',@(x) any(strcmpi(x,{'portrait','landscape'})), ...
   'papersize','letterpaper',@(x) any(strcmpi(x,{'a4paper','letterpaper'})), ...
   'preamble','config',@ischar, ...
   'rerun','auto',@(x) strcmpi(x,'auto') || isnumeric(x), ...
   'showcode',true,@islogicalscalar, ...
   'title','',@ischar, ...
   'toc',false,@islogicalscalar, ...
   'usenewfigure',false,@islogicalscalar, ...
   'numbered',false,@islogicalscalar, ...
   'scale',0.75,@isnumericscalar, ...
};

end
% End of subfunction default_latex().





%**************************************************************************
% Subfunction default_model().
function default = default_model()

default = struct();

default.acf = {
   'applyto',Inf,@(x) (isnumeric(x) && all(isinf(x))) || iscellstr(x) || ischar(x), ...
   'contributions',false,@islogicalscalar, ...
   'filter','',@ischar, ...
   'nfreq',256,@isnumericscalar, ...
   'order',0,@isnumericscalar, ...
   'select',Inf,@(x) (isnumeric(x) && all(isinf(x))) || iscellstr(x) || ischar(x), ...
};

default.bn = { ...
   'contributions',false,@islogicalscalar, ...
   'deviation',false,@islogicalscalar, ...
   'dtrends','auto',@(x) islogical(x) || strcmpi(x,'auto'), ...
   'output','dbase',@(x) any(strcmp(x,{'dbase','dpack'})), ...
};

default.chksstate = { ...
   'error',false,@islogicalscalar, ...
   'refresh',true,@islogical, ...
   'tolerance',getrealsmall(),@isnumericscalar, ...
   'warning',true,@islogicalscalar, ...
};

default.diffloglik = {...
   'display',false,@islogical,...
   'epspower',1/2,@isnumeric,...   
   'refresh',true,@islogical,...
   'solve',true,@islogical,...
   'sstate',false,@(x) islogical(x) || isempty(x) || isa(x,'function_handle'),...
};

default.estimate = { ...
   'domain','time',@(x) any(strncmpi(x,{'t','f'},1)), ...
   'epspower',1/2,@isnumeric, ...
   'logprior',false,@islogical, ...
   'maxiter',500,@(x) isnumericscalar(x) && x >= 0, ...
   'maxfunevals',2000,@(x) isnumericscalar(x) && x > 0, ...
   'nosolution','error',@(x) any(strcmpi(x,{'error','penalty'})), ...
   'output','auto',@(x) any(strcmpi(x,{'auto','dbase','dpack'})), ...
   'penalty',0,@(x) isnumericscalar(x) && x >= 0, ...
   'refresh',true,@islogical, ...
   'solve',true,@islogical, ...
   'sstate',false,@(x) islogical(x) || isempty(x) || isa(x,'function_handle'), ...
   'tolfun',1e-6,@(x) isnumeric(x) && x > 0, ...
   'tolx',1e-6,@(x) isnumeric(x) && x > 0, ...
   'typicalx','start',@(x) (ischar(x) && strcmpi(x,'start')) || isnumeric(x), ...
   'zero',false,@islogical, ...
};

default.filter = { ...
   'data','smooth',@(x) ischar(x) && any(strcmpi(x,{'smooth','predict'})), ...
   'output','auto',@(x) any(strcmpi(x,{'auto','dbase','dpack'})), ...
};

default.fisher = { ...
   'chksgf',false,@islogicalscalar, ...
   'display',false,@islogicalscalar, ...
   'epspower',1/3,@isnumericscalar, ...
   'progress',false,@islogicalscalar, ...
   'refresh',true,@islogicalscalar, ...
   'solve',true,@islogicalscalar, ...
   'sstate',[],@(x) isempty(x) || isa(x,'function_handle'), ...
   'deviation',true,@islogicalscalar, ...
   'sstateeffect',[],@(x) isempty(x) || islogical(x), ...
   'tolerance',eps()^(2/3),@isnumericscalar, ...
};

default.forecast = {...
   'anticipate',true,@islogical, ...
   'deviation',false,@islogical, ...
   'dtrends','auto',@(x) islogical(x) || strcmpi(x,'auto'), ...
   'initcond','data',@(x) any(strcmpi(x,{'data','fixed'})) || isnumeric(x), ...
   'meanonly',false,@islogicalscalar, ...
   'output','auto',@(x) any(strcmpi(x,{'auto','dpack','dbase'})), ...
   'std',[],@(x) isempty(x) || isstruct(x), ...
   'tolmse',getrealsmall('mse'),@(x) isnumeric(x) && length(x) == 1, ...
};

default.loglik = { ...
   'domain','time',@(x) any(strncmpi(x,{'t','f'},1)), ...
   'output','auto',@(x) any(strcmpi(x,{'auto','dpack','dbase'})), ...
   'persist',false,@islogical, ...
};

default.loglikf = { ...
   'band',[2,Inf],@(x) isnumeric(x) && length(x) == 2, ...
   'exclude',[],@(x) isempty(x) || ischar(x) || iscellstr(x) || islogical(x), ...
   'relative',true,@islogical, ...
   'zero',false,@islogical, ...
};

default.kalman = { ...
   'ahead',1,@(x) isnumericscalar(x) && x > 0 && x == round(x), ...
   'chkexact',false,@islogical, ...
   'chkfmse',false,@islogical, ...
   'deviation',false,@islogical, ...
   'dtrends','auto',@(x) islogical(x) || strcmpi(x,'auto'), ...
   'exclude',[],@(x) isempty(x) || ischar(x) || iscellstr(x) || islogical(x), ...
   'initcond','stochastic', ...
   @(x) (isstruct(x) && isfield(x,'mean_') && isfield(x,'mse_') && iscell(x.mean_) && iscell(x.mse_)) ...
   || any(strcmpi(x,{'stochastic','fixed','optimal'})), ...
   'initialise',[],@(x) isempty(x) || iscell(x) || isstruct(x), ...
   'lastsmooth',[],@(x) isempty(x) || isnumericscalar(x), ...
   'outoflik',{},@(x) ischar(x) || iscellstr(x), ...
   'objective','mloglik',@(x) any(strcmpi(x,{'mloglik','loglik','lik','prederr'})), ...
   'objectivesample',Inf,@isnumeric, ...
   'pedindonly',false,@islogicalscalar, ...
   'relative',true,@islogicalscalar, ...
   'meanonly',false,@islogicalscalar, ...
   'std',[],@(x) isempty(x) || isstruct(x), ...
   'symmetric',true,@islogicalscalar, ...
   'tolerance',eps()^(2/3),@isnumeric, ...
   'tolmse','auto',@(x) (ischar(x) && strcmpi(x,'auto')) || isnumericscalar(x), ...
   'weighting',[],@isnumeric, ...
};

default.model = {
   'addlead',false,@islogicalscalar, ...
   'assign',struct(),@isstruct, ...
   'epsilon',eps^(1/3),@isnumericscalar, ...
   'linear',[],@(x) isempty(x) || islogical(x), ...
   'precision','double',@(x) any(strcmp(x,{'double','single'})), ...
   'saveas','',@ischar, ...
   'std',NaN,@isnumericscalar, ...
   'symbolic',false,@islogical, ...
   'simplify',Inf,@isnumericscalar, ...
   'torigin',2000,@isnumericscalar, ...
};

default.neighbourhood = { ...
   'plot',false,@islogicalscalar, ...
   'subplot','auto',@(x) isnumeric(x) || (ischar(x) && strcmpi(x,'auto')), ...
};

default.tcorule = { ...
   'beta',1,@isnumericscalar, ...
   'display',5000,@isnumericscalar, ...
   'ginverse',false,@islogicalscalar, ...
   'initexp',@eye,@(x) isnumeric(x) || isfunc(x), ...
   'maxiter',50000,@isnumericscalar, ...
   'reset',false,@islogicalscalar, ...
   'tolexp',1e-10,@isnumericscalar, ...
   'tolrule',1e-10,@isnumericscalar, ...
   'tolvalue',1e-10,@isnumericscalar, ...
   'warning',true,@islogical, ...
};

default.resample = {...
   'deviation',false,@islogicalscalar, ...
   'method','montecarlo',@(x) any(strcmpi(x,{'montecarlo','bootstrap'})), ...
   'output','auto',@(x) any(strcmpi(x,{'auto','dpack','dbase'})), ...
   'progress',false,@islogicalscalar, ...
   'randomise',true,@islogicalscalar, ...
   'wild',false,@islogicalscalar, ...
};

default.simulate = { ...
   'anticipate',true,@islogical, ...
   'contributions',false,@(x) islogicalscalar(x), ...
   'deviation',false,@islogical, ...
   'dtrends','auto',@(x) islogical(x) || strcmpi(x,'auto'), ...
   'output','auto',@(x) any(strcmpi(x,{'auto','dbase','dpack'})), ...
   'ignoreresiduals',false,@islogicalscalar, ...
   'plan',[],@(x) isplan(x) || isempty(x), ...
   'missing',NaN,@isnumeric, ...
   'tolerance',1e-5,@isnumericscalar, ...
   'maxiter',100,@isnumericscalar, ...
   'display',true,@islogicalscalar, ...
   'lambda',1,@(x) isnumeric(x) && all(x >= 0) && all(x <= 1), ...
};

default.solve = { ...
   'alt',inf,@(x) isnumeric(x) || islogical(x), ...
   'double',true,@islogical, ...
   'expand',0,@(x) isnumeric(x) && length(x) == 1, ...
   'logbook',false,@islogical, ...
   'forward',0,@(x) isnumeric(x) && length(x) == 1, ...
   'refresh',true,@islogical, ...
   'select',true,@islogical, ...
   'system',[],@(x) isempty(x) || isstruct(x), ...
   'tolerance',getrealsmall(),@(x) isnumeric(x) && length(x) == 1, ...
   'warning',true,@islogical, ...
};

default.sourcedb = { ...
   'ndraw',1,@(x) isnumericscalar(x) && x >= 0, ...
   'deviation',false,@islogical, ...
   'dtrends','auto',@(x) islogical(x) || (ischar(x) && strcmpi(x,'auto')), ...
   'residuals',[],@(x) isempty(x) || isa(x,'function_handle'), ...
};   

default.srf = {...
   'log',true,@islogicalscalar,...
   'select',Inf,@(x) (isnumeric(x) && length(x) == 1 && isinf(x)) || iscellstr(x) || ischar(x),...
   'size','std',@(x) (ischar(x) && strcmpi(x,'std')) || isnumeric(x),...
};

default.sspace = { ...
   'triangular',true,@islogicalscalar, ...
   'removeinactive',false,@islogicalscalar, ...
};

default.sstate = {
   'algorithm','lsqnonlin',@(x) any(strcmp(x,{'lsqnonlin','fsolve'})), ...
   'blocks',false,@islogical, ...
   'display','iter',@(x) isempty(x) || islogical(x) || any(strcmp(x,{'iter','final','off','notify'})), ...
   'fix',{},@(x) isempty(x) || iscellstr(x) || ischar(x), ...
   'fixallbut',{},@(x) isempty(x) || iscellstr(x) || ischar(x), ...
   'maxiter',1000,@(x) isnumeric(x) && length(x) == 1 && round(abs(x)) == x, ...
   'maxfunevals',1000,@(x) isnumeric(x) && length(x) == 1 && round(abs(x)) == x, ...
   'optimset',struct,@(x) isempty(x) || isstruct(x), ...
   'randomise',[1,1],@(x) isnumeric(x) && length(x) == 2, ...
   'refresh',true,@islogical, ...
   'solve',false,@islogical, ...
   'tolx',1e-12,@(x) isnumeric(x) && length(x) == 1 && x > 0, ...
   'tolfun',1e-12,@(x) isnumeric(x) && length(x) == 1 && x > 0, ...
   'warning',true,@islogicalscalar, ...
};

default.symbdiff = { ...
   'simplify',Inf,@isnumericscalar, ...
};

end
% End of subfunction default_model().





%**************************************************************************
% Subfunction default_quickreport().
function default = default_quickreport()

default = struct();

default.qpdefault = {...
   'grid',true,@islogicalscalar, ...
   'highlight',[],@isnumeric, ...
   'tight',false,@islogicalscalar, ...
   'zeroline',false,@islogicalscalar, ...
};

default.qstyle = { ...
   'cascade',true,@islogicalscalar, ...
   'offset',0,@isnumericscalar, ...
   'warning',true,@islogicalscalar, ...
};

end
% End of subfunction default_quickreport().





%**************************************************************************
% Subfunction default_tseries().
function default = default_tseries()

default = struct();

config = irisget();

default.arf = { ...
   'time','forward',@(x) strcmpi(x,{'backward','forward'}), ...
};

default.chowlin = { ...
   'constant',true,@islogicalscalar, ...
   'log',false,@islogicalscalar, ...
   'ngrid',200,@(x) isnumericscalar(x) && x >= 1, ...
   'rho','auto',@(x) any(strcmpi(x,{'auto','positive'})) || (isnumericscalar(x) && x > -1 && x < 1), ...
   'timetrend',false,@islogicalscalar, ...
};

convert = { ...
   'function',[],@(x) isempty(x) || isa(x,'function_handle') || ischar(x), ...
   'missing',NaN,@(x) any(strcmpi(x,{'last'})) || (isnumeric(x) && length(x) == 1), ...
};

default.convertdaily = [convert,{ ...
   'ignorenan',true,@islogical, ...
   'method',@mean,@(x) isa(x,'function_handle') || isequal(x,'first') || isequal(x,'last'), ...
   'select',Inf,@(x) isnumeric(x), ...
   }];

default.convertaggregate = [convert,{ ...
   'ignorenan',false,@islogical, ...
   'method',@mean,@(x) isa(x,'function_handle') || isequal(x,'first') || isequal(x,'last'), ...
   'select',Inf,@(x) isnumeric(x), ...
   }];

default.convertinterp = [convert,{ ...
   'ignorenan',false,@islogical, ...
   'method','cubic',@(x) ischar(x), ...
   'position','centre',@(x) ischar(x) && any(strncmpi(x,{'c','s','e'},1)), ...
   }];

default.trend = { ...
   'break',[],@isnumeric, ...
   'log',false,@islogicalscalar, ...
   'method','regress',@(x) ischar(x) && any(strcmpi(x,{'regress','diff'})), ...
};

default.fft = {
   'full',false,@islogicalscalar, ...
};

default.filter = {
   'drift',0,@(x) isnumeric(x) && length(x) == 1, ...
   'gap',[],@(x) isempty(x) || istseries(x), ...
   'growth',[],@(x) isempty(x) || istseries(x), ...
   'lambda',[],@(x) isnumeric(x) && length(x) <= 1, ...
   'level',[],@(x) isempty(x) || istseries(x), ...
   'log',false,@islogical, ...
   'swap',false,@islogical, ...
   'forecast',[],@(x) isnumeric(x) && length(x) <= 1, ...
};

default.spy = { ...
   'dateformat','config',config.validate.dateformat, ...
   'freqletters','config',config.validate.freqletters, ...
   'months','config',config.validate.months, ...
   'standinmonth','config',config.validate.standinmonth, ...   
   'datetick',Inf,@isnumeric, ...
   'names',{},@iscellstr, ...
};

default.x12 = { ...
   'arima',0, @(x) islogical(x) || isnumeric(x), ...
   'delete',true,@islogicalscalar, ...
   'display',false,@islogicalscalar, ...
   'log',false,@islogicalscalar, ...
   'mode','auto',@(x) (isnumeric(x) && any(x == -1 : 3)) || any(strcmp(x,{'add','a','mult','m','auto','sign','pseudoadd','p','logadd','l'})), ...
   'output','d11',@(x) ischar(x) || iscellstr(x), ...
   'tdays',false,@islogicalscalar, ...
};

end
% End of subfunction default_tseries().





%**************************************************************************
% Subfunction default_utils().
function default = default_utils()

default = struct();

default.highlight = { ...
   'grade',[],@(x) isnumeric(x) && length(x) <= 1, ...
   'excludefromlegend',true,@islogicalscalar, ...
   'colour',0.8*[1,1,1],@(x) (isnumeric(x) && length(x) == 3) || ischar(x), ...
   'color',[],@(x) isempty(x) || (isnumeric(x) && length(x) == 3) || ischar(x), ...
   'bar',false,@islogical, ...
   'around',0,@isnumericscalar, ...
};

default.ploteig = { ...
   'ucircle',true,@islogical, ...
};

end
% End of subfunction default_utils().





%**************************************************************************
% Subfunction default_sstate().
function default = default_sstate()

default = struct();

default.sstate = { ...
   'assign',struct(),@(x) isempty(x) || isstruct(x), ...
   'growthnames','d%s',@(x) isfunc(x) || ischar(x), ...
};

default.compile = { ...
   'excludezero',false,@islogicalscalar, ...
   'deletesymbolicmfiles',true,@islogicalscalar, ...
   'end',Inf,@(x) isnumericscalar(x) || ischar(x), ...
   'simplify',Inf,@isnumericscalar, ...
   'start',1,@(x) isnumericscalar(x) || ischar(x), ...
   'symbolic',true,@islogicalscalar, ...
};

end
% End of subfunction default_sstate().





%**************************************************************************
% Subfunction default_VAR().
function default = default_VAR()

default = struct();
outputformat = {'output','auto',@(x) any(strcmpi(x,{'auto','dbase','tseries','array'}))};

default.acf = { ...
   'applyto',Inf,@isnumericscalar, ...
   'filter','',@ischar, ...
   'nfreq',256,@isnumericscalar, ...
   'order',0,@isnumericscalar, ...
};

default.demean = [ ...
   outputformat, ...
   ];

default.estimate = [ ...
   outputformat, { ...
   'a',[],@isnumeric, ...
   'bvar',[],@(x) isempty(x) || isnumeric(x), ...
   'c',[],@isnumeric, ...
   'g',[],@isnumeric, ...
   'order',1,@(x) isnumeric(x) && numel(1) == 1, ...
   'cointeg',[],@isnumeric, ...
   'constraints','',@(x) ischar(x) || isnumeric(x), ...
   'const',[],@(x) isempty(x) || islogicalscalar(x), ...
   'constant',true,@islogicalscalar, ...
   'covparameters',false,@islogicalscalar, ...
   'eqtnbyeqtn',false,@islogicalscalar, ...
   'maxiter',1,@isnumericscalar, ...
   'mean',[],@(x) isempty(x) || isnumeric(x), ...
   'progress',false,@islogicalscalar, ...
   'schur',true,@islogicalscalar, ...
   'tolerance',1e-5,@isnumericscalar, ...
   'ynames',@(n) sprintf('y%g',n),@(x) iscellstr(x) || isfunc(x), ...
   'enames',@(yname,n) ['res_',yname],@(x) iscellstr(x) || isfunc(x), ...
   'userdata',[],true, ...
   'warning',true,@islogicalscalar, ...
   }];

default.forecast = [ ...
   outputformat, { ...
   'deviation',false,@islogicalscalar, ...
   'meanonly',false,@islogicalscalar, ...
   'returnmse',[],@(x) isempty(x) || islogicalscalar(x), ...
   }];

default.integrate = [ ...
   outputformat, { ...
   'applyto',Inf,@(x) isnumeric(x) || islogical(x), ...
   }];

default.resample = [ ...
   outputformat, {
   'method','bootstrap',@(x) (ischar(x) && any(strcmpi(x,{'bootstrap','montecarlo'}))) || isfunc(x), ...
   'progress',false,@islogicalscalar, ...
   'randomise',false,@islogicalscalar, ...
   'wild',false,@islogicalscalar, ...
   }];

default.simulate = [ ...
   outputformat, { ...
   'contributions',false,@islogicalscalar, ...
   'deviation',false,@islogicalscalar, ...
   }];

default.sprintf = { ...
   'constant',true,@islogicalscalar, ...
   'decimal',[], @(x) isempty(x) || isnumericscalar(x), ...
   'declare',false,@islogicalscalar, ...
   'ename',[],@(x) isempty(x) || iscellstr(x) || isfunc(x), ...
   'format','%+.16e',@ischar, ...
   'hardparameter',true,@islogicalscalar, ...
   'tolerance',getrealsmall(),@isnumericscalar, ...
   'ynames',[],@(x) isempty(x) || iscellstr(x), ...
};

default.SVAR = [ ...
   outputformat, { ...
   'method','chol',@(x) any(strcmpi(x,{'chol','qr','svd'})), ...
   'std',1,@isnumeric, ...
   'ordering',[],@isnumeric, ...
   'reorderresiduals',true,@islogical, ...
   'rank',Inf,@isnumericscalar, ...
   }];

default.VAR = { ...
   'userdata',[],true, ...
};

end
% End of subfunction default_VAR().
