function [hfig,hax,hline,htit,plotdb] = qplot(cdfname,data,range,varargin)
%
% QPLOT  Quick plot function based on Graph Definition File.
%
% Syntax:
%    [hfig,hax,hline,htit,plotdb] = qplot(cdfname,d,range,...)
% Output arguments:
%    hfig [ numeric ] Handles to figures created by QPLOT.
%    hax [ cell ] Handles to axes created by QPLOT.
%    hline [ cell ] Handles to lines created by QPLOT.
%    htit [ numeric ] Handles to titles created by QPLOT.
%    plotdb [ struct ] Database with actually plotted series.
% Required input arguments:
%    cdfname [ char ] Contents definition file name.
%    d [ struct ] Database with input data.
%    range [ numeric ] Time range, i.e. <a href="dates.html">IRIS serial date numbers</a>.
% <a href="options.html">Optional input arguments:</a>
%    'caption' [ char | <a href="default.html">empty</a> ] Figure caption.
%    'plot' [ function_handle | <a href="default.html">@qpdefault</a> ] Plot function handle.
%    'saveas' [ char | <a href="default.html">empty</a> ] Save figures to file.

% The IRIS Toolbox.
% Copyright 2007-2010 Jaromir Benes.

% Validate required input arguments.
p = inputParser();
p.addRequired('filename',@(x) ischar(x) || isa(x,'function_handle'));
p.addRequired('dbase',@(x) isstruct(x));
p.addRequired('range',@isnumeric);
p.parse(cdfname,data,range);

default = {...
   'drawnow',false,@islogical,...
   'mark',{},@(x) iscell(x) || ischar(x),...
   'prefix','P%g_',@ischar,...
   'round',Inf,@(x) isnumericscalar(x) && x >= 0 && round(x) == x,...
   'style',struct(),@isstruct,...
};
[options,varargin] = extractopt(default(1:3:end),varargin{:});
options = passvalopt(default,options{:});

if nargin < 3
   range = Inf;
end

if ischar(options.mark)
   options.mark = {options.mark};
end

options.mark = strtrim(options.mark);

%********************************************************************************
% Function body.

% Read contents definition file.
gd = readcdf(cdfname);

hfig = [];
hax = {};
hline = {};
htit = [];
if isempty(gd)
   return
end

% Count number of panels on each page.
page = 0;
count = 1;
npanel = [];
while count <= length(gd)
   switch gd(count).tag
   case '!++'
      page = page + 1;
      npanel(page) = 0;
   case {'!--','!::','!**'}
      npanel(page) = npanel(page) + 1;
   end
   count = count + 1;
end

sub = 'auto';
pos = 1;
count = 1;
page = 0;
plotdb = struct();
for i = 1 : length(gd)
   switch gd(i).tag
   case '#'
      % Change subplot division.
      sub = strtrim(gd(i).title);
   case '!++'
      % New figure.
      hfig(end+1) = figure('selectionType','open');
      orient('landscape');
      hax{end+1} = [];
      hline{end+1} = {};
      if ~isempty(gd(i).title)
         ftitle(hfig(end),gd(i).title);
      end
      pos = 1;
      page = page + 1;
      [nrow,ncol] = getsubplot_(sub,npanel(page));    
   case {'!**'}
      % Skip current subplot position.
      pos = pos + 1;
   case {'!--','!::'}
      % New panel.
      % Draw a line (!--) or bar (!::) plot.
      hax{end}(end+1) = subplot(nrow,ncol,pos);      
      [tmpformula,tmplegend] = charlist2cellstr(gd(i).body,'&\n');
      nformula = length(tmpformula);
      x = cell([1,nformula]);
      [x{:}] = dbeval(data,tmpformula{:});
      switch gd(i).tag
      case '!--'
         type = 'line';
      case '!::'
         type = 'bar';
      end
      if ~isinf(options.round) && ~isnan(options.round)
         for ix = 1 : length(x)
            if istseries(x{ix})
               x{ix} = round(x{ix},options.round);
            elseif isnumeric(x{ix})
               factor = 10^options.round;
               x{ix} = round(x{ix}*factor)/factor;
            end
         end
      end
      try
         [hline{end}{end+1},tmprange,tmpdata] = ...
           qpdefault(hax{end}(end),range,x,type,@legend_,varargin{:});
      catch me
         warning('iris:qplot',...
            'Error when plotting %s.\nMatlab says: %s',...
            gd(i).body,me.message);
      end
      tmptitle = '';
      if ~isempty(gd(i).title)
         % Add title to current subplot.
         tokens = regexp(gd(i).title,'^(.*?)(\\\\.*)?$','tokens','once');
         tmptitle = strtrim(tokens{1});
         tokens{2} = strtrim(tokens{2}(3:end));         
         if ~isempty(tokens{2})
            tmptitle = [tmptitle,char(10),tokens{2}];
         end
         htit(end+1) = title(hax{end}(end),tmptitle);
      end
      % Create a name for the entry in the output database based on the
      % (user-supplied) prefix and the current panel's name. Substitute '_'
      % for any [^\w]. If not a valid Matlab name, replace with "Panel#".
      prefix = sprintf(options.prefix,count);
      if nargout > 4
         tmpname = [prefix,regexprep(tmptitle,'[^\w]+','_')];      
         if ~isvarname(tmpname)
            tmpname = sprintf('Panel%g',count);
         end
         plotdb.(tmpname) = tseries(tmprange,tmpdata,options.mark);
      end
      pos = pos + 1;
      count = count + 1;
   end
end

if ~isempty(options.style)
   qstyle(options.style,hfig);
end

if options.drawnow
   drawnow();
end

% End of function body.

%**************************************************************************
% Nested function legend_().

function outputlegend = legend_()
   % Splice legend and marks.
   outputlegend = {};
   for j = 1 : length(x)
      for k = 1 : size(x{j},2)
         outputlegend{end+1} = '';
         if length(tmplegend) >= j
            outputlegend{end} = [outputlegend{end},tmplegend{j}];
         end
         if length(options.mark) >= k
            outputlegend{end} = [outputlegend{end},options.mark{k}];
         end
      end
   end
end
% End of nested function legend_().

end
% End of primary function.

%**************************************************************************
% Subfunction getsubplot_().
function [nrow,ncol] = getsubplot_(sub,npanel)
if ~strcmpi(sub,'auto')
   tmp = sscanf(sub,'%gx%g');
   if isnumeric(tmp) && length(tmp) == 2 && ~any(isnan(tmp))
      nrow = tmp(1);
      ncol = tmp(2);
   else
      sub = 'auto';
   end
end

if strcmpi(sub,'auto')
   x = ceil(sqrt(npanel));
   if x*(x-1) >= npanel
      nrow = x;
      ncol = x-1;
   else
      nrow = x;
      ncol = x;
   end
end
end
% End of subfunction getsubplot_().


