function qstyle(gs,h,varargin)
% QSTYLE  Apply cascading styles to graphic objects.

options = passvalopt('quickreport.qstyle',varargin{:});

% Swap style struct and graphic handle if needed.
if isnumeric(gs) && ishandle(gs)
   [gs,h] = deal(h,gs);
end

%**************************************************************************
% Function body.

if ischar(gs)
   % Called with a file name.
   % Remove extension.
   [fpath,ftitle] = fileparts(gs);
   gs = rungsf_(fullfile(fpath,ftitle),@run);
elseif iscellstr(gs) && length(gs) == 1
   % Called directly with commands.
   gs{1} = strrep(gs{1},'"','''');
   gs = rungsf_(gs{1},@eval);
end

for i = h(:)'
   if ~ishandle(i)
      continue
   end
   switch get(i,'type')
      case 'figure'
         figure_(i,gs,options);
      case 'axes'
         axes_(i,gs,options);
      case 'line'
         line_(i,gs,options);
      case 'patch'
         patch_(i,gs,options);
   end
end

end

% End of primary function.

%**************************************************************************
% Subfunction rungsf_().
% Run graphic style file and create graphic style database.
function d = rungsf_(gsf,func)
   axes = [];
   figure = [];
   label = [];
   line = [];
   title = [];
   func(gsf);
   d = struct();
   d.axes = axes;
   d.figure = figure;
   d.label = label;
   d.line = line;
   d.title = title;
end

%**************************************************************************
% Subfunction applyto_()
function applyto_(h,d,field,options)

   h = findobj(h,'flat','-not','userData','excludeFromStyle');
   if isempty(h)
      return
   end

   if ~isfield(d,field) || ~isstruct(d.(field))
      return
   end   
   
   nh = length(h);
   list = fieldnames(d.(field));
   for i = 1 : length(list)
      x = d.(field).(list{i});
      if ~iscell(x)
         x = {x};
      end
      nx = numel(x);
      for j = 1 : nh
         value = x{1+rem(j-1+options.offset,nx)};
         try
            set(h(j),list{i},value);
         catch Error
            flag = exceptions_(h(j),list{i},value);
            if ~flag && options.warning
               warning('iris:qreport',...
                  'Error setting %s property ''%s''.\nMatlab says: %s',...
                  field,list{i},Error.message);
            end
         end
      end
   end
end
% End of subfunction applyto_().

%**************************************************************************
% Subfunction figure_().
function figure_(h,d,options)
   if isempty(h)
      return
   end      
   h = h(:)';
   applyto_(h,d,'figure',options);
   if options.cascade
      for i = h
         obj = findobj(i,'type','axes');
         axes_(obj(:).',d,options);
      end
   end
end
% End of subfunction figure_().

%**************************************************************************
% Subfunction axes_().
function axes_(h,d,options)
   if isempty(h)
      return
   end
   applyto_(h,d,'axes',options);
   if options.cascade
      for i = h(:)'
         applyto_(get(i,'title'),d,'title',options);
         applyto_(get(i,'xlabel'),d,'xlabel',options);
         applyto_(get(i,'ylabel'),d,'ylabel',options);
         applyto_(get(i,'zlabel'),d,'zlabel',options);
         line_(findobj(i,'type','line'),d,options);
         bar_(findobj(i,'-property','barWidth'),d,options);
         patch_(findobj(i,'type','patch'),d,options);
      end  
   end
end
% End of subfunction axes_().

%**************************************************************************
% Subfunction line_().
function line_(h,d,options)
   if isempty(h)
      return
   end   
   h = h(:)';
   applyto_(h(end:-1:1),d,'line',options);      
end
% End of subfunction line_().

%**************************************************************************
% Subfunction bar_().
function bar_(h,d,options)
   if isempty(h)
      return
   end   
   h = h(:)';
   applyto_(h(end:-1:1),d,'bar',options);
end
% End of subfunction bar_().

%**************************************************************************
% Subfunction patch_().
function patch_(h,d,options)
   if isempty(h)
      return
   end   
   h = h(:)';
   applyto_(h(end:-1:1),d,'patch',options);
end
% End of subfunction patch_().

%**************************************************************************
% Subfunction exceptions_().
function flag = exceptions_(h,name,value)
flag = false;
switch get(h,'type')
   case 'axes'
      switch lower(name)
         case 'yaxislocation'
            if strcmpi(value,'either')
               axisoneitherside(h,'y');
            end
         case 'xaxislocation'
            if strcmpi(value,'either')
               axisoneitherside(h,'x');
            end            
      end
end
end
% End of subfunction exception_().
